
#include <X11/Xlib.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/List.h>
#include <Xm/Form.h>


#include "DispRec/DispRecXtAPI.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>


char* appname="testDispRecXtAPI";

/***************/
/* global variables */
/***************/

static DispRecXtAPI_Server apiServer = DispRecXtAPI_Server_None;
static DispRecXtAPI_RecordingSession guiRecordingSession = 
    DispRecXtAPI_RecordingSession_None;
static DispRecXtAPI_GUI guiHandle;
static Widget guiEventLogger = NULL;
static Widget guiManager = NULL;

/* callback handles for recordingSession callbacks */
DispRecXtAPI_RecordingStarted_CallbackHandle 
    gRecordingStarted_LogEvent_CallbackHandle = NULL;    
    
DispRecXtAPI_RecordingNotStarted_CallbackHandle 
    gRecordingNotStarted_LogEvent_CallbackHandle = NULL;	
    
DispRecXtAPI_RecordingFinished_CallbackHandle 
    gRecordingFinished_LogEvent_CallbackHandle = NULL;	
    
DispRecXtAPI_RecordingNotFinished_CallbackHandle 
    gRecordingNotFinished_LogEvent_CallbackHandle = NULL;	
    
DispRecXtAPI_RecordingSessionDeleted_CallbackHandle 
    gRecordingSessionDeleted_LogEvent_CallbackHandle = NULL;

/******************/
/* forward decls  */
/******************/

static void RegisterCallbacksForRecordingSession(
    DispRecXtAPI_RecordingSession session,
    VtError *err );

static void UnregisterCallbacksForCurrentRecordingSession(
    VtError *err );

/******************/
/* auxiliary functions */
/******************/

/* logs a message  to a XmList widget by adding a new item with a timestamp and the given string */
static void WriteToLogger( Widget logger, const char* message )
{
    DECLERR;
    struct timeval now;
    struct tm nowStruct;
    char* messageWithTimestamp = 
	NEWARRAY( char, strlen( message ) + 200 );
    gettimeofday(&now, NULL);
    localtime_r( &(now.tv_sec), &nowStruct );
    sprintf( 
	messageWithTimestamp, 
	"%02d:%02d:%02d.%03d: %s", 
	nowStruct.tm_hour, nowStruct.tm_min, nowStruct.tm_sec, (int)( now.tv_usec / 1000 ),
	message 
    );
    XmListAddItemUnselected( 
	logger, 
	XmStringCreateSimple( messageWithTimestamp ),
	/* position = */ 1 /* = insert at the start */
    );
    DELETEARRAY( messageWithTimestamp );
}


/**********************/
/* Motif callback functions */
/**********************/
static void ShowGUICallback(
    Widget w,
    XtPointer client_data,
    XtPointer call_data)
{
    DECLERR;
    XmPushButtonCallbackStruct* cbs = 
	(XmPushButtonCallbackStruct*)call_data;
	
    printf( "ShowGUI\n");
    
    DispRecXtAPI_ShowGUI( guiHandle, /* screen = */ 0, /* x= */ 100, /* y = */ 200, err );
}

static void HideGUICallback(
    Widget w,
    XtPointer client_data,
    XtPointer call_data)
{
    DECLERR;
    XmPushButtonCallbackStruct* cbs = 
	(XmPushButtonCallbackStruct*)call_data;
	
    printf( "HideGUI\n");
    
    DispRecXtAPI_HideGUI( guiHandle, err );
}

/*****************************/
/* DispRecXtAPI  callback functions */
/*****************************/


/* callback functions for adjusting the recording session to listen to */
void 
    UpdateSession_RecordingSessionDeleted(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingSessionDeleted_CallbackHandle handle,
	enum DispRecXtAPI_RecordingSessionDeletedReason reason,
	void *client_data
    )
{
    DECLERR;
    UnregisterCallbacksForCurrentRecordingSession( err );
    guiRecordingSession = NULL;
}

void 
    UpdateSession_GUIControlsRecordingSession(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIControlsRecordingSession_CallbackHandle handle,
	DispRecXtAPI_RecordingSession session,
	void *client_data
    )
{
    DECLERR;
    guiRecordingSession = session;
    RegisterCallbacksForRecordingSession( 
	guiRecordingSession, err
    );
}

void 
    UpdateSession_GUILeavesRecordingSession(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUILeavesRecordingSession_CallbackHandle handle,
	DispRecXtAPI_RecordingSession session,
	void *client_data
    )
{
    DECLERR;
    UnregisterCallbacksForCurrentRecordingSession( err );
    guiRecordingSession = NULL;
}

/* callback functions for logging */

void 
    LogEvent_RecordingStarted(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingStarted_CallbackHandle handle,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( buf, "RecordingStarted: session = %p", session );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_RecordingNotStarted(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingNotStarted_CallbackHandle handle,
	enum DispRecXtAPI_RecordingNotStartedReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"RecordingNotStarted: session = %p, reason = %s", 
	session, 
	DispRecXtAPI_RecordingNotStartedReason_ToString( reason ) 
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_RecordingFinished(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingFinished_CallbackHandle handle,
	enum DispRecXtAPI_RecordingFinishedReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"RecordingFinished: session = %p, reason = %s", 
	session, 
	DispRecXtAPI_RecordingFinishedReason_ToString( reason ) 
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_RecordingNotFinished(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingNotFinished_CallbackHandle handle,
	enum DispRecXtAPI_RecordingNotFinishedReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"RecordingNotFinished: session = %p, reason = %s", 
	session, 
	DispRecXtAPI_RecordingNotFinishedReason_ToString( reason ) 
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_RecordingSessionDeleted(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingSessionDeleted_CallbackHandle handle,
	enum DispRecXtAPI_RecordingSessionDeletedReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"RecordingSessionDeleted: session = %p, reason = %s", 
	session, 
	DispRecXtAPI_RecordingSessionDeletedReason_ToString( reason ) 
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_GUIStarted(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIStarted_CallbackHandle handle,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIStarted: gui = %p", 
	gui
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_GUINotStarted(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUINotStarted_CallbackHandle handle,
	enum DispRecXtAPI_GUINotStartedReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUINotStarted: gui = %p, reason=%s", 
	gui,
	DispRecXtAPI_GUINotStartedReason_ToString( reason ) 	
    );
    WriteToLogger( logger, buf );
}
    
void 
    LogEvent_GUIExited(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIExited_CallbackHandle handle,
	enum DispRecXtAPI_GUIExitedReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIExited: gui = %p, reason=%s", 
	gui,
	DispRecXtAPI_GUIExitedReason_ToString( reason ) 	
    );
    WriteToLogger( logger, buf );
}
     
void 
    LogEvent_GUIRestarting(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIRestarting_CallbackHandle handle,
	enum DispRecXtAPI_GUIRestartingReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIRestarting: gui = %p, reason=%s", 
	gui,
	DispRecXtAPI_GUIRestartingReason_ToString( reason ) 	
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_GUIRestarted(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIRestarted_CallbackHandle handle,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIRestarted: gui = %p", 
	gui
    );
    WriteToLogger( logger, buf );
}
    
void 
    LogEvent_GUINotRestarted(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUINotRestarted_CallbackHandle handle,
	enum DispRecXtAPI_GUINotRestartedReason reason,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUINotRestarted: gui = %p, reason=%s", 
	gui,
	DispRecXtAPI_GUINotRestartedReason_ToString( reason ) 	
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_GUIVisible(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIVisible_CallbackHandle handle,
	int screen, int x, int y,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIVisible: gui = %p, screen=%d, x=%d, y=%d", 
	gui,
	screen, x, y
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_GUIHidden(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIHidden_CallbackHandle handle,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIHidden: gui = %p", 
	gui
    );
    WriteToLogger( logger, buf );
}
    
void 
    LogEvent_GUIControlsRecordingSession(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIControlsRecordingSession_CallbackHandle handle,
	DispRecXtAPI_RecordingSession session,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIControlsRecordingSession: gui = %p, session=%p", 
	gui, session
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_GUILeavesRecordingSession(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUILeavesRecordingSession_CallbackHandle handle,
	DispRecXtAPI_RecordingSession session,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUILeavesRecordingSession: gui = %p, session=%p", 
	gui, session
    );
    WriteToLogger( logger, buf );
}

void 
    LogEvent_GUIHandleDeleted(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIHandleDeleted_CallbackHandle handle,
	void *client_data
    )
{
    Widget logger = (Widget) client_data;
    char buf[256];
    sprintf( 
	buf, 
	"GUIHandleDeleted: gui = %p", 
	gui
    );
    WriteToLogger( logger, buf );
}

/**********************/
/* more auxiliary functions  */
/**********************/

static void RegisterCallbacksForRecordingSession(
    DispRecXtAPI_RecordingSession session,
    VtError *err )
{
    /* callback functions for adjusting the recording session to listen to */
    gRecordingSessionDeleted_LogEvent_CallbackHandle =
	DispRecXtAPI_Add_RecordingSessionDeleted_Callback(
	    session,
	    UpdateSession_RecordingSessionDeleted,
	    NULL,
	    err
	);
    
    /* callbacks for logging */
    gRecordingStarted_LogEvent_CallbackHandle =
	DispRecXtAPI_Add_RecordingStarted_Callback(
	    session,
	    LogEvent_RecordingStarted,
	    guiEventLogger,
	    err
	);
    
    gRecordingNotStarted_LogEvent_CallbackHandle =
	DispRecXtAPI_Add_RecordingNotStarted_Callback(
	    session,
	    LogEvent_RecordingNotStarted,
	    guiEventLogger,
	    err
	);
	
    gRecordingFinished_LogEvent_CallbackHandle =
	DispRecXtAPI_Add_RecordingFinished_Callback(
	    session,
	    LogEvent_RecordingFinished,
	    guiEventLogger,
	    err
	);
	
    gRecordingNotFinished_LogEvent_CallbackHandle =    
	DispRecXtAPI_Add_RecordingNotFinished_Callback(
	    session,
	    LogEvent_RecordingNotFinished,
	    guiEventLogger,
	    err
	);
	
    gRecordingSessionDeleted_LogEvent_CallbackHandle =
	DispRecXtAPI_Add_RecordingSessionDeleted_Callback(
	    session,
	    LogEvent_RecordingSessionDeleted,
	    guiEventLogger,
	    err
	);
}

static void UnregisterCallbacksForCurrentRecordingSession(
    VtError *err )
{
    if ( gRecordingStarted_LogEvent_CallbackHandle != NULL ) {
	DispRecXtAPI_Remove_RecordingStarted_Callback(
	    gRecordingStarted_LogEvent_CallbackHandle, err 
	);
	gRecordingStarted_LogEvent_CallbackHandle = NULL;
    }

    if ( gRecordingNotStarted_LogEvent_CallbackHandle != NULL ) {
	DispRecXtAPI_Remove_RecordingNotStarted_Callback(
	    gRecordingNotStarted_LogEvent_CallbackHandle, err 
	);
	gRecordingNotStarted_LogEvent_CallbackHandle = NULL;
    }
        
    if ( gRecordingFinished_LogEvent_CallbackHandle != NULL ) {
	DispRecXtAPI_Remove_RecordingFinished_Callback(
	    gRecordingFinished_LogEvent_CallbackHandle, err 
	);
	gRecordingFinished_LogEvent_CallbackHandle = NULL;
    }
	    
    if ( gRecordingNotFinished_LogEvent_CallbackHandle != NULL ) {
	DispRecXtAPI_Remove_RecordingNotFinished_Callback(
	    gRecordingNotFinished_LogEvent_CallbackHandle, err 
	);
	gRecordingNotFinished_LogEvent_CallbackHandle = NULL;
    }
	        
    if ( gRecordingSessionDeleted_LogEvent_CallbackHandle != NULL ) {
	DispRecXtAPI_Remove_RecordingSessionDeleted_Callback(
	    gRecordingSessionDeleted_LogEvent_CallbackHandle, err 
	);
	gRecordingSessionDeleted_LogEvent_CallbackHandle = NULL;
    }
}

/**********************/
/* main */
/**********************/

int main( int argc, char *argv[] )
{
    DECLERR;
    Widget toplevel, mainWidget;
    XtAppContext app;
    
    XtSetLanguageProc(NULL, NULL, NULL);
    
    toplevel = 
	XtVaAppInitialize( 
	    &app, appname, 
	    NULL, 0,
	    &argc, argv,
	    NULL, NULL
	);
	
	

    /*******************/
    /* init the DispRec- API */
    /*******************/
    DispRecXtAPI_Init( app, err );
    
    apiServer = DispRecXtAPI_Connect( XtDisplay( toplevel ), err );
    if ( apiServer == DispRecXtAPI_Server_None ) {
	fprintf( stderr, "Could not connect to DispRecCtrl - Extension\n" );
	exit(1);
    }
    
    /* start the GUI */
    guiHandle =
	DispRecXtAPI_StartGUIIfNotRunning(
	    apiServer,
	    /* user = */ getuid(),
	    /* group = */ getgid(),
	    /* keepalive = */ True,    
	    err
	);	
    
    guiRecordingSession =
	DispRecXtAPI_GetRecordingSessionOfGUI( guiHandle, err );
	
    /***********/
    /* init widgets */
    /***********/
    mainWidget = 
	XtVaCreateManagedWidget( 
	    "mainWidget",
	     xmFormWidgetClass,
	     toplevel, 
	     XmNwidth, 500,
	     XmNheight, 150,	     
	     NULL );

    /***************************/	   
    /* Widgets for controlling the GUI */
    /***************************/	   
    {  
	guiManager =
	    XtVaCreateManagedWidget( 
		"guiManager",
		 xmRowColumnWidgetClass,
		 mainWidget, 
		 XmNorientation, XmVERTICAL,
		 XmNtopAttachment, XmATTACH_FORM,
		 XmNbottomAttachment, XmATTACH_FORM,
		 XmNleftAttachment, XmATTACH_FORM,
		 XmNrightAttachment, XmATTACH_NONE,
		 NULL );
	{
	    Widget showGUI =
		XtVaCreateManagedWidget(
		    "ShowGUI",
		    xmPushButtonWidgetClass,
		    guiManager,
		    NULL
		);
	    XtAddCallback( showGUI, XmNactivateCallback, ShowGUICallback, NULL );
	}
	{
	    Widget hideGUI =
		XtVaCreateManagedWidget(
		    "HideGUI",
		    xmPushButtonWidgetClass,
		    guiManager,
		    NULL
		);
	    XtAddCallback( 
		hideGUI, 
		XmNactivateCallback, HideGUICallback, NULL );
	}
    }

    /************************************************/	   
    /* Widgets for the gui-event logger */
    /************************************************/	   
    {
	Arg args[10];
	int n = 0;
	
	XtSetArg( args[n], XmNlistSizePolicy, XmRESIZE_IF_POSSIBLE ); n++;
	XtSetArg( args[n], XmNleftWidget, guiManager ); n++;
	XtSetArg( args[n], XmNtopAttachment, XmATTACH_FORM ); n++;
	XtSetArg( args[n], XmNbottomAttachment, XmATTACH_FORM ); n++;
	XtSetArg( args[n], XmNrightAttachment, XmATTACH_FORM ); n++;
	XtSetArg( args[n], XmNleftAttachment, XmATTACH_WIDGET ); n++;
	guiEventLogger = 
	    XmCreateScrolledList( 
		mainWidget,
		"guiEventLogger",
		args, n
	    );
	XtVaSetValues(
	    guiEventLogger,
	     XmNunitType, Xm100TH_FONT_UNITS, 
	     XmNscrollBarDisplayPolicy, XmAS_NEEDED,
	     XmNwidth, 100 * 100,
	     XmNvisibleItemCount, 10,
	     NULL );		
	XtManageChild( guiEventLogger );
	
	/*** add callbacks ***/
	if ( guiRecordingSession ) {
	    RegisterCallbacksForRecordingSession( 
		guiRecordingSession, err
	    );
	}
	
	/* callback functions for adjusting the recording session to listen to */
	DispRecXtAPI_Add_GUIControlsRecordingSession_Callback(
	    guiHandle,
	    UpdateSession_GUIControlsRecordingSession,
	    NULL,
	    err
	);
	
	DispRecXtAPI_Add_GUILeavesRecordingSession_Callback(
	    guiHandle,
	    UpdateSession_GUILeavesRecordingSession,
	    guiEventLogger,
	    err
	);
	
    
	/* callback functions for logging */
	
	DispRecXtAPI_Add_GUIStarted_Callback(
	    guiHandle,
	    LogEvent_GUIStarted,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUINotStarted_Callback(
	    guiHandle,
	    LogEvent_GUINotStarted,
	    guiEventLogger,
	    err
	);
    
	DispRecXtAPI_Add_GUIExited_Callback(
	    guiHandle,
	    LogEvent_GUIExited,
	    guiEventLogger,
	    err
	);
    
	DispRecXtAPI_Add_GUIRestarting_Callback(
	    guiHandle,
	    LogEvent_GUIRestarting,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUIRestarted_Callback(
	    guiHandle,
	    LogEvent_GUIRestarted,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUINotRestarted_Callback(
	    guiHandle,
	    LogEvent_GUINotRestarted,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUIVisible_Callback(
	    guiHandle,
	    LogEvent_GUIVisible,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUIHidden_Callback(
	    guiHandle,
	    LogEvent_GUIHidden,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUIControlsRecordingSession_Callback(
	    guiHandle,
	    LogEvent_GUIControlsRecordingSession,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUILeavesRecordingSession_Callback(
	    guiHandle,
	    LogEvent_GUILeavesRecordingSession,
	    guiEventLogger,
	    err
	);
	
	DispRecXtAPI_Add_GUIHandleDeleted_Callback(
	    guiHandle,
	    LogEvent_GUIHandleDeleted,
	    guiEventLogger,
	    err
	);
	    	    
    }

    XtRealizeWidget( toplevel );
    XtAppMainLoop(app);
    
    return 0;
}
