
#include <X11/Xlib.h>
#include "DispRec/DispRecCtrlExtension.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#define  COMPRESS

void ListenForEvents(DispRecCtrl_RecordingSession myRecordingSession)
{
    Display* dpy;
    int event_base, error_base;
    Bool haveDispRecCtrl;

    dpy=XOpenDisplay(NULL);

    if ( dpy == NULL ) {
	fprintf(stderr, "error: Could not open the display.\n");
	exit(1);
    }

    haveDispRecCtrl = 
	XDispRecCtrl_QueryExtension(
	    dpy, &event_base, &error_base);

    printf("haveDispRecCtrl=%d\n", haveDispRecCtrl);
    if (haveDispRecCtrl) {
	Bool finished;
	/* Select events */
	XDispRecCtrl_SelectInput(dpy, DispRecCtrl_NotifyMask);
	/* Loop for events */
	finished = False;
	while(!finished) {
	    XEvent event;
	    XNextEvent(dpy, &event);
	    if (event.type == event_base) {
		XDispRecCtrl_NotifyEvent *notifyevent=
		    (XDispRecCtrl_NotifyEvent*)&event;
		switch(notifyevent->subtype) {
		    case DispRecCtrl_NotifyStarted: {
			XDispRecCtrl_NotifyStartedEvent *myevent=
			    (XDispRecCtrl_NotifyStartedEvent*)notifyevent;
			printf(
			    "\nNotifyStartedEvent: recordingSession = %d\n", 
			    (int)myevent->recordingSession 
			);
		    } break;
		    case DispRecCtrl_NotifyNotStarted: {
			XDispRecCtrl_NotifyNotStartedEvent *myevent=
			    (XDispRecCtrl_NotifyNotStartedEvent*)notifyevent;
			printf(
			    "\nNotifyNotStartedEvent: recordingSession = %d, reason = %d ( = %s ), reasonArg1 = %d\n", 
			    (int)myevent->recordingSession,
			    myevent->reason,
			    XDispRecCtrl_ReasonToString(myevent->reason),
			    (int)myevent->reasonArg1
			);
		    } break;
		    case DispRecCtrl_NotifyFinished: {
			XDispRecCtrl_NotifyFinishedEvent *myevent=
			    (XDispRecCtrl_NotifyFinishedEvent*)notifyevent;
			printf(
			    "\nNotifyFinishedEvent: recordingSession = %d, reason = %d ( = %s )\n", 
			    (int)myevent->recordingSession,
			    myevent->reason,
			    XDispRecCtrl_ReasonToString(myevent->reason)
			);
			if ( 
				myevent->recordingSession 
				== 
				myRecordingSession 
			) {

				finished = True;
			}
		    } break;
		    case DispRecCtrl_NotifyNotFinished: {
			XDispRecCtrl_NotifyNotFinishedEvent *myevent=
			    (XDispRecCtrl_NotifyNotFinishedEvent*)notifyevent;
			printf(
			    "\nNotifyNotFinishedEvent: recordingSession = %d, reason = %d ( = %s )\n", 
			    (int)myevent->recordingSession,
			    myevent->reason,
			    XDispRecCtrl_ReasonToString(myevent->reason)
			);
			if ( 
				myevent->recordingSession 
				== 
				myRecordingSession 
			) {
				finished = True;
			}
		    } break;
		    case DispRecCtrl_NotifySwitchedDir: {
			XDispRecCtrl_NotifySwitchedDirEvent *myevent=
			    (XDispRecCtrl_NotifySwitchedDirEvent*)notifyevent;
			printf(
			    "\nNotifySwitchedDirEvent: recordingSession = %d\n", 
			    (int)myevent->recordingSession
			);			
		    } break;
		    case DispRecCtrl_NotifyNotSwitchedDir: {
			XDispRecCtrl_NotifyNotSwitchedDirEvent *myevent=
			    (XDispRecCtrl_NotifyNotSwitchedDirEvent*)notifyevent;
			printf(
			    "\nNotifyNotSwitchedDirEvent: recordingSession = %d, reason = %d ( = %s ), stillRecordingToOldDir=%d\n", 
			    (int)myevent->recordingSession,
			    myevent->reason,
			    XDispRecCtrl_ReasonToString(myevent->reason),
			    (int)(myevent->stillRecordingToOldDir)
			);			
		    } break;
		}
	    }
	}
    }	
}


int main(int argc, char* argv[])
{
    Display* dpy;
    Bool haveDispRecCtrl=False;
    int event_base, error_base;
    int major_version, minor_version;
    Status status;
    int childpid;
    DispRecCtrl_RecordingSession recordingSession;
    Bool finishOtherRecordingsBeforeStart = False;

    char recordingDir[1024];

    /* parse commandline */
    {
        Bool haveRecDir = False;
 	int argIdx;
    	for ( argIdx = 1; argIdx < argc; argIdx++ ) {
		const char* arg = argv[argIdx];
		if ( strcmp(arg, "-f") == 0 ) {
			finishOtherRecordingsBeforeStart = True;
		} else if ( arg[0] != '-' ) {
			strncpy( recordingDir, arg, 1024 );
			haveRecDir = True;
		}
    	}
	if ( !haveRecDir ) {
    		sprintf( recordingDir, "/var/tmp/testDispRecCtrlExt.%d", getpid() );
	}
    }
    
    printf("recording dir=\"%s\"\n", recordingDir);

    dpy=XOpenDisplay(NULL);

    if ( dpy == NULL ) {
	fprintf(stderr, "error: Could not open the display.\n");
	exit(1);
    }
    
    haveDispRecCtrl = 
	XDispRecCtrl_QueryExtension(
    	dpy, &event_base, &error_base);

    printf("haveDispRecCtrl=%d\n", haveDispRecCtrl);
    if (haveDispRecCtrl) {

	recordingSession=
		XDispRecCtrl_CreateRecordingSession( 
			dpy, 
			/* flags = */
				XDispRecCtrl_CreateRecordingSession_Flag_TieLifetimeToClient 
		);
    
	childpid = fork();
	if (childpid != 0) {
	    DECLERR;
	    Bool finishRecording = False;
	    /***************************/
	    /* parent process */
	    /***************************/
    
	    status = XDispRecCtrl_QueryVersion(
		dpy, &major_version, &minor_version);
	    printf("Version=(%d, %d)\n", major_version, minor_version);
	    fflush(stdout);
    
    #ifdef COMPRESS
	    /* Turn on compression with gzip */
	    XDispRecCtrl_EnableCompression(dpy, recordingSession, "gzip", "/usr/bin/gzip  --fast  ");
    #else
	    XDispRecCtrl_DisableCompression(dpy, recordingSession);
    #endif
    
	    /* configure sound */
	    {
		    XDispRecCtrl_SoundRecConfig soundConfig = 
			XDispRecCtrl_CreateSoundRecConfig(err);
		    XDispRecCtrl_SoundRecConfig_SetCompressionMethod(soundConfig, "none", err);
		    XDispRecCtrl_SetSoundRecordingConfig(
			dpy, recordingSession, soundConfig );
		    XDispRecCtrl_FreeSoundRecConfig(soundConfig);
	    }
	    sleep(1); /* give the child process time to startup */
    
	    printf("sending request to start recording\n");
	    fflush(stdout);
    
	    XDispRecCtrl_StartRecording(
		dpy, recordingSession, recordingDir,
		DispRecCtrl_DirMode_CreateDirWhichHasToBeNew,
		0, 
		/* endless = */ False, 
		/* doSoundRecording = */ False, 
		/* muteSoundRecording = */ False,
		finishOtherRecordingsBeforeStart
	    );
	    XFlush(dpy);
	    
	    /* wait some time, so display of the returned events does not interfere with display
		of the "press key to finish recording" */
	    sleep(1); 
    
	    do {
		size_t maxCommandSize = 2048;
		char commandBuf[2048];
		size_t commandSize = 0;
		char* command = NULL;
		printf ("enter \"f\" to finish recording\n");
		printf ("enter \"d directory\" to switch to the recording directory directory\n");
		fflush(stdout);
		command = fgets( commandBuf, maxCommandSize, stdin );
		if ( command != NULL ) {
			switch( command[0] ) {
				case 'f': finishRecording = True; break;
				case 'd': {
					if ( 
						( command[1] == ' ' ) &&
						( command[2] != 0 )
					) {
						char* newRecDir = command+2;
						/* remove trailing newline */
						int newRecDirSize = strlen(newRecDir);
						if ( newRecDir[newRecDirSize-1] == '\n' ) {
							newRecDir[newRecDirSize-1] =0;
						}
						printf("sending request to switch to dir \"%s\"\n", newRecDir);
						/* call API-function */
						XDispRecCtrl_SwitchRecordingDir(
							dpy,
							recordingSession,
							newRecDir,
							DispRecCtrl_DirMode_CreateDirWhichHasToBeNew
						);
					} else {
						printf("malformed d-command\n"); 
					}
				} break;
				default:
					printf("unknown command ( ignored )\n");
			}
		}
	   } while(!finishRecording);
    
	    printf("sending request to finish recording\n");
	    fflush(stdout);
	    XDispRecCtrl_FinishRecording(dpy, recordingSession);
	    XFlush(dpy);
    
	    /* wait for child process to terminate */
	    {
		int status;
		waitpid( childpid, &status, 0 );
	    }
	    XFlush(dpy);
	    XCloseDisplay(dpy);
	    return 0;
	} else {
	    /***************************/
	    /* Child process (listen for events) */
	    /***************************/
	    ListenForEvents(recordingSession);
	    return 0;
	}
    }
    return 0;
}

