/****************************************
Filename: DRError.h
Description: Error handling
Author: Christian Linhart
Copyright: (C)1997 Christian Linhart
****************************************/

#ifndef VT_error_h
#define VT_error_h

#ifndef VT_error_no_DISPREC_PRIVATE
#endif

#include <X11/Xfuncproto.h>

/* forward decl of VtErrorRec */
typedef struct VtErrorRec_struct VtErrorRec;

/*** data type for errors (pointer to error record).
  NULL means "no error" ***/
typedef VtErrorRec *VtError;

#ifndef Bool
#  ifndef _XTYPEDEF_BOOL
#   define _XTYPEDEF_BOOL
typedef int Bool;
#  endif
#endif

_XFUNCPROTOBEGIN

/*** forward decls ***/

/*** Error types ***/
/* some comments for the error-types are taken from Linux
   man-pages (or from the SunOS man-pages for error-codes
   which are not described in the Linux-manpages) */
typedef enum{
    VtError_NoError
	,VtError_L1WriteError
	,VtError_L1ReadError

	/***********************************/
	/*** errors on opening a file for reading ***/
	/***********************************/

	/* no more detail on reason available */
	,VtError_CantOpenFileForReading 

	/* The requested access to the file is not allowed, or
	   one of the directories in pathname  did  not  allow
	   search (execute) permission. */
	,VtError_CantOpenFileForReading_EACCES

	/* A signal was caught  during  the  open() system call. */
	,VtError_CantOpenFileForReading_EINTR

	/* An I/O error occurred while reading from
	   or writing to the file system. */
	,VtError_CantOpenFileForReading_EIO
	
	    /* pathname contains a reference to  a  circular  sym
              bolic link, ie a symbolic link whose expansion con
              tains a reference to itself. */
	,VtError_CantOpenFileForReading_ELOOP

	    /* The process already has the maximum number of files
              open. */
	,VtError_CantOpenFileForReading_EMFILE

	    /* pathname was too long */
	,VtError_CantOpenFileForReading_ENAMETOOLONG

	    /* The limit on the total number of files open on  the
	       system has been reached. */
	,VtError_CantOpenFileForReading_ENFILE

	    /* A directory component in pathname does not exist or
              is a dangling symbolic link */
	,VtError_CantOpenFileForReading_ENOENT

	    /* A component used as a directory in pathname is not,
	       in fact, a directory. */
	,VtError_CantOpenFileForReading_ENOTDIR
	
	/****************************************/
	/*** errors on opening a  file for writing and/or ***/
	/*** on creating a file                                               ***/
	/****************************************/
	/* no more detail on reason available */
	,VtError_CantOpenFileForWriting

	/* The requested access to the file is not allowed, or
              one of the directories in pathname  did  not  allow
              search (execute) permission */
	,VtError_CantOpenFileForWriting_EACCES

	/* The file  does  not  exist,  O_CREAT  is
                         specified,  and  the  directory in which
                         the entry for  the  new  file  is  being
                         placed  cannot  be  extended because the
                         user's quota of disk blocks on the  file
                         system containing the directory has been
                         exhausted.

                         The file  does  not  exist,  O_CREAT  is
                         specified,   and  the  user's  quota  of
                         inodes on the file system on  which  the
                         file   is   being   created   has   been
                         exhausted. */
	,VtError_CantOpenFileForWriting_EDQUOT

	/* pathname already exists and O_CREAT and O_EXCL were
              used. */
	,VtError_CantOpenFileForWriting_EEXIST

	/* A signal was caught  during  the  open() system call. */
	,VtError_CantOpenFileForWriting_EINTR

	/* An I/O error occurred while reading from
	   or writing to the file system. */
	,VtError_CantOpenFileForWriting_EIO
	
	/* pathname  refers  to  a  directory  and  the access
              requested involved writing */
	,VtError_CantOpenFileForWriting_EISDIR

	/* pathname contains a reference to  a  circular  sym
              bolic link, ie a symbolic link whose expansion con
              tains a reference to itself */
	,VtError_CantOpenFileForWriting_ELOOP

	/* The process already has the maximum number of files
              open */
	,VtError_CantOpenFileForWriting_EMFILE

	/* pathname was too long */
	,VtError_CantOpenFileForWriting_ENAMETOOLONG

	/* The limit on the total number of files open on  the
              system has been reached */
	,VtError_CantOpenFileForWriting_ENFILE

	/* A directory component in pathname does not exist or
              is a dangling symbolic link */
	,VtError_CantOpenFileForWriting_ENOENT

	/* pathname was to be created but the device  contain
              ing pathname has no room for the new file */
	,VtError_CantOpenFileForWriting_ENOSPC

	/* A component used as a directory in pathname is not,
	   in fact, a directory */
	,VtError_CantOpenFileForWriting_ENOTDIR

	/* O_NDELAY is set, the  named  file  is  a
                         FIFO,  O_WRONLY  is  set, and no process
                         has the file open for reading */
	,VtError_CantOpenFileForWriting_ENXIO

	/* pathname refers to a file on a read-only filesystem
              and write access was requested */
	,VtError_CantOpenFileForWriting_EROFS

	/* pathname refers to an  executable  image  which  is
              currently  being  executed  and  write  access  was
              requested. */
	,VtError_CantOpenFileForWriting_ETXTBSY

	/****************************************/
	/*** errors on closing a file for writing ***/
	/****************************************/

	/* no more detail on reason available */
	,VtError_CantCloseFileForWriting

	/* fd isn't a valid open file descriptor. */
	,VtError_CantCloseFileForWriting_EBADF

	/* An I/O error occurred. */
	,VtError_CantCloseFileForWriting_EIO

	/* renaming to the final file failed because the file exists 
	   and the file-descriptor was opened with O_EXCL */
	,VtError_CantCloseFileForWriting_EEXIST

	/***********************************/
	/*** errors on opening a file for reading ***/
	/***********************************/
	/* no more detail on reason available */
	,VtError_CantOpenDir
	
	/* Search permission is denied for the component  of  the
	path  prefix  of  dirname or read permission is denied
	for dirname. */
	,VtError_CantOpenDir_EACCES
	
	/* Too many symbolic links were encountered in  resolving
	path. */
	,VtError_CantOpenDir_ELOOP
	
	/* The length of the dirname argument  exceeds  PATH_MAX,
       or  a pathname component is longer than NAME_MAX while
       _POSIX_NO_TRUNC is in effect. 
	    or
	Pathname resolution of a  symbolic  link  produced  an
	intermediate result whose length exceeds PATH_MAX. */
	,VtError_CantOpenDir_ENAMETOOLONG

	/* A component of  dirname  does  not  name  an  existing
	directory or dirname is an empty string. */
	,VtError_CantOpenDir_ENOENT

	/* A component of dirname is not a directory. */
	,VtError_CantOpenDir_ENOTDIR

	/* There are OPEN_MAX file descriptors currently open  in
	the calling process. */
	,VtError_CantOpenDir_EMFILE

	/* Too many files are currently open in the system. */
	,VtError_CantOpenDir_ENFILE
	
	/* the directory does not contain a keyfile, thus it does not contain a valid recording */
	, VtError_CantOpenDir_NoKeyFile

	/****************************************/
	/*** errors on closing a file for reading ***/
	/****************************************/

	/* no more detail on reason available */
	,VtError_CantCloseFileForReading

	/* fd isn't a valid open file descriptor. */
	,VtError_CantCloseFileForReading_EBADF

	/* An I/O error occurred. */
	,VtError_CantCloseFileForReading_EIO

	/****************************************/
	/*** errors on moving a file ***/
	/****************************************/

	/* no more detail on reason available */
	,VtError_CantRenameFile

	/* Write  permission is denied for the directory containing oldpath
	or newpath, or, search permission  is  denied  for  one  of  the
      directories in the path prefix of oldpath or newpath, or oldpath
      is a directory and does not allow write  permission  (needed  to
      update the ..  entry).  (See also path_resolution(7).) */
	,VtError_CantRenameFile_EACCES

	/* The  rename fails because oldpath or newpath is a directory that
      is in use by some process (perhaps as current working directory,
      or  as root directory, or because it was open for reading) or is
      in use by the system (for example as  mount  point),  while  the
      system considers this an error.  (Note that there is no require-
      ment to return EBUSY in such cases --  there  is  nothing  wrong
      with  doing  the  rename  anyway  -- but it is allowed to return
      EBUSY if the system cannot otherwise handle such situations.) */
	,VtError_CantRenameFile_EBUSY

	/* oldpath or newpath points outside your accessible address space. */
	,VtError_CantRenameFile_EFAULT

	/* The new pathname contained a path prefix of the  old,  or,  more
      generally,  an  attempt was made to make a directory a subdirec-
      tory of itself. */
	,VtError_CantRenameFile_EINVAL

	/* newpath is an existing directory, but oldpath is  not  a  direc-
      tory. */
	,VtError_CantRenameFile_EISDIR

	/* Too many symbolic links were encountered in resolving oldpath or
      newpath. */
	,VtError_CantRenameFile_ELOOP

	/* oldpath already has the maximum number of links to it, or it was
      a directory and the directory containing newpath has the maximum
      number of links. */
	,VtError_CantRenameFile_EMLINK

	/* oldpath or newpath was too long. */
	,VtError_CantRenameFile_ENAMETOOLONG

	/* The link named by oldpath does not exist; or, a directory compo-
      nent  in  newpath  does  not exist; or, oldpath or newpath is an
      empty string. */
	,VtError_CantRenameFile_ENOENT

	/* Insufficient kernel memory was available. */
	,VtError_CantRenameFile_ENOMEM

	/* The device containing the file has no room for the new directory
      entry. */
	,VtError_CantRenameFile_ENOSPC

	/* A component used as a directory in oldpath or newpath is not, in
      fact, a directory.  Or, oldpath  is  a  directory,  and  newpath
      exists but is not a directory. */
	,VtError_CantRenameFile_ENOTDIR

	/* newpath is a nonempty directory, that is, contains entries other
      than "." and "..". */
	,VtError_CantRenameFile_ENOTEMPTY

	/* newpath is a nonempty directory, that is, contains entries other
      than "." and "..". */
	,VtError_CantRenameFile_EEXIST

	/* The directory containing oldpath has the  sticky  bit  (S_ISVTX)
      set  and  the process's effective user ID is neither the user ID
      of the file to be deleted nor that of the  directory  containing
      it,  and the process is not privileged (Linux: does not have the
      CAP_FOWNER capability); or newpath is an existing file  and  the
      directory containing it has the sticky bit set and the process's
      effective user ID is neither the user  ID  of  the  file  to  be
      replaced  nor  that  of  the  directory  containing  it, and the
      process is not privileged (Linux: does not have  the  CAP_FOWNER
      capability);  or  the  file  system containing pathname does not
      support renaming of the type requested. */
	,VtError_CantRenameFile_EPERM

	/* The file is on a read-only file system. */
	,VtError_CantRenameFile_EROFS

	/* oldpath and newpath are not on the  same  mounted  file  system.
      (Linux  permits  a file system to be mounted at multiple points,
      but rename() does not work across different mount  points,  even
      if the same file system is mounted on both.) */
	,VtError_CantRenameFile_EXDEV

	/****************************************/
	/*** errors on duplicating a file-descriptor ***/
	/****************************************/

	/* fd  isn't  an  open  file  descriptor. */
	,VtError_DupFailed_EBADF

	/* The  process  already has the maximum number of file descriptors
      	open and tried to open a new one. */
	,VtError_DupFailed_EMFILE

	/* unknown error code --> report default error */
	,VtError_DupFailed

	/****************************************/
	/*** errors on reading a symlink ***/
	/****************************************/

	/* Search  permission is denied for a component of the path prefix.
        (See also path_resolution(7).) */
	,VtError_CantReadSymlink_EACCES

	/* buf extends outside the process's allocated address space. */
	,VtError_CantReadSymlink_EFAULT

	/* bufsiz is not positive. */
	/* or: The named file is not a symbolic link. */
	,VtError_CantReadSymlink_EINVAL

	/* An I/O error occurred while reading from the file system. */
	,VtError_CantReadSymlink_EIO

	/* Too many symbolic links  were  encountered  in  translating  the
      	   pathname. */
	,VtError_CantReadSymlink_ELOOP

      	/* A pathname, or a component of a pathname, was too long. */
	,VtError_CantReadSymlink_ENAMETOOLONG

	/* The named file does not exist. */
	,VtError_CantReadSymlink_ENOENT

	/* Insufficient kernel memory was available. */
	,VtError_CantReadSymlink_ENOMEM

      	/* A component of the path prefix is not a directory. */
	,VtError_CantReadSymlink_ENOTDIR

	/* unknown error code --> report default error */
	,VtError_CantReadSymlink


	/****************************/
	/*** errors with out of memory ***/
	/****************************/
	,VtError_OutOfMemory
	,VtError_BaseServerOutOfMemory
	,VtError_RpservOutOfMemory

	/****************************/
	/*** errors with memoryaccess ***/
	/****************************/
	,VtError_PrematureEndOfX11FrameReached

} VtErrorType;


/*** Data records for error types ***/



/* for 
   VtError_CantOpenFileForReading,
   VtError_CantOpenFileForReading_...,
   VtError_CantOpenFileForWriting,
   VtError_CantOpenFileForWriting_...,
   */
typedef struct{
    char* filename;
} VtError_FileOpenError;

typedef struct{
    int fd;
    char* filename;
} VtError_FileCloseError;

/* for
    VtError_CantOpenDir,
    VtError_CantOpenDir_...
    */
typedef struct{
    char* dirname;
} VtError_DirOpenError;

typedef struct{
	char *oldpath;
	char *newpath;
} VtError_RenameError;

typedef struct{
	int fd;
} VtError_dup;

typedef struct{
	char* path;
} VtError_ReadSymlinkError;

/*** error record ***/
struct VtErrorRec_struct{
    VtErrorType type;
    Bool free_on_delete; /* True: May be freed,
			  False: is statically allocated --> may not be freed */
    union{
	VtError_FileOpenError file_open;
	VtError_FileCloseError file_close;
	VtError_DirOpenError dir_open;
	VtError_RenameError file_rename;
	VtError_dup dup;
	VtError_ReadSymlinkError readlink;
    } data;
};


/***********************************/
/* Macros for convenient handling of errors */
/***********************************/

/* Some classes of errors (for use in switch statements) */
#define VtError_CantOpenFileForReading_CLASS \
VtError_CantOpenFileForReading: \
case VtError_CantOpenFileForReading_EACCES: \
case VtError_CantOpenFileForReading_EINTR: \
case VtError_CantOpenFileForReading_EIO: \
case VtError_CantOpenFileForReading_ELOOP: \
case VtError_CantOpenFileForReading_EMFILE: \
case VtError_CantOpenFileForReading_ENAMETOOLONG: \
case VtError_CantOpenFileForReading_ENFILE: \
case VtError_CantOpenFileForReading_ENOENT: \
case VtError_CantOpenFileForReading_ENOTDIR
	
#define VtError_CantOpenFileForWriting_CLASS \
VtError_CantOpenFileForWriting: \
case VtError_CantOpenFileForWriting_EACCES: \
case VtError_CantOpenFileForWriting_EDQUOT: \
case VtError_CantOpenFileForWriting_EEXIST: \
case VtError_CantOpenFileForWriting_EINTR: \
case VtError_CantOpenFileForWriting_EIO: \
case VtError_CantOpenFileForWriting_EISDIR: \
case VtError_CantOpenFileForWriting_ELOOP: \
case VtError_CantOpenFileForWriting_EMFILE: \
case VtError_CantOpenFileForWriting_ENAMETOOLONG: \
case VtError_CantOpenFileForWriting_ENFILE: \
case VtError_CantOpenFileForWriting_ENOENT: \
case VtError_CantOpenFileForWriting_ENOSPC: \
case VtError_CantOpenFileForWriting_ENOTDIR: \
case VtError_CantOpenFileForWriting_ENXIO: \
case VtError_CantOpenFileForWriting_EROFS: \
case VtError_CantOpenFileForWriting_ETXTBSY

#define VtError_CantCloseFileForWriting_CLASS \
VtError_CantCloseFileForWriting: \
case VtError_CantCloseFileForWriting_EBADF: \
case VtError_CantCloseFileForWriting_EIO: \
case VtError_CantCloseFileForWriting_EEXIST

#define VtError_CantOpenDir_CLASS \
VtError_CantOpenDir: \
case VtError_CantOpenDir_EACCES: \
case VtError_CantOpenDir_ELOOP: \
case VtError_CantOpenDir_ENAMETOOLONG: \
case VtError_CantOpenDir_ENOENT: \
case VtError_CantOpenDir_ENOTDIR: \
case VtError_CantOpenDir_EMFILE: \
case VtError_CantOpenDir_ENFILE: \
case VtError_CantOpenDir_NoKeyFile

#define VtError_CantCloseFileForReading_CLASS \
VtError_CantCloseFileForReading: \
case VtError_CantCloseFileForReading_EBADF: \
case VtError_CantCloseFileForReading_EIO

#define VtError_CantRenameFile_CLASS \
VtError_CantRenameFile: \
case VtError_CantRenameFile_EACCES: \
case VtError_CantRenameFile_EBUSY: \
case VtError_CantRenameFile_EFAULT: \
case VtError_CantRenameFile_EINVAL: \
case VtError_CantRenameFile_EISDIR: \
case VtError_CantRenameFile_ELOOP: \
case VtError_CantRenameFile_EMLINK: \
case VtError_CantRenameFile_ENAMETOOLONG: \
case VtError_CantRenameFile_ENOENT: \
case VtError_CantRenameFile_ENOMEM: \
case VtError_CantRenameFile_ENOSPC: \
case VtError_CantRenameFile_ENOTDIR: \
case VtError_CantRenameFile_ENOTEMPTY: \
case VtError_CantRenameFile_EEXIST: \
case VtError_CantRenameFile_EPERM: \
case VtError_CantRenameFile_EROFS: \
case VtError_CantRenameFile_EXDEV

#define VtError_DupFailed_CLASS \
VtError_DupFailed_EBADF: \
case VtError_DupFailed_EMFILE: \
case VtError_DupFailed

#define VtError_CantReadSymlink_CLASS \
VtError_CantReadSymlink_EACCES: \
case VtError_CantReadSymlink_EFAULT: \
case VtError_CantReadSymlink_EINVAL: \
case VtError_CantReadSymlink_EIO: \
case VtError_CantReadSymlink_ELOOP: \
case VtError_CantReadSymlink_ENAMETOOLONG: \
case VtError_CantReadSymlink_ENOENT: \
case VtError_CantReadSymlink_ENOMEM: \
case VtError_CantReadSymlink_ENOTDIR: \
case VtError_CantReadSymlink

/* Declare an err-pointer and init it with no-error */
#define DECLERR \
VtError errval = NULL; \
VtError *err = &errval \


#define DECLNAMEDERR(errVarname) \
VtError errVarname##_errval__ = NULL; \
VtError *errVarname = &errVarname##_errval__ \

#define SWAPERR(errVarname1, errVarname2) \
	{ \
		VtError tmp = *errVarname1; \
		*errVarname1 = *errVarname2; \
		*errVarname2 = tmp; \
	}


/* Delete the current error */
#define DELNAMEDERR(errVarname) \
			if ( \
				(errVarname != NULL ) \
				&& \
				(*errVarname != NULL) \
				&& \
				(*errVarname)->free_on_delete \
			) { \
			   VtError_DeleteErrorData(*errVarname); \
			   DELETE(*errVarname); \
			} \
                        *errVarname = NULL; 

#define DELERR DELNAMEDERR(err)

#define ERROR_TYPE() ((*err)->type)

#define ERROR_TYPE_ARG(errorObject) ((errorObject)->type)

#define HAVE_ERROR() ((err!=NULL) && (*err != NULL) && ( ERROR_TYPE() != VtError_NoError ) )

#define HAVE_ERROR_ARG(errorObject) \
    (((errorObject) != NULL) && ( ERROR_TYPE_ARG(errorObject) != VtError_NoError ) )

/* Return a value on error */
#define RETVALONERR \
if (HAVE_ERROR()) return ERR_RETVAL

/* Return on error in a void function */
#define RETONERR \
if (HAVE_ERROR()) return

/* Return a value on error and delete the error */
#define DEL_RETVALONERR \
if (HAVE_ERROR()) { \
    DELERR; \
    return ERR_RETVAL; \
}

/* Return on error in a void function */
#define DEL_RETONERR \
if (HAVE_ERROR()) { \
    DELERR; \
    return; \
}

#define ERROR_DATA(kind) ((*err)->data.kind)

#define NEW_ERROR  VtError_New(err)


/*********************************************/
/* Functions which may only be used by the previously */
/* defined macros                                                                       */
/*********************************************/

void VtError_DeleteErrorData(VtError err);

void VtError_New(VtError *err);

/* generates a string description of the given error. 
    The string has to be deallocated by the caller using DELETEARRAY */
char* VtErrorToString( VtError errorObject );

_XFUNCPROTOEND

#endif
