/****************************************
Filename: DRMem.h
Description: Memory management
Author: Christian Linhart
Copyright: (C)1997, 1998 Christian Linhart
****************************************/

#ifndef VTINC_mem_h
#define VTINC_mem_h

#undef abs
#include <stdlib.h>
#include "DRError.h"

#include <X11/Xfuncproto.h>

_XFUNCPROTOBEGIN
/* Malloc with error check */
void* Vtmem_alloc(size_t size, VtError *err);

/* set out of memory error */
void Vtmem_SetOutOfMemError( VtError *err );

_XFUNCPROTOEND

#define NEW_FORPLAINC(type) (type*)Vtmem_alloc(sizeof(type), err)
#define NEWARRAY_FORPLAINC(type, dim) (type*)Vtmem_alloc(sizeof(type)*(dim), err)
#define DELETE_FORPLAINC(ptr)  { if (ptr != NULL) free(ptr); }
#define DELETEARRAY_FORPLAINC(ptr)  { if (ptr != NULL) free(ptr); }

#if defined(__cplusplus) || defined(c_plusplus)

template<typename T>
    T* Vtmem_NewPCheck( T* p, VtError *err )
    {
	if ( p == NULL ) {
	    Vtmem_SetOutOfMemError( err );
	}
	return p;
    }

#define NEW(typeAndOptionalArgs) Vtmem_NewPCheck(new typeAndOptionalArgs, err)
#define NEWARRAY(type, dim) Vtmem_NewPCheck( new type[dim], err )
#define DELETE(ptr)  { if (ptr != NULL) delete ptr; }
#define DELETEARRAY(ptr)  { if (ptr != NULL) delete[] ptr; }


#else
#define NEW(type) NEW_FORPLAINC(type)
#define NEWARRAY(type, dim) NEWARRAY_FORPLAINC(type,dim)
#define DELETE(ptr) DELETE_FORPLAINC(ptr)
#define DELETEARRAY(ptr) DELETEARRAY_FORPLAINC(ptr)
#endif


#endif
