/****************************************
Filename: libDispRecCtrlExtension/DispRecCtrlExtension.h
Description: DispRecCtrl-Extension external declarations
Author: Christian Linhart
Copyright: (C)1997-2011 Christian Linhart
****************************************/

/* Based on shape.h which is Copyright (c) 1989  X Consortium */

#ifndef _DispRecCtrl_H_
#define _DispRecCtrl_H_


#include <X11/Xfuncproto.h>
#include <X11/Xmd.h>
#include "DispRec/DRError.h"
#include "DispRec/DRTime.h"



/* Minor request numbers */
#define X_DispRecCtrl_QueryVersion		0
#define X_DispRecCtrl_StartRecording		1
#define X_DispRecCtrl_FinishRecording			4
#define X_DispRecCtrl_EnableCompression			5
#define X_DispRecCtrl_DisableCompression			6
#define X_DispRecCtrl_SetMaxFilesysUsage		9
#define X_DispRecCtrl_DisableMaxFilesysUsage		10
#define X_DispRecCtrl_Active 13
#define X_DispRecCtrl_GetBaseDisplay 15
#define X_DispRecCtrl_SelectInput 16
#define X_DispRecCtrl_InputSelected 17
#define X_DispRecCtrl_ReparentToBaseWindow 18
#define X_DispRecCtrl_UnreparentWindow 19
#define X_DispRecCtrl_CreateBaseColormap 22
#define X_DispRecCtrl_FreeBaseColormap 23
#define X_DispRecCtrl_BaseColormap_StoreColors 24
#define X_DispRecCtrl_AllocateUserColorInBaseColormap 25
#define X_DispRecCtrl_DeleteUserColor 26
#define X_DispRecCtrl_AuthorizeReuseOfUserColorID 27
#define X_DispRecCtrl_ChangeBaseColormapOfReparentedWindow 28
#define X_DispRecCtrl_RedrawEverything 29
#define X_DispRecCtrl_InhibitColormapInstall 30
#define X_DispRecCtrl_EnableColormapInstall 31
#define X_DispRecCtrl_SetMotionKeyRatio 32
#define X_DispRecCtrl_GetLicenseInfo 34
#define X_DispRecCtrl_DeepSync 35
#define X_DispRecCtrl_SetSoundRecordingConfig 44
#define X_DispRecCtrl_CreateRecordingSession 47
#define X_DispRecCtrl_DeleteRecordingSession 48
#define X_DispRecCtrl_CreateGUIHandle 49
#define X_DispRecCtrl_DeleteGUIHandle 50
#define X_DispRecCtrl_StartGUI 51
#define X_DispRecCtrl_SetGUIKeepalive 52
#define X_DispRecCtrl_StartGUIIfNotRunning 53
#define X_DispRecCtrl_QuitGUI 54
#define X_DispRecCtrl_KillGUI 55
#define X_DispRecCtrl_ShowGUI 56
#define X_DispRecCtrl_HideGUI 57
#define X_DispRecCtrl_GUIRunning 58
#define X_DispRecCtrl_GUIVisible 59
#define X_DispRecCtrl_GetRecordingSessionOfGUI 60
/* GUI communicates its status to the server */
#define X_DispRecCtrl_IAmThisGUI 61
#define X_DispRecCtrl_IndicateThisGUIShuttingDown 62
#define X_DispRecCtrl_IndicateThisGUIHidden 63
#define X_DispRecCtrl_IndicateThisGUIVisible 64
#define X_DispRecCtrl_IndicateThisGUIControlsRecordingSession 65
#define X_DispRecCtrl_IndicateThisGUILeavesRecordingSession 66
/* misc requests which have been added later */
#define X_DispRecCtrl_SwitchRecordingDir 67
#define X_DispRecCtrl_ListRecordingSessions 73

/* Events */
/* major event numbers */
#define DispRecCtrl_Notify 0
#define DispRecCtrl_UserColor 1
#define DispRecCtrl_DRSoundRec 2
#define DispRecCtrl_InterceptedEvent 3

#define DispRecCtrl_NumberEvents (DispRecCtrl_InterceptedEvent + 1 )

/* event masks */
#define DispRecCtrl_NotifyMask (1L << 0)
#define DispRecCtrl_UserColorMask (1L << 1)
#define DispRecCtrl_DRSoundRecMask (1L << 2)
#define DispRecCtrl_InterceptedEventMask (1L << 3)

/* minor event numbers for DispRecCtrl_Notify */
#define DispRecCtrl_NotifyStarted 0
#define DispRecCtrl_NotifyNotStarted 1
#define DispRecCtrl_NotifyFinished 3
#define DispRecCtrl_NotifyNotFinished 4
#define DispRecCtrl_NotifyRecordingSessionDeleted 5
#define DispRecCtrl_NotifyGUIStarted 6
#define DispRecCtrl_NotifyGUINotStarted 7
#define DispRecCtrl_NotifyGUIExited 8
#define DispRecCtrl_NotifyGUIRestarting 9
#define DispRecCtrl_NotifyGUIRestarted 10
#define DispRecCtrl_NotifyGUINotRestarted 11
#define DispRecCtrl_NotifyGUIVisible 12
#define DispRecCtrl_NotifyGUIHidden 13
#define DispRecCtrl_NotifyGUIControlsRecordingSession 14
#define DispRecCtrl_NotifyGUILeavesRecordingSession 15
#define DispRecCtrl_NotifyGUIHandleDeleted 16
#define DispRecCtrl_NotifySwitchedDir 18
#define DispRecCtrl_NotifyNotSwitchedDir 19


/* minor event numbers for DispRecCtrl_UserColor */
#define DispRecCtrl_UserColor_ChangePixel 0
#define DispRecCtrl_UserColor_Delete 1

/* minor event numbers  for DispRecCtrl_DRSoundRec */
#define DispRecCtrl_DRSoundRec_RecLevel 1


/* Values for the reason field of events or requests */
/* ("length" is in time units, "size" is in storage units */
#define DispRecCtrl_Reason_ByClientRequest 0
#define DispRecCtrl_Reason_AlreadyRecording 1
#define DispRecCtrl_Reason_CannotOpenFile 2
#define DispRecCtrl_Reason_MaxLengthReached 3
#define DispRecCtrl_Reason_MaxSizeReached 4
#define DispRecCtrl_Reason_MaxFsUsageReached 5
#define DispRecCtrl_Reason_BadValue 6
#define DispRecCtrl_Reason_WriteError 7
#define DispRecCtrl_Reason_FileDisappeared 8
#define DispRecCtrl_Reason_TotalMaxLengthReached 9
#define DispRecCtrl_Reason_NoLicense 10
#define DispRecCtrl_Reason_SoundDeviceReadError 11
#define DispRecCtrl_Reason_SoundDeviceConfigError 12
#define DispRecCtrl_Reason_SoundDeviceOpenError 13
#define DispRecCtrl_Reason_SoundDeviceMissingCapabilities 14
#define DispRecCtrl_Reason_ChildprocessDiedUnexpectedly 15
#define DispRecCtrl_Reason_CannotStartSoundCompression 16
#define DispRecCtrl_Reason_NoSoundSupport 17
#define DispRecCtrl_Reason_NotRecording 18
#define DispRecCtrl_Reason_Killed 19
#define DispRecCtrl_Reason_AlreadyRunning 20
#define DispRecCtrl_Reason_ProcessTerminated 21
#define DispRecCtrl_Reason_NotPaused 22
#define DispRecCtrl_Reason_DueToSwitchDir 23
#define DispRecCtrl_Reason_MissingDirectory 24
#define DispRecCtrl_Reason_SwitchDirNotSupportedInEndlessMode 25
	/* switch dir is not supported when the endless-mode ( round-buffer-mode ) is active */
#define DispRecCtrl_Reason_BadDirectory 26
	/* the directory name is malformed in some way ( e.g. containing "//" or similar things ) */
#define DispRecCtrl_Reason_DirectoryDoesNotExist 27
#define DispRecCtrl_Reason_SwitchToNextFileFailed 28
#define DispRecCtrl_Reason_WritingCntFileFailed 29
#define DispRecCtrl_Reason_SoundControlFileNotOpened 30
#define DispRecCtrl_Reason_InternalError 31
#define DispRecCtrl_Reason_ErrorReadingPipeFromSoundRecordingProcess 32
#define DispRecCtrl_Reason_FailedToCreateRecDir 33
#define DispRecCtrl_Reason_RecDirAlreadyExists 34
#define DispRecCtrl_Reason_TiedClientDisconnected 35
#define DispRecCtrl_Reason_DueToOptionFinishOtherRecordingsBeforeStart 36
#define DispRecCtrl_Reason_ServerShutdown 37
#define DispRecCtrl_Reason_KeyFileWriterDied 38

/* internal reasons */
#define DispRecCtrl_Reason_NotRecording_DontSendEvent -2


/*** Type definitions ***/
/* UserColorID: for user-color management (only needed with PseudoColor visuals) */
typedef CARD32 DispRecCtrl_UserColorID;
#define DispRecCtrl_UserColorID_NULL 0

/* RecordingSession: identifies a  recording process. 
    Is necessary for two reasons:
	1. StartRecording works asynchronously, so
	    the ID is necessary to identify the RecordingStartedEvent or
	    RecordingNotStartedEvent with the request.
	2. Future versions of the recording engine may support
	    multiple recordings running simultanously.
	    Then the ID is needed for many requests in order
	    to select the recording which is meant.
*/
typedef CARD32 DispRecCtrl_RecordingSession;
/* invalid recording ID: */
#define DispRecCtrl_RecordingSession_None 0

typedef CARD32 DispRecCtrl_GUI;
/* invalid recording ID: */
#define DispRecCtrl_GUI_None 0

enum DispRecCtrl_DirMode
{
	DispRecCtrl_DirMode_CreateDir,
	DispRecCtrl_DirMode_CreateDirWhichHasToBeNew,
	DispRecCtrl_DirMode_DirMustAlreadyExist
};


/* server must also have acess to client-functions for recursively applying DeepSync */
/* #ifndef _DispRecCtrl_SERVER_ */

/*********/
/* Events */
/*********/

/*** Events for recording management ***/
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype */
} XDispRecCtrl_NotifyEvent;

/*** Subtypes of XDispRecCtrl_NotifyEvent ***/

/* notification that the recording has started */
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyStarted) */
    DispRecCtrl_RecordingSession recordingSession;
} XDispRecCtrl_NotifyStartedEvent;

/* notification that the recording has not started */ 
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyNotStarted) */
    DispRecCtrl_RecordingSession recordingSession;
    unsigned int reason;
    /* for some reasons there is additional data, stored in reasonArg1, ...
       Here's a list of the semantics:

       reason DispRecCtrl_Reason_AlreadyRecording:
		reasonArg1: 
			the session-id of the session which is actively recording 
    */
    unsigned long reasonArg1;	
} XDispRecCtrl_NotifyNotStartedEvent;


/* notification that the recording has successfully switched to another directory */
/* please note that this notification does not mean that the old directory is finished.
   Some background processes may still write data to the old directory. */
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifySwitchedDir) */
    DispRecCtrl_RecordingSession recordingSession;
} XDispRecCtrl_NotifySwitchedDirEvent;

/* notification that switching to another directory did not work */ 
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyNotSwitchedDir) */
    DispRecCtrl_RecordingSession recordingSession;
    unsigned int reason;
    Bool stillRecordingToOldDir; /* if True, it is still recording to the old directory */
				 /* if False, recording has stopped */
} XDispRecCtrl_NotifyNotSwitchedDirEvent;



/* notification that the recording has finished */
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyFinished) */
    DispRecCtrl_RecordingSession recordingSession;
    unsigned int reason;
} XDispRecCtrl_NotifyFinishedEvent;

/* notification that the recording has not finished despite a request to do so */
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyFinished) */
    DispRecCtrl_RecordingSession recordingSession;
    unsigned int reason;
} XDispRecCtrl_NotifyNotFinishedEvent;

/* notification that the recording-session has been deleted */
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyRecordingSessionDeleted) */
    DispRecCtrl_RecordingSession recordingSession;
    unsigned int reason;
} XDispRecCtrl_NotifyRecordingSessionDeletedEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIStarted) */
    DispRecCtrl_GUI gui;
} XDispRecCtrl_NotifyGUIStartedEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUINotStarted) */
    DispRecCtrl_GUI gui;
    unsigned int reason;
} XDispRecCtrl_NotifyGUINotStartedEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIExited) */
    DispRecCtrl_GUI gui;
    unsigned int reason;
} XDispRecCtrl_NotifyGUIExitedEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIRestarting) */
    DispRecCtrl_GUI gui;
    unsigned int reason;
} XDispRecCtrl_NotifyGUIRestartingEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIRestarted) */
    DispRecCtrl_GUI gui;
} XDispRecCtrl_NotifyGUIRestartedEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUINotRestarted) */
    DispRecCtrl_GUI gui;
    unsigned int reason;
} XDispRecCtrl_NotifyGUINotRestartedEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIVisible) */
    DispRecCtrl_GUI gui;
    int screen; 
    int x; 
    int y;
} XDispRecCtrl_NotifyGUIVisibleEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIHidden) */
    DispRecCtrl_GUI gui;
} XDispRecCtrl_NotifyGUIHiddenEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIControlsRecordingSession) */
    DispRecCtrl_GUI gui;
    DispRecCtrl_RecordingSession recordingSession;
} XDispRecCtrl_NotifyGUIControlsRecordingSessionEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUILeavesRecordingSession) */
    DispRecCtrl_GUI gui;
    DispRecCtrl_RecordingSession recordingSession;
} XDispRecCtrl_NotifyGUILeavesRecordingSessionEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_Notify) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_NotifyGUIHandleDeleted) */
    DispRecCtrl_GUI gui;
} XDispRecCtrl_NotifyGUIHandleDeletedEvent;


/*** Events for user-color management ***/

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_UserColor) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype */
} XDispRecCtrl_UserColorEvent;

/*** Subtypes of XDispRecCtrl_UserColorEvent ***/

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_UserColor) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_UserColor_ChangePixel) */
    DispRecCtrl_UserColorID user_color_id; 
    unsigned long oldpixel;
    XColor provided_color;
} XDispRecCtrl_UserColor_ChangePixelEvent;

typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_UserColor) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_UserColor_Delete) */
    DispRecCtrl_UserColorID user_color_id; 
} XDispRecCtrl_UserColor_DeleteEvent;

/*** Events for sound recording ***/
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_DRSoundRec) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype */
    DispRecCtrl_RecordingSession recordingSession;
} XDispRecCtrl_DRSoundRecEvent;

/*** Subtypes of XDispRecCtrl_DRSoundRecEvent ***/


/* recording level.
    This event delivers some statistical data of recent audio data recorded,
    This is useful for driving a recording-level display.
    These events are generated quite frequently.
    (should the frequence be configurable via DispRecCtrlExt?)
     */
typedef struct {
    int	type;		    /* of event (eventBase + DispRecCtrl_DRSoundRec) */
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* true if this came frome a SendEvent request */
    Display *display;	    /* Display the event was read from */
    int subtype;           /* event subtype (DispRecCtrl_DRSoundRec_RecLevel) */    
    DispRecCtrl_RecordingSession recordingSession;
    unsigned long fVeff; /* effective voltage (square-root of arithmetic average of square of sample */
    unsigned long fOverdrivePpm; /* how manyoverdrive samples per million samples */
    unsigned long long fVeffSquare;		/* effective voltage squared (omit square-root of formula for Veff)  */
    unsigned long fMaxLevel;	/* maximum level, e.g. 32768 for 16bit (=15 bit level, 1 bit sign) */
} XDispRecCtrl_DRSoundRec_RecLevelEvent;
    


_XFUNCPROTOBEGIN

extern Bool XDispRecCtrl_QueryExtension (
    Display*	/* dpy */,
    int*	/* event_base */,
    int*	/* error_base */
);

extern Status XDispRecCtrl_QueryVersion (
    Display*	/* dpy */,
    int*	/* major_version */,
    int*	/* minor_version */
);


#define XDispRecCtrl_CreateRecordingSession_Flag_TieLifetimeToClient 1
extern DispRecCtrl_RecordingSession 
    XDispRecCtrl_CreateRecordingSession(
	Display*	dpy,
	int flags
);

extern void 
    XDispRecCtrl_DeleteRecordingSession(
	Display*	dpy,
	DispRecCtrl_RecordingSession recordingSession
);


/* returns an array of all recording sessions. 
	*count_return is the number of recording sessions in that array
	the returned array has to be deallocated with free()

	if flags contain XDispRecCtrl_ListRecordingSessions_Flag_Active, 
	then only sessions which actively record, will be returned.
*/
#define XDispRecCtrl_ListRecordingSessions_Flag_Active 1
extern DispRecCtrl_RecordingSession*
    XDispRecCtrl_ListRecordingSessions(
	Display* dpy,
	unsigned long flags,
	int *count_return
);

#ifdef DR_SOME_FUTURE_VERSION
/* TODO: This needs an additional event which signals that this operation is finished.
	Reason: This operation may take longer because it may involve finishing a recording. */
extern void 
    XDispRecCtrl_DeleteAllRecordingSessions(
	Display*	dpy
    );
#endif

/* changed parameters */
extern void XDispRecCtrl_StartRecording(
    Display*	/* dpy */,
    DispRecCtrl_RecordingSession recordingSession,
    const char* dirname,
    enum DispRecCtrl_DirMode dir_mode,
    unsigned int length_recording_time,  /* in seconds, 0 means unlimited */
    Bool endless,
	/* True: endless recording, only the last length seconds are
		guaranteed to be recorded. (some more time may be recorded due to
		the way endless recording is implemented)
	    False: stop recording after max_length seconds
	*/
    Bool doSoundRecording,
    Bool muteSoundRecording,
    Bool finishOtherRecordingsBeforeStart
	/* True: Finish any other recording before starting this recording.
	   	Please note: since finishing takes some time, 
			it may take longer until the start is completed. 
	   False: return a NotStarted event when another recording is already active */ 
);

/* continue recording into a new directory. 
   if dirname is a relative directory, this is interpreted to be relative to
   the current recording directory of this session.

   The directory itself must already exist before issuing this request. 

   Each directory ( the current one and the new one ) will contain
   self-contained recordings.
   Plus, there is the possibility for sequential playback of both recordings
   without interruption. 
   The Soundtrack is only seemlessly joined if no sound-compression is used. */
extern void XDispRecCtrl_SwitchRecordingDir(
    Display*	/* dpy */,
    DispRecCtrl_RecordingSession recordingSession,
    const char* dirname, 
    enum DispRecCtrl_DirMode dir_mode
);

extern void XDispRecCtrl_FinishRecording(
    Display*	dpy,
    DispRecCtrl_RecordingSession recordingSession
);

#ifdef DR_SOME_FUTURE_VERSION
/* TODO: This needs an additional event which signals that this operation is finished.
	Reason: This operation may take longer because it may involve finishing a recording. */
/* finish all recordings, even from other sessions */
extern void XDispRecCtrl_FinishAllRecordings(
    Display*	dpy
);
#endif

extern void XDispRecCtrl_EnableCompression(
    Display*	dpy,
    DispRecCtrl_RecordingSession recordingSession,
    const char* /* compression_method_name */,
    const char* /* compression_command */
);

extern void XDispRecCtrl_DisableCompression(
    Display*	dpy,
    DispRecCtrl_RecordingSession recordingSession
);


extern void XDispRecCtrl_SetMaxFilesysUsage(
    Display*	/* dpy */,
    DispRecCtrl_RecordingSession recordingSession,
    unsigned int /* max_filesys_usage_at_start */,  /* in ppm */
    unsigned int /* max_filesys_usage_while_recording */, 
    /* in ppm */
    unsigned int /* max_filesys_usage_tolerance */  /* in Bytes */
);

extern void XDispRecCtrl_DisableMaxFilesysUsage(
    Display*	/* dpy */,
    DispRecCtrl_RecordingSession recordingSession
);

/* TODO: configure really by extra function? should this be a param to StartRecording? */
extern void XDispRecCtrl_SetMotionKeyRatio(
    Display*	/* dpy */,
    DispRecCtrl_RecordingSession recordingSession,
    float motionKeyRatio 
	/* motion-files are approximately motionKeyRatio times as large as key-files */
);



/**** Sound recording config ****/
typedef struct XDispRecCtrl_SoundRecConfig_struct
    *XDispRecCtrl_SoundRecConfig;

/* creates a soundrec config and fills it with default values */    
extern XDispRecCtrl_SoundRecConfig
    XDispRecCtrl_CreateSoundRecConfig(
	VtError *err
    );

/* frees a sound rec config and all data contained in it */
extern void
    XDispRecCtrl_FreeSoundRecConfig( 
	XDispRecCtrl_SoundRecConfig cfg );
    
/* set sampling rate in samples per second */
extern void XDispRecCtrl_SoundRecConfig_SetSamplingRate(
    XDispRecCtrl_SoundRecConfig cfg, int samplingRate);
    
/* set channels */
extern void XDispRecCtrl_SoundRecConfig_SetChannels(
    XDispRecCtrl_SoundRecConfig cfg, int channels);

extern void XDispRecCtrl_SoundRecConfig_SetBitsPerSample(
    XDispRecCtrl_SoundRecConfig cfg, int bitsPerSample);

/* set output bitrate of the compression program in bits/second.
 This is just a hint for the compression and need not be followed. */
extern void XDispRecCtrl_SoundRecConfig_SetOutputBitrate(
    XDispRecCtrl_SoundRecConfig cfg, int outputBitrate);
    
/* set the update rate for reclevel events.
    update rate is in updates per 1000 seconds (milliHertz),
    i.e. 500 means an update every two seconds,
    or 10000 means an update every 100 milliseconds 
*/
extern void XDispRecCtrl_SoundRecConfig_SetRecLevelUpdateRate(
    XDispRecCtrl_SoundRecConfig cfg, int recLevelUpdateRate);
    
extern void XDispRecCtrl_SoundRecConfig_SetDeviceName(
    XDispRecCtrl_SoundRecConfig cfg, 
    const char* deviceName,
    VtError *err);

extern void XDispRecCtrl_SoundRecConfig_SetSoundApiName(
    XDispRecCtrl_SoundRecConfig cfg, 
    const char* soundApiName,
    VtError *err);

extern void XDispRecCtrl_SoundRecConfig_SetCompressionMethod(
    XDispRecCtrl_SoundRecConfig cfg, 
    const char* compressionMethod,
    VtError *err);

extern void XDispRecCtrl_SoundRecConfig_SetCompressionProgram(
    XDispRecCtrl_SoundRecConfig cfg,
    const char* compressionProgram,
    VtError *err);
    
extern void XDispRecCtrl_SoundRecConfig_SetFadeOutTimeBeforePause( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff value,
    VtError *err
);

extern void XDispRecCtrl_SoundRecConfig_SetFadeInTimeAfterResume( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff value,
    VtError *err
);

extern void XDispRecCtrl_SoundRecConfig_SetSplitSoundFiles( 
    XDispRecCtrl_SoundRecConfig cfg,
    Bool splitSoundFiles,
    VtError *err
);

extern void XDispRecCtrl_SoundRecConfig_SetTimeBetweenSplits_ControlFile( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff timeBetweenSplits_ControlFile,
    VtError *err
);

extern void XDispRecCtrl_SoundRecConfig_SetTimeBetweenSplits_DataFile( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff timeBetweenSplits_DataFile,
    VtError *err
);
    
/* send the sound recording config to the server */
extern void XDispRecCtrl_SetSoundRecordingConfig(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession,
    XDispRecCtrl_SoundRecConfig cfg);

/**** state ****/
/* returns true if there is a recording active in the given recordingSession */
extern Bool XDispRecCtrl_Active(
    Display*	/* dpy */,
    DispRecCtrl_RecordingSession recordingSession
);


/* changed parameters */
extern void XDispRecCtrl_GetCurrentFileInfo(
    Display*	/* dpy */,
    DispRecCtrl_RecordingSession recordingSession,
    Bool* /* active */,
    char** /* filename */
);

extern void XDispRecCtrl_GetBaseDisplay(
    Display*	/* dpy */,
    char** /* display_name */
);

/**** Event config (for XDispRecCtrl events ) ****/
extern void XDispRecCtrl_SelectInput(
    Display*	/* dpy */,
    unsigned long   /* event_mask */
);

extern unsigned long XDispRecCtrl_InputSelected(
    Display*	/* dpy */
);


/**** utilities for special things,
    like having the record&play dialog displayed during playback ****/
    
void XDispRecCtrl_ReparentToBaseWindow(
    Display* dpy,
    Window window, 
    Window appwindow, /* WindowID in demorec-proxy. The window which
			should be used with ConfigureWindow requests.
			Different from window if window is provided by the
			windowmanager and appwindow is the main window
			of the application managed by the window manager */
    Window base_window, /* WindowID in the base-Display of 
			   demorec-proxy to which the window should be reparented */
    Colormap base_colormap, /* ColormapID of the Colormap
			       which should be used via the UserColor
			       Mechanism. If None, then window keeps
			       its current colormap. */
    int x_offset, int y_offset,
    Bool coordsAreOffsets,
    Bool doDepReparenting
    );

void XDispRecCtrl_UnreparentWindow(
    Display* dpy,
    Window window
    );


Colormap XDispRecCtrl_CreateBaseColormap(
    Display* dpy, /* The demorec-proxy-Display */
    Window base_window, /* Window-ID in the basedisplay
			   of the demorec-proxy which provides dpy */
    Visual* base_visual,
    int alloc
);
    
void XDispRecCtrl_FreeBaseColormap(
    Display* dpy, /* The demorec-proxy-Display */
    Colormap base_colormap /* Colormap-ID in the basedisplay */
);

void XDispRecCtrl_BaseColormap_StoreColors(
    Display* dpy, /* The demorec-proxy-Display */
    Colormap base_colormap, /* Colormap-ID in the basedisplay */
    const XColor *colors, int ncolors
);


DispRecCtrl_UserColorID 
XDispRecCtrl_AllocateUserColorInBaseColormap(
    Display* dpy, /* The demorec-proxy-Display */
    Colormap base_colormap, /* Colormap-ID in the basedisplay */
    const XColor* user_color, 
    XColor* provided_color /* Output parameter: The color which is
			      provided for approximating user_color */
);

void XDispRecCtrl_DeleteUserColor(
    Display* dpy, /* The demorec-proxy-Display */
    DispRecCtrl_UserColorID user_color_id
);

void XDispRecCtrl_AuthorizeReuseOfUserColorID(
    Display* dpy, /* The demorec-proxy-Display */
    DispRecCtrl_UserColorID user_color_id
);

void XDispRecCtrl_ChangeBaseColormapOfReparentedWindow(
    Display* dpy, /* The demorec-proxy-Display */
    Window base_window, /* Window-ID in the basedisplay */
    Colormap base_colormap /* Colormap-ID in the basedisplay */
);

void XDispRecCtrl_RedrawEverything(
    Display* dpy /* The demorec-proxy-Display */
);

void XDispRecCtrl_InhibitColormapInstall(
    Display* dpy /* The demorec-proxy-Display */
);

void XDispRecCtrl_EnableColormapInstall (
    Display* dpy /* The demorec-proxy-Display */
);

    

/* get license info in a form suitable to be displayed to the user,
    the returned license-info has to be freed by the client of this extension.
    Returns NULL on communication error. */
char* XDispRecCtrl_GetLicenseInfo( Display *dpy );


/* causes the demorec-proxy to synchronize with its base-display.
    If the base-display is itself provided by an demorec-proxy, then call
    XDispRecCtrl_DeepSync for it, too so synchronization is propagated
    through a chain of demorec-proxys */
void XDispRecCtrl_DeepSync( Display *dpy );

/********************************************************************************/
/* Controlling the GUI */
/********************************************************************************/

DispRecCtrl_GUI XDispRecCtrl_CreateGUIHandle(
    Display *dpy
);

/* to be called only from the GUI if it is running.
    if the GUI is not running: anybody can call it  */
void XDispRecCtrl_DeleteGUIHandle(
    Display *dpy,
    DispRecCtrl_GUI gui
);

/* start a GUI and set the keepalive property 
    the GUI is started invisible. 
    Please call XDispRecCtrl_ShowGUI to make it visible. */
void XDispRecCtrl_StartGUI(
    Display *dpy,
    DispRecCtrl_GUI gui,
    uid_t user,
    gid_t group,
    Bool keepalive
);

/* set the keepalive property of a DispRecCtrl_GUI - handle.
    keepalive = True means that the GUI associated with that handle
    is automatically restarted if it terminates. */
void XDispRecCtrl_SetGUIKeepalive(
    Display *dpy,
    DispRecCtrl_GUI gui,
    Bool keepalive
);


/* if a GUI is already running, then returns the DispRecCtrl_GUI - handle
    for that GUI and sets the keepalive property.
    Otherwise, a new DispRecCtrl_GUI - handle is created,
    a new GUI started on that handle */
DispRecCtrl_GUI XDispRecCtrl_StartGUIIfNotRunning(
    Display *dpy,
    uid_t user,
    gid_t group,
    Bool keepalive
);

/* gracefully quit the given GUI: implicitely disables keepalive */
void XDispRecCtrl_QuitGUI(
    Display *dpy,
    DispRecCtrl_GUI gui
);

/* kill the given GUI: implicitely disables keepalive */
void XDispRecCtrl_KillGUI(
    Display *dpy,
    DispRecCtrl_GUI gui
);

void XDispRecCtrl_ShowGUI(
    Display *dpy,
    DispRecCtrl_GUI gui,
    int screen, int x, int y
);

void XDispRecCtrl_HideGUI(
    Display *dpy,
    DispRecCtrl_GUI gui
);

Bool XDispRecCtrl_GUIRunning(
    Display *dpy,
    DispRecCtrl_GUI gui
);

Bool XDispRecCtrl_GUIVisible(
    Display *dpy,
    DispRecCtrl_GUI gui
);

DispRecCtrl_RecordingSession
    XDispRecCtrl_GetRecordingSessionOfGUI(
	Display *dpy,
	DispRecCtrl_GUI gui
    );
    


/********************************************************************************/
/* Utils */
/********************************************************************************/

const char* XDispRecCtrl_ReasonToString( int reason );

_XFUNCPROTOEND


/* #endif _DispRecCtrl_SERVER_ */ 

#endif /* _DispRecCtrl_H_ */

