/****************************************
Filename: lib/DispRecXtAPI/DispRecXtAPI.h
Description: API for DisplayRecorder/DemoRecorder integrated into Xt
Author: Christian Linhart
Copyright: (C)2006 Christian Linhart
****************************************/

#ifndef __DispRecXtAPI_h__
#define __DispRecXtAPI_h__

#include <X11/Intrinsic.h>

#if 0
#include "DispRec/error.h"
#include "DispRec/DispRecCtrlExtension.h"
#else
#include "DispRec/DRError.h"
#include "DispRec/DRMem.h"
#include "DispRec/DispRecCtrlExtension.h"
#endif

/*****************************************
******************************************
* Data types  
******************************************
*****************************************/

/*****************************************
* Main types
*****************************************/

/**
* Object which identifies a connection to a DisplayRecorder/DemoRecorder X11-Proxy.
*/
typedef struct DispRecXtAPI_Server_struct* DispRecXtAPI_Server;
#define DispRecXtAPI_Server_None ((DispRecXtAPI_Server)NULL)

/**
* A recording session is an object which can be used to condigure, start and stop a recording.
*/
typedef struct DispRecXtAPI_RecordingSession_struct* DispRecXtAPI_RecordingSession;
#define DispRecXtAPI_RecordingSession_None ((DispRecXtAPI_RecordingSession)NULL)

/**
* DispRecXtAPI_GUI is a handle for communicating with a DemoRecorder GUI
*/
typedef struct DispRecXtAPI_GUI_struct* DispRecXtAPI_GUI;
#define DispRecXtAPI_GUI_None ((DispRecXtAPI_GUI)NULL)




/*****************************************
* enums ( with functions for converting an enum value to a string )
*****************************************/

/***** for  recording sessions *****/
enum DispRecXtAPI_RecordingNotStartedReason{
    DispRecXtAPI_RecordingNotStartedReason_AlreadyRecording = DispRecCtrl_Reason_AlreadyRecording
    , DispRecXtAPI_RecordingNotStartedReason_CannotOpenFile = DispRecCtrl_Reason_CannotOpenFile
    , DispRecXtAPI_RecordingNotStartedReason_MaxLengthReached = DispRecCtrl_Reason_MaxLengthReached
    , DispRecXtAPI_RecordingNotStartedReason_MaxSizeReached = DispRecCtrl_Reason_MaxSizeReached
    , DispRecXtAPI_RecordingNotStartedReason_MaxFsUsageReached = DispRecCtrl_Reason_MaxFsUsageReached
    , DispRecXtAPI_RecordingNotStartedReason_BadValue = DispRecCtrl_Reason_BadValue
    , DispRecXtAPI_RecordingNotStartedReason_WriteError = DispRecCtrl_Reason_WriteError
    , DispRecXtAPI_RecordingNotStartedReason_FileDisappeared = DispRecCtrl_Reason_FileDisappeared
    , DispRecXtAPI_RecordingNotStartedReason_TotalMaxLengthReached = DispRecCtrl_Reason_TotalMaxLengthReached
    , DispRecXtAPI_RecordingNotStartedReason_NoLicense = DispRecCtrl_Reason_NoLicense
    , DispRecXtAPI_RecordingNotStartedReason_SoundDeviceReadError = DispRecCtrl_Reason_SoundDeviceReadError
    , DispRecXtAPI_RecordingNotStartedReason_SoundDeviceConfigError = DispRecCtrl_Reason_SoundDeviceConfigError
    , DispRecXtAPI_RecordingNotStartedReason_SoundDeviceOpenError = DispRecCtrl_Reason_SoundDeviceOpenError
    , DispRecXtAPI_RecordingNotStartedReason_SoundDeviceMissingCapabilities = DispRecCtrl_Reason_SoundDeviceMissingCapabilities
    , DispRecXtAPI_RecordingNotStartedReason_ChildprocessDiedUnexpectedly = DispRecCtrl_Reason_ChildprocessDiedUnexpectedly
    , DispRecXtAPI_RecordingNotStartedReason_CannotStartSoundCompression = DispRecCtrl_Reason_CannotStartSoundCompression
    , DispRecXtAPI_RecordingNotStartedReason_NoSoundSupport = DispRecCtrl_Reason_NoSoundSupport
};
const char* DispRecXtAPI_RecordingNotStartedReason_ToString( 
    enum DispRecXtAPI_RecordingNotStartedReason
);

enum DispRecXtAPI_RecordingFinishedReason{
    DispRecXtAPI_RecordingFinishedReason_ByClientRequest = DispRecCtrl_Reason_ByClientRequest
    , DispRecXtAPI_RecordingFinishedReason_CannotOpenFile = DispRecCtrl_Reason_CannotOpenFile
    , DispRecXtAPI_RecordingFinishedReason_MaxLengthReached = DispRecCtrl_Reason_MaxLengthReached
    , DispRecXtAPI_RecordingFinishedReason_MaxSizeReached = DispRecCtrl_Reason_MaxSizeReached
    , DispRecXtAPI_RecordingFinishedReason_MaxFsUsageReached = DispRecCtrl_Reason_MaxFsUsageReached
    , DispRecXtAPI_RecordingFinishedReason_BadValue = DispRecCtrl_Reason_BadValue
    , DispRecXtAPI_RecordingFinishedReason_WriteError = DispRecCtrl_Reason_WriteError
    , DispRecXtAPI_RecordingFinishedReason_FileDisappeared = DispRecCtrl_Reason_FileDisappeared
    , DispRecXtAPI_RecordingFinishedReason_TotalMaxLengthReached = DispRecCtrl_Reason_TotalMaxLengthReached
    , DispRecXtAPI_RecordingFinishedReason_NoLicense = DispRecCtrl_Reason_NoLicense
    , DispRecXtAPI_RecordingFinishedReason_SoundDeviceReadError = DispRecCtrl_Reason_SoundDeviceReadError
    , DispRecXtAPI_RecordingFinishedReason_SoundDeviceConfigError = DispRecCtrl_Reason_SoundDeviceConfigError
    , DispRecXtAPI_RecordingFinishedReason_SoundDeviceOpenError = DispRecCtrl_Reason_SoundDeviceOpenError
    , DispRecXtAPI_RecordingFinishedReason_SoundDeviceMissingCapabilities = DispRecCtrl_Reason_SoundDeviceMissingCapabilities
    , DispRecXtAPI_RecordingFinishedReason_ChildprocessDiedUnexpectedly = DispRecCtrl_Reason_ChildprocessDiedUnexpectedly
    , DispRecXtAPI_RecordingFinishedReason_CannotStartSoundCompression = DispRecCtrl_Reason_CannotStartSoundCompression
    , DispRecXtAPI_RecordingFinishedReason_NoSoundSupport = DispRecCtrl_Reason_NoSoundSupport
    , DispRecXtAPI_RecordingFinishedReason_NotRecording = DispRecCtrl_Reason_NotRecording
};
const char* DispRecXtAPI_RecordingFinishedReason_ToString( 
    enum DispRecXtAPI_RecordingFinishedReason
);

enum DispRecXtAPI_RecordingNotFinishedReason{
    DispRecXtAPI_RecordingNotFinishedReason_NotRecording = DispRecCtrl_Reason_NotRecording
};
const char* DispRecXtAPI_RecordingNotFinishedReason_ToString( 
    enum DispRecXtAPI_RecordingNotFinishedReason
);

enum DispRecXtAPI_RecordingSessionDeletedReason{
    DispRecXtAPI_RecordingSessionDeletedReason_ByClientRequest 
	= DispRecCtrl_Reason_ByClientRequest
};
const char* DispRecXtAPI_RecordingSessionDeletedReason_ToString( 
    enum DispRecXtAPI_RecordingSessionDeletedReason
);

/**** for the GUI ****/
enum DispRecXtAPI_GUINotStartedReason{
    DispRecXtAPI_GUINotStartedReason_CannotOpenFile = DispRecCtrl_Reason_CannotOpenFile
    , DispRecXtAPI_GUINotStartedReason_BadValue = DispRecCtrl_Reason_BadValue
    , DispRecXtAPI_GUINotStartedReason_NoLicense = DispRecCtrl_Reason_NoLicense
    
    /* only in reply to DispRecXtAPI_StartGUIIfNotRunning: */
    , DispRecXtAPI_GUINotStartedReason_AlreadyRunning = DispRecCtrl_Reason_AlreadyRunning
};
const char* DispRecXtAPI_GUINotStartedReason_ToString(
    enum DispRecXtAPI_GUINotStartedReason
);

enum DispRecXtAPI_GUIExitedReason{
    DispRecXtAPI_GUIExitedReason_ByClientRequest = DispRecCtrl_Reason_ByClientRequest
    , DispRecXtAPI_GUIExitedReason_NoLicense = DispRecCtrl_Reason_NoLicense
    , DispRecXtAPI_GUIExitedReason_Killed = DispRecCtrl_Reason_Killed
};
const char* DispRecXtAPI_GUIExitedReason_ToString(
    enum DispRecXtAPI_GUIExitedReason
);

enum DispRecXtAPI_GUIRestartingReason{
    DispRecXtAPI_GUIRestartingReason_ProcessTerminated = DispRecCtrl_Reason_ProcessTerminated
};
const char* DispRecXtAPI_GUIRestartingReason_ToString(
    enum DispRecXtAPI_GUIRestartingReason
);

enum DispRecXtAPI_GUINotRestartedReason{
    DispRecXtAPI_GUINotRestartedReason_CannotOpenFile = DispRecCtrl_Reason_CannotOpenFile
    , DispRecXtAPI_GUINotRestartedReason_BadValue = DispRecCtrl_Reason_BadValue
    , DispRecXtAPI_GUINotRestartedReason_NoLicense = DispRecCtrl_Reason_NoLicense
    , DispRecXtAPI_GUINotRestartedReason_AlreadyRunning = DispRecCtrl_Reason_AlreadyRunning 
};
const char* DispRecXtAPI_GUINotRestartedReason_ToString(
    enum DispRecXtAPI_GUINotRestartedReason
);

/*****************************************
* Callback functions and handles along with aux data
*****************************************/

/**** recording sessions callbacks ****/
typedef struct DispRecXtAPI_RecordingStarted_CallbackHandle_struct
    *DispRecXtAPI_RecordingStarted_CallbackHandle;
typedef void 
    (*DispRecXtAPI_RecordingStarted_CallbackProc)(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingStarted_CallbackHandle handle,
	void *client_data
    );
    
typedef struct DispRecXtAPI_RecordingNotStarted_CallbackHandle_struct
    *DispRecXtAPI_RecordingNotStarted_CallbackHandle;
typedef void 
    (*DispRecXtAPI_RecordingNotStarted_CallbackProc)(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingNotStarted_CallbackHandle handle,
	enum DispRecXtAPI_RecordingNotStartedReason,
	void *client_data
    );
    
typedef struct DispRecXtAPI_RecordingFinished_CallbackHandle_struct
    *DispRecXtAPI_RecordingFinished_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_RecordingFinished_CallbackProc)(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingFinished_CallbackHandle handle,
	enum DispRecXtAPI_RecordingFinishedReason,
	void *client_data
    );

typedef struct DispRecXtAPI_RecordingNotFinished_CallbackHandle_struct
    *DispRecXtAPI_RecordingNotFinished_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_RecordingNotFinished_CallbackProc)(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingNotFinished_CallbackHandle handle,
	enum DispRecXtAPI_RecordingNotFinishedReason,
	void *client_data
    );

typedef struct DispRecXtAPI_RecLevel_struct{
    unsigned long fVeff; /* effective voltage (square-root of arithmetic average of square of sample */
    unsigned long fOverdrivePpm; /* how manyoverdrive samples per million samples */
    unsigned long long fVeffSquare;		/* effective voltage squared (omit square-root of formula for Veff)  */
    unsigned long fMaxLevel;	/* maximum level, e.g. 32768 for 16bit (=15 bit level, 1 bit sign) */
} DispRecXtAPI_RecLevel;
    
typedef struct DispRecXtAPI_RecLevel_CallbackHandle_struct
    *DispRecXtAPI_RecLevel_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_RecLevel_CallbackProc)(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecLevel_CallbackHandle handle,
	DispRecXtAPI_RecLevel reclevel,
	void *client_data
    );
    
typedef struct DispRecXtAPI_RecordingSessionDeleted_CallbackHandle_struct
    *DispRecXtAPI_RecordingSessionDeleted_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_RecordingSessionDeleted_CallbackProc)(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingSessionDeleted_CallbackHandle handle,
	enum DispRecXtAPI_RecordingSessionDeletedReason,
	void *client_data
    );
    

/**** GUI callbacks ****/

/** Start and exit of GUI program **/

typedef struct DispRecXtAPI_GUIStarted_CallbackHandle_struct
    *DispRecXtAPI_GUIStarted_CallbackHandle;
typedef void 
    (*DispRecXtAPI_GUIStarted_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIStarted_CallbackHandle handle,
	void *client_data
    );
    
typedef struct DispRecXtAPI_GUINotStarted_CallbackHandle_struct
    *DispRecXtAPI_GUINotStarted_CallbackHandle;
typedef void 
    (*DispRecXtAPI_GUINotStarted_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUINotStarted_CallbackHandle handle,
	enum DispRecXtAPI_GUINotStartedReason reason,
	void *client_data
    );
    
typedef struct DispRecXtAPI_GUIExited_CallbackHandle_struct
    *DispRecXtAPI_GUIExited_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUIExited_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIExited_CallbackHandle handle,
	enum DispRecXtAPI_GUIExitedReason reason,
	void *client_data
    );
    
/* When a GUI exists and keepAlive is activated, then it is automatically restarted.
    In this case, no GUIExited event is created but a GUIRestarting-event.
    The GUIRestarting-event indicates that the startup-process of the new GUI 
    has been started.
    When the startup-process succeeds, a GUIRestarted-event is sent.
    When the startup-process fails, a GUINotRestarted-event is sent.
*/
 
typedef struct DispRecXtAPI_GUIRestarting_CallbackHandle_struct
    *DispRecXtAPI_GUIRestarting_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUIRestarting_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIRestarting_CallbackHandle handle,
	enum DispRecXtAPI_GUIRestartingReason reason,
	void *client_data
    );
    
typedef struct DispRecXtAPI_GUIRestarted_CallbackHandle_struct
    *DispRecXtAPI_GUIRestarted_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUIRestarted_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIRestarted_CallbackHandle handle,
	void *client_data
    );
    
typedef struct DispRecXtAPI_GUINotRestarted_CallbackHandle_struct
    *DispRecXtAPI_GUINotRestarted_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUINotRestarted_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUINotRestarted_CallbackHandle handle,
	enum DispRecXtAPI_GUINotRestartedReason reason,
	void *client_data
    );

/** Visible or Hidden status of GUI (program keeps running) **/

/* called when the GUI becomes visible after being hidden or
    when the gui is moved from one screen to another */
typedef struct DispRecXtAPI_GUIVisible_CallbackHandle_struct
    *DispRecXtAPI_GUIVisible_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUIVisible_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIVisible_CallbackHandle handle,
	int screen, int x, int y,
	void *client_data
    );

typedef struct DispRecXtAPI_GUIHidden_CallbackHandle_struct
    *DispRecXtAPI_GUIHidden_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUIHidden_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIHidden_CallbackHandle handle,
	void *client_data
    );

/** RecordingSession controlled by GUI 
    ( Please note: a recording session does not mean that a recording is active.
	for this, please register appropriate callbacks on the recording session **/

typedef struct DispRecXtAPI_GUIControlsRecordingSession_CallbackHandle_struct
    *DispRecXtAPI_GUIControlsRecordingSession_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUIControlsRecordingSession_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIControlsRecordingSession_CallbackHandle handle,
	DispRecXtAPI_RecordingSession session,
	void *client_data
    );

typedef struct DispRecXtAPI_GUILeavesRecordingSession_CallbackHandle_struct
    *DispRecXtAPI_GUILeavesRecordingSession_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUILeavesRecordingSession_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUILeavesRecordingSession_CallbackHandle handle,
	DispRecXtAPI_RecordingSession session,
	void *client_data
    );

typedef struct DispRecXtAPI_GUIHandleDeleted_CallbackHandle_struct
    *DispRecXtAPI_GUIHandleDeleted_CallbackHandle;    
typedef void 
    (*DispRecXtAPI_GUIHandleDeleted_CallbackProc)(
	DispRecXtAPI_GUI gui,
	DispRecXtAPI_GUIHandleDeleted_CallbackHandle handle,
	void *client_data
    );

    
/*****************************************
******************************************
* Functions   
******************************************
*****************************************/

/* 
* Please note: many functions will use VtError for signalling error conditions/
* To use VtError, please proceed as follows:
*
*    1. Declare the error-object called "err"
* 	DECLERR;
*
*    2. Call a function with error object
*	DispRecXtAPI_Init( app_context, err )
*
*    3. Check the error object named "err" for an error condition
*	HAVE_ERROR()
*
*
* Example:
*
*	int foo() 
*	{
*	    DECLERR;
*	    DispRecXtAPI_Init( app_context, err );
*	    if ( HAVE_ERROR() ) {
*		fprintf(stderr, "error during init of DispRecXtAPI. Aborting\n");
*		exit(1);
*	    }
*	}
*
*/

/*****************************************
Auxiliary functions
*****************************************/

/* memory management: Free arrays returned by API calls */
void DispRecXtAPI_FreeArray( void *array );

/*****************************************
Init and Shutdown of API and connection to server 
*****************************************/

/**
* init this API and attach to the given app_context
*/
void DispRecXtAPI_Init( XtAppContext app_context, VtError *err );

/**
* shutdown this API
*/
void DispRecXtAPI_Shutdown( VtError *err );

/**
* connect to a DisplayRecorder/DemoRecorder X11-Proxy through the given Display.
* On error, DispRecXtAPI_Server_None is returned and the error-object set accordingly 
*/
DispRecXtAPI_Server DispRecXtAPI_Connect( Display* dpy, VtError *err );

/**
* Disconnect the server connection
*/
void DispRecXtAPI_Disconnect( DispRecXtAPI_Server, VtError *err );


/*****************************************
Recording Sessions 
*****************************************/

/**
* Create a new recording session
*/
DispRecXtAPI_RecordingSession 
    DispRecXtAPI_CreateRecordingSession( 
	DispRecXtAPI_Server server, int flags, VtError *err 
    );


/**
* Delete a recording session
*/
void 
    DispRecXtAPI_DeleteRecordingSession( 
	DispRecXtAPI_RecordingSession session, VtError *err 
    );
    
/**
* Returns a list of all recording sessions of the given server.
* This includes recording sessions created by other clients.
*
* The list is returned as a pointer to an array of length *count.
* The size of the list is returned in "count".
*
* The list has to be deallocated by the caller using DispRecXtAPI_FreeArray();
*/
DispRecXtAPI_RecordingSession*
    DispRecXtAPI_GetAllRecordingSessions(
	DispRecXtAPI_Server server, int* count, VtError *err);
	

/*****  Configure the recording session *****/

/* TODO */


/***** Start and Stop recording *****/

/**
* Sends a request to start recording to the given directory "dest_dir"
*
* Please note: 
*	This is an asynchronous request which means that
 *	an error-code OK does not confirm actual start of the recording
*	For this, please register and listen to the appropriate callbacks.
*/
void DispRecXtAPI_StartRecording(
    DispRecXtAPI_RecordingSession session,
    const char* dest_dir,
    enum DispRecCtrl_DirMode dir_mode,
    VtError *err );
    
void DispRecXtAPI_FinishRecording(
    DispRecXtAPI_RecordingSession session,
    VtError *err );
    

/***** Query current state *****/

/**
* return True when the given session is currently recording 
*/
Bool DispRecXtAPI_Active(
    DispRecXtAPI_RecordingSession session,
    VtError *err );

/**
* return True if there is any recording session which is currently recording
*/
Bool DispRecXtAPI_AnyActive(
    DispRecXtAPI_Server server,
    VtError *err );

/**
* Returns a list of all recording sessions of the given server which currently do recording
* This includes recording sessions created by other clients.
*
* The list is returned as a pointer to an array of length *count.
* The size of the list is returned in "count".
*
* As of now, not more than one session may be active at any time.
* But this may change in the future.
*
* The list has to be deallocated by the caller using DispRecXtAPI_FreeArray();
*/
DispRecXtAPI_RecordingSession*
    DispRecXtAPI_GetAllActiveRecordingSessions(
	DispRecXtAPI_Server server, int* count, VtError *err);

/**
* If the recording session is controlled by a GUI, then return a handle to that GUI,
* otherwise return the DispRecXtAPI_GUI_None handle.
*/
DispRecXtAPI_GUI 
    DispRecXtAPI_GetGUIOfRecordingSession(
	DispRecXtAPI_RecordingSession session,
	VtError *err);

/***** Register callbacks *****/

/**
* Hint for all Recording-sessions callback:
* 	if the recording session is DispRecXtAPI_RecordingSession_None,
*	then this means that the callback is registered for all recording sessions of all servers.
*/


DispRecXtAPI_RecordingStarted_CallbackHandle
    DispRecXtAPI_Add_RecordingStarted_Callback(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingStarted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_RecordingStarted_Callback(
    DispRecXtAPI_RecordingStarted_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_RecordingNotStarted_CallbackHandle
    DispRecXtAPI_Add_RecordingNotStarted_Callback(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingNotStarted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_RecordingNotStarted_Callback(
    DispRecXtAPI_RecordingNotStarted_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_RecordingFinished_CallbackHandle
    DispRecXtAPI_Add_RecordingFinished_Callback(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingFinished_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_RecordingFinished_Callback(
    DispRecXtAPI_RecordingFinished_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_RecordingNotFinished_CallbackHandle
    DispRecXtAPI_Add_RecordingNotFinished_Callback(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingNotFinished_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_RecordingNotFinished_Callback(
    DispRecXtAPI_RecordingNotFinished_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_RecLevel_CallbackHandle
    DispRecXtAPI_Add_RecLevel_Callback(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecLevel_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_RecLevel_Callback(
    DispRecXtAPI_RecLevel_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_RecordingSessionDeleted_CallbackHandle
    DispRecXtAPI_Add_RecordingSessionDeleted_Callback(
	DispRecXtAPI_RecordingSession session,
	DispRecXtAPI_RecordingSessionDeleted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_RecordingSessionDeleted_Callback(
    DispRecXtAPI_RecordingSessionDeleted_CallbackHandle handle,
    VtError *err
);

/*****************************************
GUI
*****************************************/

/**
* Create a new GUI handle (GUI not started yet by this function)
*/
DispRecXtAPI_GUI
    DispRecXtAPI_CreateGUIHandle( 
	DispRecXtAPI_Server server, VtError *err 
    );
    
void DispRecXtAPI_DestroyGUIHandle(
    DispRecXtAPI_GUI, VtError *err );
    
/**
* Returns a list of all gui-handles of the given server.
* This includes gui-handles created by other clients.
*
* The list is returned as a pointer to an array of length *count.
* The size of the list is returned in "count".
*
* The list has to be deallocated by the caller using DispRecXtAPI_FreeArray();
*/
DispRecXtAPI_GUI*
    DispRecXtAPI_GetAllGUIs(
	DispRecXtAPI_Server server, int* count, VtError *err);
	
/**** start and kill ****/

/**
* Request starting the GUI
* This request is handeled asynchronously, so be sure to add and listen to the appropriate callbacks
* for getting feedback about success or failure of the starting.
*
* The gui is started as the given user and group,
* This selecting of user and group only works if the DemoRecorder X11-proxy is run as root,
* otherwise user and group are ignored and the gui is run as the same user as the X11-proxy
*
* keepalive = True means that the GUI associated with that handle 
* is automatically restarted if it terminates.
*
* The GUI is started in invisible state. 
* Please call DispRecXtAPI_ShowGUI to make it visible. 
*/
void DispRecXtAPI_StartGUI(
    DispRecXtAPI_GUI gui_handle,
    uid_t user,
    gid_t group,
    Bool keepalive,    
    VtError *err);
    
/**
* Request starting the GUI if no GUI is already running.
* This request is handeled asynchronously, so be sure to add and listen to the appropriate callbacks
* for getting feedback about success or failure of the starting.
*
* if a GUI is already running, then if returns the DispRecXtAPI_GUI - handle of that GUI
* and sets the keepalive-property.
*
* The gui is started as the given user and group,
* This selecting of user and group only works if the DemoRecorder X11-proxy is run as root,
* otherwise user and group are ignored and the gui is run as the same user as the X11-proxy
*
* keepalive = True means that the GUI associated with that handle 
* is automatically restarted if it terminates.
*
* If the GUI is not yet running, it is started in invisible state. 
* Please call DispRecXtAPI_ShowGUI to make it visible. 
*/
DispRecXtAPI_GUI DispRecXtAPI_StartGUIIfNotRunning(
    DispRecXtAPI_Server server,
    uid_t user,
    gid_t group,
    Bool keepalive,    
    VtError *err);
  

/* set the keepalive property of a DispRecCtrl_GUI - handle.
    keepalive = True means that the GUI associated with that handle
    is automatically restarted if it terminates. */
void DispRecXtAPI_SetGUIKeepalive(
    DispRecXtAPI_Server server,
    DispRecXtAPI_GUI gui,
    Bool keepalive,
    VtError *err
);
    

/**
* kill the given gui program
*/
void DispRecXtAPI_KillGUI(
    DispRecXtAPI_GUI gui_handle,
    VtError *err);


/****change visibility of gui ****/
/**
* make the gui visible and show it on the given screen  at the given coordinates
*/
void DispRecXtAPI_ShowGUI(
    DispRecXtAPI_GUI gui_handle,
    int screen,
    int x, int y,
    VtError *err);

void DispRecXtAPI_HideGUI(
    DispRecXtAPI_GUI gui_handle,
    VtError *err);
    
/**** query ****/

/** 
* returns True if there is a GUI program running under this GUI-Handle
*/
Bool DispRecXtAPI_GUIRunning( 
    DispRecXtAPI_GUI gui_handle, VtError *err );

/** 
* returns True if the GUI is visible
*/
Bool DispRecXtAPI_GUIVisible( 
    DispRecXtAPI_GUI gui_handle, VtError *err );

/**
* if the GUI has a recording session, then return it, 
* otherwise return DispRecXtAPI_RecordingSession_None
*/
DispRecXtAPI_RecordingSession
     DispRecXtAPI_GetRecordingSessionOfGUI( 
	DispRecXtAPI_GUI gui_handle, VtError *err
    );
    
/***** Register callbacks *****/
    
/**
* Hint for all GUI callbacks:
* 	if the gui_handle is DispRecXtAPI_GUI_None,
*	then this means that the callback is registered for all GUIs attached to the server.
*/

DispRecXtAPI_GUIStarted_CallbackHandle
    DispRecXtAPI_Add_GUIStarted_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIStarted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIStarted_Callback(
    DispRecXtAPI_GUIStarted_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUINotStarted_CallbackHandle
    DispRecXtAPI_Add_GUINotStarted_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUINotStarted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUINotStarted_Callback(
    DispRecXtAPI_GUINotStarted_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUIExited_CallbackHandle
    DispRecXtAPI_Add_GUIExited_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIExited_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIExited_Callback(
    DispRecXtAPI_GUIExited_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUIRestarting_CallbackHandle
    DispRecXtAPI_Add_GUIRestarting_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIRestarting_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIRestarting_Callback(
    DispRecXtAPI_GUIRestarting_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUIRestarted_CallbackHandle
    DispRecXtAPI_Add_GUIRestarted_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIRestarted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIRestarted_Callback(
    DispRecXtAPI_GUIRestarted_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUINotRestarted_CallbackHandle
    DispRecXtAPI_Add_GUINotRestarted_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUINotRestarted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUINotRestarted_Callback(
    DispRecXtAPI_GUINotRestarted_CallbackHandle handle,
    VtError *err
);


DispRecXtAPI_GUIVisible_CallbackHandle
    DispRecXtAPI_Add_GUIVisible_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIVisible_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIVisible_Callback(
    DispRecXtAPI_GUIVisible_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUIHidden_CallbackHandle
    DispRecXtAPI_Add_GUIHidden_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIHidden_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIHidden_Callback(
    DispRecXtAPI_GUIHidden_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUIControlsRecordingSession_CallbackHandle
    DispRecXtAPI_Add_GUIControlsRecordingSession_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIControlsRecordingSession_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIControlsRecordingSession_Callback(
    DispRecXtAPI_GUIControlsRecordingSession_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUILeavesRecordingSession_CallbackHandle
    DispRecXtAPI_Add_GUILeavesRecordingSession_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUILeavesRecordingSession_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUILeavesRecordingSession_Callback(
    DispRecXtAPI_GUILeavesRecordingSession_CallbackHandle handle,
    VtError *err
);

DispRecXtAPI_GUIHandleDeleted_CallbackHandle
    DispRecXtAPI_Add_GUIHandleDeleted_Callback(
	DispRecXtAPI_GUI gui_handle,
	DispRecXtAPI_GUIHandleDeleted_CallbackProc proc,
	void *client_data,
	VtError *err
    );
void DispRecXtAPI_Remove_GUIHandleDeleted_Callback(
    DispRecXtAPI_GUIHandleDeleted_CallbackHandle handle,
    VtError *err
);

#endif
