/****************************************
Filename: libDispRecCtrlExtension/DispRecCtrlExtensionClient.c
Description: DispRecCtrl-Extension client-side code
Author: Christian Linhart
Copyright: (C)1997-2011 Christian Linhart
****************************************/

/* Based on ???.c which is Copyright (c) 1989  X Consortium */


#define NEED_EVENTS
#define NEED_REPLIES

#include <X11/Xlib.h>
#include <VtUtils.h>
#include <DRMem.h>
#include <DRConfig.h>

#include <X11/Xlibint.h>
#include <X11/Xutil.h>
#if 0
#include "region.h"			/* in Xlib sources */ 
#endif
#include <X11/extensions/Xext.h>			/* in ../include */
#include <X11/extensions/extutil.h>			/* in ../include */


#include "DispRecCtrlExtension_str.h"

static XExtensionInfo _DispRecCtrl_info_data;
static XExtensionInfo *DispRecCtrl_info = &_DispRecCtrl_info_data;
static /* const */ char *DispRecCtrl_extension_name = DispRecCtrl_NAME;

#define DispRecCtrlCheckExtension(dpy,i,val) \
  XextCheckExtension (dpy, i, DispRecCtrl_extension_name, val)
#define DispRecCtrlSimpleCheckExtension(dpy,i) \
  XextSimpleCheckExtension (dpy, i, DispRecCtrl_extension_name)


/*****************************************************************************
 *                                                                           *
 *			   private utility routines                          *
 *                                                                           *
 *****************************************************************************/

static int close_display();
static Bool wire_to_event();
static Status event_to_wire();
static /* const */ XExtensionHooks DispRecCtrl_extension_hooks = {
    NULL,				/* create_gc */
    NULL,				/* copy_gc */
    NULL,				/* flush_gc */
    NULL,				/* free_gc */
    NULL,				/* create_font */
    NULL,				/* free_font */
    close_display,			/* close_display */
    wire_to_event,			/* wire_to_event */
    event_to_wire,			/* event_to_wire */
    NULL,				/* error */
    NULL,				/* error_string */
};

static XEXT_GENERATE_FIND_DISPLAY (find_display, DispRecCtrl_info,
				   DispRecCtrl_extension_name, 
				   &DispRecCtrl_extension_hooks,
				   DispRecCtrl_NumberEvents, NULL)

static XEXT_GENERATE_CLOSE_DISPLAY (close_display, DispRecCtrl_info)


static Bool wire_to_event (dpy, re, event)
    Display *dpy;
    XEvent  *re; /* result */
    xEvent  *event; /* input from the wire */
{
    XExtDisplayInfo *info = find_display (dpy);
    Bool retval=False;

    DispRecCtrlCheckExtension (dpy, info, False);

    switch ((event->u.u.type & 0x7f) - info->codes->first_event) {
    case DispRecCtrl_Notify: {
	XDispRecCtrl_NotifyEvent *se = 
	    (XDispRecCtrl_NotifyEvent *) re;
	xDispRecCtrl_NotifyEvent* sevent = 
	    (xDispRecCtrl_NotifyEvent *) event;
	se->type = sevent->type & 0x7f;
	se->serial = _XSetLastRequestRead(dpy,(xGenericReply *) event);
	se->send_event = (sevent->type & 0x80) != 0;
	se->display = dpy;
	se->subtype = sevent->subtype;

	switch (((xDispRecCtrl_NotifyEvent *)event)->subtype) {	

	case DispRecCtrl_NotifyStarted: {
	    XDispRecCtrl_NotifyStartedEvent *result = 
		(XDispRecCtrl_NotifyStartedEvent *) se;
	    xDispRecCtrl_NotifyStartedEvent* fromwire = 
		(xDispRecCtrl_NotifyStartedEvent *) sevent;	    
	    result->recordingSession = fromwire->recordingSession;
	    retval=True;
	} break;

	case DispRecCtrl_NotifyNotStarted: {
	    XDispRecCtrl_NotifyNotStartedEvent *result = 
		(XDispRecCtrl_NotifyNotStartedEvent *) se;
	    xDispRecCtrl_NotifyNotStartedEvent* fromwire = 
		(xDispRecCtrl_NotifyNotStartedEvent *) sevent;	    
	    result->recordingSession = fromwire->recordingSession;
	    result->reason = fromwire->reason;
	    result->reasonArg1 = fromwire->reasonArg1;
	    retval=True;
	} break;

	case DispRecCtrl_NotifySwitchedDir: {
	    XDispRecCtrl_NotifySwitchedDirEvent *result = 
		(XDispRecCtrl_NotifySwitchedDirEvent *) se;
	    xDispRecCtrl_NotifySwitchedDirEvent* fromwire = 
		(xDispRecCtrl_NotifySwitchedDirEvent *) sevent;	    
	    result->recordingSession = fromwire->recordingSession;
	    retval=True;
	} break;

	case DispRecCtrl_NotifyNotSwitchedDir: {
	    XDispRecCtrl_NotifyNotSwitchedDirEvent *result = 
		(XDispRecCtrl_NotifyNotSwitchedDirEvent *) se;
	    xDispRecCtrl_NotifyNotSwitchedDirEvent* fromwire = 
		(xDispRecCtrl_NotifyNotSwitchedDirEvent *) sevent;	    
	    result->recordingSession = fromwire->recordingSession;
	    result->reason = fromwire->reason;
	    result->stillRecordingToOldDir = 
		( fromwire->flags & flagDispRecCtrl_NotifyNotSwitchedDirEvent_StillRecording ) != 0;
	    retval=True;
	} break;


	case DispRecCtrl_NotifyFinished: {
	    XDispRecCtrl_NotifyFinishedEvent *result = 
		(XDispRecCtrl_NotifyFinishedEvent *) se;
	    xDispRecCtrl_NotifyFinishedEvent* fromwire = 
		(xDispRecCtrl_NotifyFinishedEvent *) sevent;
	    result->recordingSession = fromwire->recordingSession;
	    result->reason = fromwire->reason;
	    retval=True;
	} break;

	case DispRecCtrl_NotifyNotFinished: {
	    XDispRecCtrl_NotifyNotFinishedEvent *result = 
		(XDispRecCtrl_NotifyNotFinishedEvent *) se;
	    xDispRecCtrl_NotifyNotFinishedEvent* fromwire = 
		(xDispRecCtrl_NotifyNotFinishedEvent *) sevent;
	    result->recordingSession = fromwire->recordingSession;
	    result->reason = fromwire->reason;
	    retval=True;
	} break;

	case DispRecCtrl_NotifyRecordingSessionDeleted: {
	    XDispRecCtrl_NotifyRecordingSessionDeletedEvent *result = 
		(XDispRecCtrl_NotifyRecordingSessionDeletedEvent *) se;
	    xDispRecCtrl_NotifyRecordingSessionDeletedEvent* fromwire = 
		(xDispRecCtrl_NotifyRecordingSessionDeletedEvent *) sevent;
	    result->recordingSession = fromwire->recordingSession;
	    result->reason = fromwire->reason;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUIStarted: {
	    XDispRecCtrl_NotifyGUIStartedEvent *result = 
		(XDispRecCtrl_NotifyGUIStartedEvent *) se;
	    xDispRecCtrl_NotifyGUIStartedEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIStartedEvent *) sevent;
	    result->gui = fromwire->gui;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUINotStarted: {
	    XDispRecCtrl_NotifyGUINotStartedEvent *result = 
		(XDispRecCtrl_NotifyGUINotStartedEvent *) se;
	    xDispRecCtrl_NotifyGUINotStartedEvent* fromwire = 
		(xDispRecCtrl_NotifyGUINotStartedEvent *) sevent;
	    result->gui = fromwire->gui;
	    result->reason = fromwire->reason;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUIExited: {
	    XDispRecCtrl_NotifyGUIExitedEvent *result = 
		(XDispRecCtrl_NotifyGUIExitedEvent *) se;
	    xDispRecCtrl_NotifyGUIExitedEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIExitedEvent *) sevent;
	    result->gui = fromwire->gui;
	    result->reason = fromwire->reason;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUIRestarting: {
	    XDispRecCtrl_NotifyGUIRestartingEvent *result = 
		(XDispRecCtrl_NotifyGUIRestartingEvent *) se;
	    xDispRecCtrl_NotifyGUIRestartingEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIRestartingEvent *) sevent;
	    result->gui = fromwire->gui;
	    result->reason = fromwire->reason;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUIRestarted: {
	    XDispRecCtrl_NotifyGUIRestartedEvent *result = 
		(XDispRecCtrl_NotifyGUIRestartedEvent *) se;
	    xDispRecCtrl_NotifyGUIRestartedEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIRestartedEvent *) sevent;
	    result->gui = fromwire->gui;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUINotRestarted: {
	    XDispRecCtrl_NotifyGUINotRestartedEvent *result = 
		(XDispRecCtrl_NotifyGUINotRestartedEvent *) se;
	    xDispRecCtrl_NotifyGUINotRestartedEvent* fromwire = 
		(xDispRecCtrl_NotifyGUINotRestartedEvent *) sevent;
	    result->gui = fromwire->gui;
	    result->reason = fromwire->reason;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUIVisible: {
	    XDispRecCtrl_NotifyGUIVisibleEvent *result = 
		(XDispRecCtrl_NotifyGUIVisibleEvent *) se;
	    xDispRecCtrl_NotifyGUIVisibleEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIVisibleEvent *) sevent;
	    result->gui = fromwire->gui;
	    result->screen = fromwire->screen;
	    result->x = fromwire->x; 
	    result->y = fromwire->y;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUIHidden: {
	    XDispRecCtrl_NotifyGUIHiddenEvent *result = 
		(XDispRecCtrl_NotifyGUIHiddenEvent *) se;
	    xDispRecCtrl_NotifyGUIHiddenEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIHiddenEvent *) sevent;
	    result->gui = fromwire->gui;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUIControlsRecordingSession: {
	    XDispRecCtrl_NotifyGUIControlsRecordingSessionEvent *result = 
		(XDispRecCtrl_NotifyGUIControlsRecordingSessionEvent *) se;
	    xDispRecCtrl_NotifyGUIControlsRecordingSessionEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIControlsRecordingSessionEvent *) sevent;
	    result->gui = fromwire->gui;
	    result->recordingSession = fromwire->recordingSession;
	    retval=True;
	} break;
	
	case DispRecCtrl_NotifyGUILeavesRecordingSession: {
	    XDispRecCtrl_NotifyGUILeavesRecordingSessionEvent *result = 
		(XDispRecCtrl_NotifyGUILeavesRecordingSessionEvent *) se;
	    xDispRecCtrl_NotifyGUILeavesRecordingSessionEvent* fromwire = 
		(xDispRecCtrl_NotifyGUILeavesRecordingSessionEvent *) sevent;
	    result->gui = fromwire->gui;
	    result->recordingSession = fromwire->recordingSession;
	    retval=True;
	} break;

	case DispRecCtrl_NotifyGUIHandleDeleted: {
	    XDispRecCtrl_NotifyGUIHandleDeletedEvent *result = 
		(XDispRecCtrl_NotifyGUIHandleDeletedEvent *) se;
	    xDispRecCtrl_NotifyGUIHandleDeletedEvent* fromwire = 
		(xDispRecCtrl_NotifyGUIHandleDeletedEvent *) sevent;
	    result->gui = fromwire->gui;
	    retval=True;
	} break;

	
	default: retval=False;

	} /* end switch */
    } break;
    case DispRecCtrl_UserColor: {
	XDispRecCtrl_UserColorEvent *se = 
	    (XDispRecCtrl_UserColorEvent *) re;
	xDispRecCtrl_UserColorEvent* sevent = 
	    (xDispRecCtrl_UserColorEvent *) event;
	se->type = sevent->type & 0x7f;
	se->serial = _XSetLastRequestRead(dpy,(xGenericReply *) event);
	se->send_event = (sevent->type & 0x80) != 0;
	se->display = dpy;
	se->subtype = sevent->subtype;

	switch (((xDispRecCtrl_UserColorEvent *)event)->subtype) {	
	case DispRecCtrl_UserColor_ChangePixel: {
	    XDispRecCtrl_UserColor_ChangePixelEvent *result = 
		(XDispRecCtrl_UserColor_ChangePixelEvent *) se;
	    xDispRecCtrl_UserColor_ChangePixelEvent* fromwire = 
		(xDispRecCtrl_UserColor_ChangePixelEvent *) sevent;
	    result->user_color_id = fromwire->user_color_id;
	    result->oldpixel = fromwire->oldpixel;
	    result->provided_color = 
		VtUtils_xColorItem_to_XColor(
		    &(fromwire->provided_color));
	    retval=True;	    
	} break;
	case DispRecCtrl_UserColor_Delete: {
	    XDispRecCtrl_UserColor_DeleteEvent *result = 
		(XDispRecCtrl_UserColor_DeleteEvent *) se;
	    xDispRecCtrl_UserColor_DeleteEvent* fromwire = 
		(xDispRecCtrl_UserColor_DeleteEvent *) sevent;
	    result->user_color_id = fromwire->user_color_id;
	    retval=True;	    
	} break;
	default: retval=False;	    
	} /* end switch */
    } break;
    case DispRecCtrl_DRSoundRec: {
	XDispRecCtrl_DRSoundRecEvent *se = 
	    (XDispRecCtrl_DRSoundRecEvent *) re;
	xDispRecCtrl_DRSoundRecEvent* sevent = 
	    (xDispRecCtrl_DRSoundRecEvent *) event;
	se->type = sevent->type & 0x7f;
	se->serial = _XSetLastRequestRead(dpy,(xGenericReply *) event);
	se->send_event = (sevent->type & 0x80) != 0;
	se->display = dpy;
	se->subtype = sevent->subtype;
	se->recordingSession = sevent->recordingSession;
	switch (sevent->subtype) {
	    case DispRecCtrl_DRSoundRec_RecLevel: {
		XDispRecCtrl_DRSoundRec_RecLevelEvent *result = 
		    (XDispRecCtrl_DRSoundRec_RecLevelEvent *) se;
		xDispRecCtrl_DRSoundRec_RecLevelEvent* fromwire = 
		    (xDispRecCtrl_DRSoundRec_RecLevelEvent *) sevent;
		result->fVeff = fromwire->fVeff;
		result->fOverdrivePpm = fromwire->fOverdrivePpm;
		result->fVeffSquare = fromwire->fVeffSquare;
		result->fMaxLevel = fromwire->fMaxLevel;
		retval = True;
	    } break;
	    default: retval=False;
	} /* end switch */
    } break;
    default: retval=False;;
    } /* end switch */
    return retval;
}

static Status event_to_wire (dpy, re, event)
    Display *dpy;
    XEvent  *re; /* input */
    xEvent  *event; /* output to wire */
{
    XExtDisplayInfo *info = find_display (dpy);
    Status retstatus=0;

    DispRecCtrlCheckExtension (dpy, info, 0);

    switch ((re->type & 0x7f) - info->codes->first_event) {
    case DispRecCtrl_Notify: {
    	XDispRecCtrl_NotifyEvent* se = 
	    (XDispRecCtrl_NotifyEvent *) re;
	xDispRecCtrl_NotifyEvent* sevent = 
	    (xDispRecCtrl_NotifyEvent *) event;
    	sevent->type = se->type | (se->send_event ? 0x80 : 0);
    	sevent->sequenceNumber = se->serial & 0xffff;
    	sevent->subtype= se->subtype;

	switch (sevent->subtype) {

	case DispRecCtrl_NotifyStarted: {
	    XDispRecCtrl_NotifyStartedEvent *input = 
		(XDispRecCtrl_NotifyStartedEvent *) se;
	    xDispRecCtrl_NotifyStartedEvent* towire = 
		(xDispRecCtrl_NotifyStartedEvent *) sevent;	    
	    towire->recordingSession = input->recordingSession;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyNotStarted: {
	    XDispRecCtrl_NotifyNotStartedEvent *input = 
		(XDispRecCtrl_NotifyNotStartedEvent *) se;
	    xDispRecCtrl_NotifyNotStartedEvent* towire = 
		(xDispRecCtrl_NotifyNotStartedEvent *) sevent;	    
	    towire->recordingSession = input->recordingSession;
	    towire->reason = input->reason;
	    towire->reasonArg1 = input->reasonArg1;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifySwitchedDir: {
	    XDispRecCtrl_NotifySwitchedDirEvent *input = 
		(XDispRecCtrl_NotifySwitchedDirEvent *) se;
	    xDispRecCtrl_NotifySwitchedDirEvent* towire = 
		(xDispRecCtrl_NotifySwitchedDirEvent *) sevent;	    
	    towire->recordingSession = input->recordingSession;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyNotSwitchedDir: {
	    XDispRecCtrl_NotifyNotSwitchedDirEvent *input = 
		(XDispRecCtrl_NotifyNotSwitchedDirEvent *) se;
	    xDispRecCtrl_NotifyNotSwitchedDirEvent* towire = 
		(xDispRecCtrl_NotifyNotSwitchedDirEvent *) sevent;	    
	    towire->recordingSession = input->recordingSession;
	    towire->reason = input->reason;
	    towire->flags =
		( 
			input->stillRecordingToOldDir 
			?
		  	flagDispRecCtrl_NotifyNotSwitchedDirEvent_StillRecording 
			: 
			0
		);
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyFinished: {
	    XDispRecCtrl_NotifyFinishedEvent *input = 
		(XDispRecCtrl_NotifyFinishedEvent *) se;
	    xDispRecCtrl_NotifyFinishedEvent* towire = 
		(xDispRecCtrl_NotifyFinishedEvent *) sevent;
	    towire->recordingSession = input->recordingSession;
	    towire->reason = input->reason;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyNotFinished: {
	    XDispRecCtrl_NotifyNotFinishedEvent *input = 
		(XDispRecCtrl_NotifyNotFinishedEvent *) se;
	    xDispRecCtrl_NotifyNotFinishedEvent* towire = 
		(xDispRecCtrl_NotifyNotFinishedEvent *) sevent;
	    towire->recordingSession = input->recordingSession;
	    towire->reason = input->reason;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyRecordingSessionDeleted: {
	    XDispRecCtrl_NotifyRecordingSessionDeletedEvent *input = 
		(XDispRecCtrl_NotifyRecordingSessionDeletedEvent *) se;
	    xDispRecCtrl_NotifyRecordingSessionDeletedEvent* towire = 
		(xDispRecCtrl_NotifyRecordingSessionDeletedEvent *) sevent;
	    towire->recordingSession = input->recordingSession;
	    towire->reason = input->reason;
	    retstatus=1;
	} break;
	
	case DispRecCtrl_NotifyGUIStarted: {
	    XDispRecCtrl_NotifyGUIStartedEvent *input = 
		(XDispRecCtrl_NotifyGUIStartedEvent *) se;
	    xDispRecCtrl_NotifyGUIStartedEvent* towire = 
		(xDispRecCtrl_NotifyGUIStartedEvent *) sevent;
	    towire->gui = input->gui;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUINotStarted: {
	    XDispRecCtrl_NotifyGUINotStartedEvent *input = 
		(XDispRecCtrl_NotifyGUINotStartedEvent *) se;
	    xDispRecCtrl_NotifyGUINotStartedEvent* towire = 
		(xDispRecCtrl_NotifyGUINotStartedEvent *) sevent;
	    towire->gui = input->gui;
	    towire->reason = input->reason;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUIExited: {
	    XDispRecCtrl_NotifyGUIExitedEvent *input = 
		(XDispRecCtrl_NotifyGUIExitedEvent *) se;
	    xDispRecCtrl_NotifyGUIExitedEvent* towire = 
		(xDispRecCtrl_NotifyGUIExitedEvent *) sevent;
	    towire->gui = input->gui;
	    towire->reason = input->reason;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUIRestarting: {
	    XDispRecCtrl_NotifyGUIRestartingEvent *input = 
		(XDispRecCtrl_NotifyGUIRestartingEvent *) se;
	    xDispRecCtrl_NotifyGUIRestartingEvent* towire = 
		(xDispRecCtrl_NotifyGUIRestartingEvent *) sevent;
	    towire->gui = input->gui;
	    towire->reason = input->reason;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUIRestarted: {
	    XDispRecCtrl_NotifyGUIRestartedEvent *input = 
		(XDispRecCtrl_NotifyGUIRestartedEvent *) se;
	    xDispRecCtrl_NotifyGUIRestartedEvent* towire = 
		(xDispRecCtrl_NotifyGUIRestartedEvent *) sevent;
	    towire->gui = input->gui;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUINotRestarted: {
	    XDispRecCtrl_NotifyGUINotRestartedEvent *input = 
		(XDispRecCtrl_NotifyGUINotRestartedEvent *) se;
	    xDispRecCtrl_NotifyGUINotRestartedEvent* towire = 
		(xDispRecCtrl_NotifyGUINotRestartedEvent *) sevent;
	    towire->gui = input->gui;
	    towire->reason = input->reason;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUIVisible: {
	    XDispRecCtrl_NotifyGUIVisibleEvent *input = 
		(XDispRecCtrl_NotifyGUIVisibleEvent *) se;
	    xDispRecCtrl_NotifyGUIVisibleEvent* towire = 
		(xDispRecCtrl_NotifyGUIVisibleEvent *) sevent;
	    towire->gui = input->gui;
	    towire->screen = input->screen;
	    towire->x = input->x;
	    towire->y = input->y;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUIHidden: {
	    XDispRecCtrl_NotifyGUIHiddenEvent *input = 
		(XDispRecCtrl_NotifyGUIHiddenEvent *) se;
	    xDispRecCtrl_NotifyGUIHiddenEvent* towire = 
		(xDispRecCtrl_NotifyGUIHiddenEvent *) sevent;
	    towire->gui = input->gui;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUIControlsRecordingSession: {
	    XDispRecCtrl_NotifyGUIControlsRecordingSessionEvent *input = 
		(XDispRecCtrl_NotifyGUIControlsRecordingSessionEvent *) se;
	    xDispRecCtrl_NotifyGUIControlsRecordingSessionEvent* towire = 
		(xDispRecCtrl_NotifyGUIControlsRecordingSessionEvent *) sevent;
	    towire->gui = input->gui;
	    towire->recordingSession = input->recordingSession;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUILeavesRecordingSession: {
	    XDispRecCtrl_NotifyGUILeavesRecordingSessionEvent *input = 
		(XDispRecCtrl_NotifyGUILeavesRecordingSessionEvent *) se;
	    xDispRecCtrl_NotifyGUILeavesRecordingSessionEvent* towire = 
		(xDispRecCtrl_NotifyGUILeavesRecordingSessionEvent *) sevent;
	    towire->gui = input->gui;
	    towire->recordingSession = input->recordingSession;
	    retstatus=1;
	} break;

	case DispRecCtrl_NotifyGUIHandleDeleted: {
	    XDispRecCtrl_NotifyGUIHandleDeletedEvent *input = 
		(XDispRecCtrl_NotifyGUIHandleDeletedEvent *) se;
	    xDispRecCtrl_NotifyGUIHandleDeletedEvent* towire = 
		(xDispRecCtrl_NotifyGUIHandleDeletedEvent *) sevent;
	    towire->gui = input->gui;
	    retstatus=1;
	} break;


	default: retstatus=0;
	} /* end switch */
    } break;
    case DispRecCtrl_UserColor: {
    	XDispRecCtrl_UserColorEvent* se = 
	    (XDispRecCtrl_UserColorEvent *) re;
	xDispRecCtrl_UserColorEvent* sevent = 
	    (xDispRecCtrl_UserColorEvent *) event;
    	sevent->type = se->type | (se->send_event ? 0x80 : 0);
    	sevent->sequenceNumber = se->serial & 0xffff;
    	sevent->subtype= se->subtype;

	switch (sevent->subtype) {
	case DispRecCtrl_UserColor_ChangePixel: {
	    XDispRecCtrl_UserColor_ChangePixelEvent *input = 
		(XDispRecCtrl_UserColor_ChangePixelEvent *) se;
	    xDispRecCtrl_UserColor_ChangePixelEvent* towire = 
		(xDispRecCtrl_UserColor_ChangePixelEvent *) sevent;
	    towire->user_color_id = input->user_color_id;
	    towire->oldpixel = input->oldpixel;
	    towire->provided_color =
		VtUtils_XColor_to_xColorItem(
		    &(input->provided_color));
	    retstatus=1;
	} break;
	case DispRecCtrl_UserColor_Delete: {
	    XDispRecCtrl_UserColor_DeleteEvent *input = 
		(XDispRecCtrl_UserColor_DeleteEvent *) se;
	    xDispRecCtrl_UserColor_DeleteEvent* towire = 
		(xDispRecCtrl_UserColor_DeleteEvent *) sevent;
	    towire->user_color_id = input->user_color_id;
	    retstatus=1;
	} break;
	default: retstatus=0;
	} /* end switch */
    } break;
    case DispRecCtrl_DRSoundRec: {
    	XDispRecCtrl_DRSoundRecEvent* se = 
	    (XDispRecCtrl_DRSoundRecEvent *) re;
	xDispRecCtrl_DRSoundRecEvent* sevent = 
	    (xDispRecCtrl_DRSoundRecEvent *) event;
    	sevent->type = se->type | (se->send_event ? 0x80 : 0);
    	sevent->sequenceNumber = se->serial & 0xffff;
    	sevent->subtype= se->subtype;
	sevent->recordingSession = se->recordingSession;

	switch (sevent->subtype) {
	    case DispRecCtrl_DRSoundRec_RecLevel: {
		XDispRecCtrl_DRSoundRec_RecLevelEvent *input = 
		    (XDispRecCtrl_DRSoundRec_RecLevelEvent *) se;
		xDispRecCtrl_DRSoundRec_RecLevelEvent* towire = 
		    (xDispRecCtrl_DRSoundRec_RecLevelEvent *) sevent;
		towire->fVeff = input->fVeff;
		towire->fOverdrivePpm = input->fOverdrivePpm;
		towire->fVeffSquare = input->fVeffSquare;
		towire->fMaxLevel = input->fMaxLevel;
		retstatus=1;
	    } break;
	    default: retstatus = 0;
	} /* end switch */
    } break;
    default: retstatus=0;
    } /* end switch */
    return retstatus;
}


/****************************************************************************
 *                                                                          *
 *			    DispRecCtrl public interfaces                         *
 *                                                                          *
 ****************************************************************************/

Bool XDispRecCtrl_QueryExtension (dpy, event_basep, error_basep)
    Display *dpy;
    int *event_basep, *error_basep;
{
    XExtDisplayInfo *info = find_display (dpy);

    if (XextHasExtension(info)) {
	*event_basep = info->codes->first_event;
	*error_basep = info->codes->first_error;
	return True;
    } else {
	return False;
    }
}


Status XDispRecCtrl_QueryVersion(dpy, major_versionp, minor_versionp)
    Display *dpy;
    int	    *major_versionp, *minor_versionp;
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_QueryVersionReply	    rep;
    register xDispRecCtrl_QueryVersionReq  *req;

    DispRecCtrlCheckExtension (dpy, info, 0);

    LockDisplay (dpy);
    GetReq (DispRecCtrl_QueryVersion, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_QueryVersion;
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return 0;
    }
    *major_versionp = rep.majorVersion;
    *minor_versionp = rep.minorVersion;
    UnlockDisplay (dpy);
    SyncHandle ();
    return 1;
}

DispRecCtrl_RecordingSession 
    XDispRecCtrl_CreateRecordingSession(
    	Display*	dpy,
	int flags
    ) 
{
    DispRecCtrl_RecordingSession recordingSession;
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_CreateRecordingSessionReply rep;
    register xDispRecCtrl_CreateRecordingSessionReq *req;

    DispRecCtrlCheckExtension(
	dpy, info, DispRecCtrl_UserColorID_NULL);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_CreateRecordingSession, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_CreateRecordingSession;
    req->flags = flags;
    
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    recordingSession = rep.recordingSession;
    UnlockDisplay(dpy);
    SyncHandle();

    return recordingSession;
}

void XDispRecCtrl_DeleteRecordingSession(
    Display*	dpy,
    DispRecCtrl_RecordingSession recordingSession
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_DeleteRecordingSessionReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_DeleteRecordingSession, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_DeleteRecordingSession;

    req->recordingSession = recordingSession;
	
    UnlockDisplay(dpy);
    SyncHandle();
}

DispRecCtrl_RecordingSession*
    XDispRecCtrl_ListRecordingSessions(
	Display* dpy,
	unsigned long flags,
	int *count_return
)
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_ListRecordingSessionsReply	    rep;
    register xDispRecCtrl_ListRecordingSessionsReq *req;
    int count = 0;
    DispRecCtrl_RecordingSession* result;
    CARD32* rawResult;

    DispRecCtrlCheckExtension (dpy, info, False);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_ListRecordingSessions, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_ListRecordingSessions;

    req->flags = flags;

    /* read fixed size part of the reply */
    if (!_XReply (dpy, (xReply *) &rep, 0, xFalse)) {
	UnlockDisplay (dpy);
	SyncHandle ();
        if ( count_return != NULL ) {
	    *count_return = 0;
        }
	return NULL;
    }

    /* get count and set count_return */
    count = rep.count;
    if ( count_return != NULL ) {
	*count_return = count;
    }

    /* allocations for the result */
    result = Xmalloc(count * sizeof(DispRecCtrl_RecordingSession));
    if ( result == NULL ) {
	UnlockDisplay (dpy);
	SyncHandle ();
	if ( count_return != NULL ) {
	    *count_return = 0;
	}
	return NULL;
    }

    rawResult = Xmalloc(count * sizeof(CARD32));
    if ( rawResult == NULL ) {
	XFree(result);
	UnlockDisplay (dpy);
	SyncHandle ();
	if ( count_return != NULL ) {
	    *count_return = 0;
	}
	return NULL;
    }

    /* init rawResult */
    memset(rawResult, 0, count * sizeof(CARD32));
    
    /* read rawResult */
    _XReadPad(dpy, (char*)rawResult, count * sizeof(CARD32));

    /* translate rawResult to result */
    {
	int idx;
	for( idx = 0; idx < count; idx++ ) {
		result[idx] = rawResult[idx];				
	}
    }

    XFree(rawResult);

    UnlockDisplay (dpy);
    SyncHandle ();

    return result;
}


void XDispRecCtrl_StartRecording(
    Display*  dpy,
    DispRecCtrl_RecordingSession recordingSession,
    const char* dirname,
    enum DispRecCtrl_DirMode dir_mode,
    unsigned int length_recording_time,  /* in seconds, 0 means unlimited */
    Bool endless,
    Bool doSoundRecording,
    Bool muteSoundRecording,
    Bool finishOtherRecordingsBeforeStart
    )
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_StartRecordingReq *req;
    register long nbytes, length_dirname;
    
    length_dirname = strlen(dirname);

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_StartRecording, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_StartRecording;
    
    req->recordingSession = recordingSession;
    req->fFlags = 
	( endless ? _DispRecCtrl_StartRecording_Flags_Endless : 0 )
	|
	( doSoundRecording ? 
	    _DispRecCtrl_StartRecording_Flags_DoSoundRecording : 0 )
	|
	( muteSoundRecording ?
	    _DispRecCtrl_StartRecording_Flags_MuteSoundRecording : 0 )
	|
	( finishOtherRecordingsBeforeStart ?
	    _DispRecCtrl_StartRecording_Flags_FinishOtherRecordingsBeforeStart : 0 )
    ;
    switch( dir_mode ) {
	case DispRecCtrl_DirMode_CreateDir:
		req->fFlags |= _DispRecCtrl_StartRecording_Flags_CreateRecDir;
		break;
	case DispRecCtrl_DirMode_CreateDirWhichHasToBeNew:
		req->fFlags |= 
			  _DispRecCtrl_StartRecording_Flags_CreateRecDir
			| _DispRecCtrl_StartRecording_Flags_FailIfRecDirHasExistedBefore
			;
		break;
	case DispRecCtrl_DirMode_DirMustAlreadyExist:
		/* no flags to set */
		break;
	default:
		/* Bad Value */	;
    }
    req->length_recording_time = length_recording_time;
    req->length_dirname = length_dirname;
    
    /* compute the length of the dirname data */
    nbytes=4*((length_dirname+3)/4); /* pad to next 32-bit boundary */

    /* compute total request length (nbytes is a multiple of 4)*/
    req->length +=  nbytes/4;

    /* Write the dirname */
    Data16(dpy, dirname, length_dirname);
  
    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

void XDispRecCtrl_SwitchRecordingDir(
    Display*  dpy,
    DispRecCtrl_RecordingSession recordingSession,
    const char* dirname,
    enum DispRecCtrl_DirMode dir_mode
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_SwitchRecordingDirReq *req;
    register long nbytes, length_dirname;
    
    length_dirname = strlen(dirname);

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_SwitchRecordingDir, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_SwitchRecordingDir;
    
    req->recordingSession = recordingSession;
    req->fFlags = 0; 
    switch( dir_mode ) {
	case DispRecCtrl_DirMode_CreateDir:
		req->fFlags |= _DispRecCtrl_SwitchRecordingDir_Flags_CreateRecDir;
		break;
	case DispRecCtrl_DirMode_CreateDirWhichHasToBeNew:
		req->fFlags |= 
			  _DispRecCtrl_SwitchRecordingDir_Flags_CreateRecDir
			| _DispRecCtrl_SwitchRecordingDir_Flags_FailIfRecDirHasExistedBefore
			;
		break;
	case DispRecCtrl_DirMode_DirMustAlreadyExist:
		/* no flags to set */
		break;
	default:
		/* Bad Value */	;
    }
    req->length_dirname = length_dirname;
    
    /* compute the length of the dirname data */
    nbytes=4*((length_dirname+3)/4); /* pad to next 32-bit boundary */

    /* compute total request length (nbytes is a multiple of 4)*/
    req->length +=  nbytes/4;

    /* Write the dirname */
    Data16(dpy, dirname, length_dirname);
  
    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

void XDispRecCtrl_FinishRecording(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_FinishRecordingReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_FinishRecording, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_FinishRecording;

    req->recordingSession = recordingSession;
    
    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

void XDispRecCtrl_EnableCompression(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession,
    const char* compression_method_name,
    const char* compression_command)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_EnableCompressionReq *req;
    int length_compression_method_name = 
	strlen(compression_method_name);
    int length_compression_command = 
	strlen(compression_command);
    register long nbytes;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_EnableCompression, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_EnableCompression;

    req->recordingSession = recordingSession;

    req->length_compression_method_name = 
	length_compression_method_name;
    req->length_compression_command = 
	length_compression_command;

    /* compute the length of string data */
    nbytes = 4*((length_compression_method_name+3)/4);
    nbytes += 4*((length_compression_command+3)/4);

     /* compute total request length (nbytes is a multiple of 4)*/
    req->length +=  nbytes/4;

    /* Write the strings */
    Data16(dpy, 
	   compression_method_name, 
	   length_compression_method_name);
    Data16(dpy, 
	   compression_command, 
	   length_compression_command);

    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

void XDispRecCtrl_DisableCompression(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_DisableCompressionReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_DisableCompression, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_DisableCompression;
    
    req->recordingSession = recordingSession;
    
    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

void XDispRecCtrl_SetMaxFilesysUsage(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession,    
    unsigned int max_filesys_usage_at_start, /* in ppm */
    unsigned int max_filesys_usage_while_recording, /* in ppm */
    unsigned int max_filesys_usage_tolerance /* in Bytes */)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_SetMaxFilesysUsageReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_SetMaxFilesysUsage, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_SetMaxFilesysUsage;

    req->recordingSession = recordingSession;

    req->max_filesys_usage_at_start = max_filesys_usage_at_start;
    req->max_filesys_usage_while_recording = 
	max_filesys_usage_while_recording;
    req->max_filesys_usage_tolerance = 
	max_filesys_usage_tolerance;
    
    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

void XDispRecCtrl_DisableMaxFilesysUsage(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_DisableMaxFilesysUsageReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_DisableMaxFilesysUsage, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_DisableMaxFilesysUsage;
    
    req->recordingSession = recordingSession;
    
    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

void XDispRecCtrl_SetMotionKeyRatio(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession,
    float motionKeyRatio
	/* motion-files are approximately motionKeyRatio times as large as key-files */
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_SetMotionKeyRatioReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_SetMotionKeyRatio, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_SetMotionKeyRatio;
	
    req->recordingSession = recordingSession;
	
    req->motionKeyRatioFixedPoint = 
	(int)( motionKeyRatio * (float)(DispRecCtrl_MotionKeyRatioBase) );

    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

/**** Sound recording config ****/
struct XDispRecCtrl_SoundRecConfig_struct{
    int fSamplingRate;
    int fChannels;
    int fBitsPerSample;
    int fOutputBitrate;
    int fRecLevelUpdateRate; /* updates per 1000 seconds */
    char* fDeviceName;
    char* fSoundApiName;
    char* fCompressionMethod;
    char* fCompressionProgram;
    
    struct timediff fFadeOutTimeBeforePause;
    struct timediff fFadeInTimeAfterResume;
    
    /* split sound files */
    Bool fSplitSoundFiles;
    struct timediff fTimeBetweenSplits_ControlFile; /*approximate time between splits. Actual time will be larger or equal. */
    struct timediff fTimeBetweenSplits_DataFile; /*approximate time between splits. Actual time will be larger or equal. */
    
};

/* creates a soundrec config and fills it with default values */    
XDispRecCtrl_SoundRecConfig
    XDispRecCtrl_CreateSoundRecConfig( VtError *err )
{
    XDispRecCtrl_SoundRecConfig soundRecConfig = 
	NEW( struct XDispRecCtrl_SoundRecConfig_struct );
    /* set default values */
    soundRecConfig->fSamplingRate = 22050; /* half of CD */
    soundRecConfig->fChannels = 1;
    soundRecConfig->fBitsPerSample = 16;
    soundRecConfig->fOutputBitrate = 20000;
    soundRecConfig->fRecLevelUpdateRate = 10000;
    soundRecConfig->fDeviceName = VtUtils_strdup("default", err);
    soundRecConfig->fSoundApiName = VtUtils_strdup("", err);
    soundRecConfig->fCompressionMethod = VtUtils_strdup("none", err);
    soundRecConfig->fCompressionProgram = VtUtils_strdup("", err);
    
    soundRecConfig->fFadeOutTimeBeforePause = VtTimeDiffMsecs(50);
    soundRecConfig->fFadeInTimeAfterResume = VtTimeDiffMsecs(50);
    
    soundRecConfig->fSplitSoundFiles = False;
    soundRecConfig->fTimeBetweenSplits_ControlFile = VtTimeDiffSecs(120);
    soundRecConfig->fTimeBetweenSplits_DataFile = VtTimeDiffSecs(60);
    
    /* return the config  */
    return soundRecConfig;
}

void XDispRecCtrl_FreeSoundRecConfig(
	XDispRecCtrl_SoundRecConfig cfg)
{
    if ( cfg->fDeviceName != NULL ) {
	DELETEARRAY( cfg->fDeviceName );
	cfg->fDeviceName = NULL;
    }
    if ( cfg->fSoundApiName != NULL ) {
	DELETEARRAY( cfg->fSoundApiName );
	cfg->fSoundApiName = NULL;
    }
    if ( cfg->fCompressionMethod != NULL ) {
	DELETEARRAY( cfg->fCompressionMethod );
	cfg->fCompressionMethod = NULL;
    }
    if ( cfg->fCompressionProgram != NULL ) {
	DELETEARRAY( cfg->fCompressionProgram );
	cfg->fCompressionProgram = NULL;
    }
    DELETE( cfg );
}

void XDispRecCtrl_SoundRecConfig_SetSamplingRate(
    XDispRecCtrl_SoundRecConfig cfg, int samplingRate)
{
    cfg->fSamplingRate = samplingRate;
}
    
void XDispRecCtrl_SoundRecConfig_SetChannels(
    XDispRecCtrl_SoundRecConfig cfg, int channels)
{
    cfg->fChannels = channels;
}

void XDispRecCtrl_SoundRecConfig_SetBitsPerSample(
    XDispRecCtrl_SoundRecConfig cfg, int bitsPerSample)
{
    cfg->fBitsPerSample = bitsPerSample;
}

void XDispRecCtrl_SoundRecConfig_SetOutputBitrate(
    XDispRecCtrl_SoundRecConfig cfg, int outputBitrate)
{
    cfg->fOutputBitrate = outputBitrate;
}

void XDispRecCtrl_SoundRecConfig_SetRecLevelUpdateRate(
    XDispRecCtrl_SoundRecConfig cfg, int recLevelUpdateRate)
{
    cfg->fRecLevelUpdateRate = recLevelUpdateRate;
}

void XDispRecCtrl_SoundRecConfig_SetDeviceName(
    XDispRecCtrl_SoundRecConfig cfg, 
    const char* deviceName,
    VtError *err)
{
    if ( cfg->fDeviceName != NULL ) {
	DELETEARRAY( cfg->fDeviceName );
    }
    cfg->fDeviceName = VtUtils_strdup(deviceName, err);
}

void XDispRecCtrl_SoundRecConfig_SetSoundApiName(
    XDispRecCtrl_SoundRecConfig cfg, 
    const char* soundApiName,
    VtError *err)
{
	if ( cfg->fSoundApiName != NULL ) {
		DELETEARRAY( cfg->fSoundApiName );
	}
	cfg->fSoundApiName = 
		VtUtils_strdup( soundApiName, err );
}


void XDispRecCtrl_SoundRecConfig_SetCompressionMethod(
    XDispRecCtrl_SoundRecConfig cfg, 
    const char* compressionMethod,
    VtError *err)
{
    if ( cfg->fCompressionMethod != NULL ) {
	DELETEARRAY( cfg->fCompressionMethod );
    }
    cfg->fCompressionMethod = VtUtils_strdup(compressionMethod, err);
}

void XDispRecCtrl_SoundRecConfig_SetCompressionProgram(
    XDispRecCtrl_SoundRecConfig cfg, 
    const char* compressionProgram,
    VtError *err)
{
    if ( cfg->fCompressionProgram != NULL ) {
	DELETEARRAY( cfg->fCompressionProgram );
    }
    cfg->fCompressionProgram = VtUtils_strdup( compressionProgram, err );
}

void XDispRecCtrl_SoundRecConfig_SetFadeOutTimeBeforePause( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff value,
    VtError *err
)
{
    cfg->fFadeOutTimeBeforePause = value;
}

void XDispRecCtrl_SoundRecConfig_SetFadeInTimeAfterResume( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff value,
    VtError *err
)
{
    cfg->fFadeInTimeAfterResume = value;
}

void XDispRecCtrl_SoundRecConfig_SetSplitSoundFiles( 
    XDispRecCtrl_SoundRecConfig cfg,
    Bool splitSoundFiles,
    VtError *err
)
{
    cfg->fSplitSoundFiles = splitSoundFiles;
}

void XDispRecCtrl_SoundRecConfig_SetTimeBetweenSplits_ControlFile( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff timeBetweenSplits_ControlFile,
    VtError *err
)
{
    cfg->fTimeBetweenSplits_ControlFile = timeBetweenSplits_ControlFile;
}

void XDispRecCtrl_SoundRecConfig_SetTimeBetweenSplits_DataFile( 
    XDispRecCtrl_SoundRecConfig cfg,
    struct timediff timeBetweenSplits_DataFile,
    VtError *err
)
{
    cfg->fTimeBetweenSplits_DataFile = timeBetweenSplits_DataFile;
}

void XDispRecCtrl_SetSoundRecordingConfig(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession,
    XDispRecCtrl_SoundRecConfig cfg
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_SetSoundRecordingConfigReq *req;
    int length_fDeviceName = 
	strlen(cfg->fDeviceName);
    int length_fCompressionMethod = 
	strlen(cfg->fCompressionMethod);
    int length_fCompressionProgram = 
	strlen(cfg->fCompressionProgram);
    int length_fSoundApiName = 
	strlen(cfg->fSoundApiName);
    int length_fSoundApiNameData = 0;

    register long nbytes;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_SetSoundRecordingConfig, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_SetSoundRecordingConfig;

    req->recordingSession = recordingSession;

    /* set data-fields */
    req->fSamplingRate = cfg->fSamplingRate;
    req->fChannels = cfg->fChannels;
    req->fBitsPerSample = cfg->fBitsPerSample;
    req->fOutputBitrate = cfg->fOutputBitrate;
    req->fRecLevelUpdateRate = cfg->fRecLevelUpdateRate;
    
    req->fFadeOutTimeBeforePause.fSecs = cfg->fFadeOutTimeBeforePause.td_sec;
    req->fFadeOutTimeBeforePause.fUsecs = cfg->fFadeOutTimeBeforePause.td_usec;
    req->fFadeInTimeAfterResume.fSecs = cfg->fFadeInTimeAfterResume.td_sec;
    req->fFadeInTimeAfterResume.fUsecs = cfg->fFadeInTimeAfterResume.td_usec;
    
    /* split sound files */
    if ( cfg->fSplitSoundFiles ) {
	req->fFlags |= _DispRecCtrl_SetSoundRecordingConfig_Flags_SplitSoundFiles;
    }
    req->fSplitSoundFiles_TimeBetweenSplits_ControlFile.fSecs = cfg->fTimeBetweenSplits_ControlFile.td_sec; 
    req->fSplitSoundFiles_TimeBetweenSplits_ControlFile.fUsecs = cfg->fTimeBetweenSplits_ControlFile.td_usec; 
    req->fSplitSoundFiles_TimeBetweenSplits_DataFile.fSecs = cfg->fTimeBetweenSplits_DataFile.td_sec; 
    req->fSplitSoundFiles_TimeBetweenSplits_DataFile.fUsecs = cfg->fTimeBetweenSplits_DataFile.td_usec; 
    
    req->fLength_DeviceName =
	length_fDeviceName;
    req->fLength_CompressionMethod =
	length_fCompressionMethod;
    req->fLength_CompressionProgram =
	length_fCompressionProgram;

    /* compute the length of string data */
    nbytes = 4*((length_fDeviceName+3)/4);
    nbytes += 4*((length_fCompressionMethod+3)/4);
    nbytes += 4*((length_fCompressionProgram+3)/4);

    /* set sound api if requested */
    if ( length_fSoundApiName > 0 ) {
	req->fFlags |= 
		_DispRecCtrl_SetSoundRecordingConfig_Flags_HaveSoundApiName;
	length_fSoundApiNameData = 
		4*((length_fSoundApiName+2+3)/4);
		/* additional "+2" is for length */
	nbytes += length_fSoundApiNameData;
    }

     /* compute total request length (nbytes is a multiple of 4)*/
    req->length +=  nbytes/4;

    /* Write the strings */
    Data16(dpy, 
	   cfg->fDeviceName, 
	   length_fDeviceName);
    Data16(dpy, 
	   cfg->fCompressionMethod, 
	   length_fCompressionMethod);
    Data16(dpy, 
	   cfg->fCompressionProgram, 
	   length_fCompressionProgram);
    if ( length_fSoundApiName > 0 ) {
	DECLERR;
	char* soundApiNameData = NEWARRAY( char, length_fSoundApiNameData );
	/* set the length in the first two bytes */
	*((uint16*)soundApiNameData) = length_fSoundApiName;
	/* copy the rest of the string */
	strncpy(
		soundApiNameData+2, cfg->fSoundApiName, length_fSoundApiName );
	/* add length and string to the request */
	Data16(dpy,
		soundApiNameData,
		length_fSoundApiNameData);
	/* cleanup */
	DELETEARRAY( soundApiNameData );
    }

    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}


/****** end of Sound recording config ******/

Bool XDispRecCtrl_Active(
    Display* dpy,
    DispRecCtrl_RecordingSession recordingSession)
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_ActiveReply	    rep;
    register xDispRecCtrl_ActiveReq *req;
    Bool isActive;

    DispRecCtrlCheckExtension (dpy, info, False);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_Active, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_Active;

    req->recordingSession = recordingSession;

    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return False;
    }

    isActive = rep.active;

    UnlockDisplay (dpy);
    SyncHandle ();

    return isActive;
}


void XDispRecCtrl_GetBaseDisplay(
    Display* dpy, 
    char** display_name)
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_GetBaseDisplayReply	    rep;
    register xDispRecCtrl_GetBaseDisplayReq *req;
    int length_display_name;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_GetBaseDisplay, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_GetBaseDisplay;

    if (!_XReply (dpy, (xReply *) &rep, 0, xFalse)) {
	*display_name = NULL;
	UnlockDisplay (dpy);
	SyncHandle ();
	return;
    }

    length_display_name = rep.length_display_name;

    /* Allocate and read the display_name */
    *display_name = Xmalloc((length_display_name+1) * sizeof(char));
    if (*display_name == NULL) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return;
    }
    _XReadPad(dpy, *display_name, length_display_name);
    (*display_name)[length_display_name]=(char)0; /* set end of string */

    UnlockDisplay (dpy);
    SyncHandle ();
}


void XDispRecCtrl_SelectInput(
    Display* dpy,
    unsigned long event_mask)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_SelectInputReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_SelectInput, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_SelectInput;

    req->event_mask = event_mask;
    
    UnlockDisplay(dpy);
    SyncHandle();

    XFlush(dpy);
}

unsigned long XDispRecCtrl_InputSelected(Display* dpy)
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_InputSelectedReply	    rep;
    register xDispRecCtrl_InputSelectedReq *req;
    unsigned long event_mask;

    DispRecCtrlCheckExtension (dpy, info, 0L);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_InputSelected, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_InputSelected;

    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return 0L;
    }

    event_mask = rep.event_mask;

    UnlockDisplay (dpy);
    SyncHandle ();

    return event_mask;
}


/**** utilities for special things,
    like having the record&play dialog displayed during playback ****/

void XDispRecCtrl_ReparentToBaseWindow(
    Display* dpy,
    Window window, 
    Window appwindow, /* WindowID in rpserv. The window which
			should be used with ConfigureWindow requests.
			Different from window if window is provided by the
			windowmanager and appwindow is the main window
			of the application managed by the window manager */
    Window base_window, /* WindowID in the base-Display of 
			   rpserv to which the window should be reparented */
    Colormap base_colormap, /* ColormapID of the Colormap
			       which should be used via the UserColor
			       Mechanism. If None, then window keeps
			       its current colormap. */
    int x, int y,
    Bool coordsAreOffsets,
    Bool doDepReparenting
    )
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_ReparentToBaseWindowReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_ReparentToBaseWindow, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_ReparentToBaseWindow;

    req->window = window;
    req->appwindow = appwindow;
    req->base_window = base_window;
    req->base_colormap = base_colormap;
    req->x = x;
    req->y = y;
    req->flags = 0;
    if ( coordsAreOffsets ) {
	req->flags |= xDispRecCtrl_ReparentToBaseWindow_Flag_CoordsAreOffsets;
    }
    if ( doDepReparenting ) {
	req->flags |= xDispRecCtrl_ReparentToBaseWindow_Flag_DoDepReparenting;
    }
    
    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_UnreparentWindow(
    Display* dpy,
    Window window
    )
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_UnreparentWindowReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_UnreparentWindow, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_UnreparentWindow;

    req->window = window;
    
    UnlockDisplay(dpy);
    SyncHandle();
}

Colormap XDispRecCtrl_CreateBaseColormap(
    Display* dpy, /* The rpserv-Display */
    Window base_window, /* Window-ID in the basedisplay
			   of the rpserv which provides dpy */
    Visual* base_visual,
    int alloc
)
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_CreateBaseColormapReply rep;
    register xDispRecCtrl_CreateBaseColormapReq *req;
    Colormap base_colormap;

    DispRecCtrlCheckExtension (dpy, info, None);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_CreateBaseColormap, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_CreateBaseColormap;

    req->base_window = base_window;
    req->base_visual_id = XVisualIDFromVisual(base_visual);
    req->alloc = alloc;
    
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    base_colormap = rep.base_colormap;
    UnlockDisplay(dpy);
    SyncHandle();

    return base_colormap;
}
    
void XDispRecCtrl_FreeBaseColormap(
    Display* dpy, /* The rpserv-Display */
    Colormap base_colormap /* Colormap-ID in the basedisplay */
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_FreeBaseColormapReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_FreeBaseColormap, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_FreeBaseColormap;

    req->base_colormap = base_colormap;
    
    UnlockDisplay(dpy);
    SyncHandle();
}


void XDispRecCtrl_BaseColormap_StoreColors(
    Display* dpy, /* The rpserv-Display */
    Colormap base_colormap, /* Colormap-ID in the basedisplay */
    const XColor *colors, int ncolors
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_BaseColormap_StoreColorsReq *req;
    register int i;
    xColorItem citem;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_BaseColormap_StoreColors, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType =
	X_DispRecCtrl_BaseColormap_StoreColors;
    
    req->base_colormap = base_colormap;
    
    /* compute total request length (taken from lib/X11/StColors.c) */
    req->length += (ncolors * SIZEOF(xColorItem)) >> 2;
    /* assume size is multiple of 4 */

    /* Write the colors (taken from lib/X11/StColors.c) */
    for (i=0; i<ncolors; i++) {
	citem = VtUtils_XColor_to_xColorItem(&(colors[i]));

	/* note that xColorItem doesn't contain all 16-bit quantities, so
	   we can't use Data16 */
	Data(dpy, (char *)&citem, (long) SIZEOF(xColorItem));
	/* assume size is multiple of 4 */
    }
  
    UnlockDisplay(dpy);
    SyncHandle();
    XFlush(dpy);
}

DispRecCtrl_UserColorID 
XDispRecCtrl_AllocateUserColorInBaseColormap(
    Display* dpy, /* The rpserv-Display */
    Colormap base_colormap, /* Colormap-ID in the basedisplay */
    const XColor* user_color, 
    XColor* provided_color /* Output parameter: The color which is
			      provided for approximating user_color */
)
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_AllocateUserColorInBaseColormapReply rep;
    register xDispRecCtrl_AllocateUserColorInBaseColormapReq *req;
    DispRecCtrl_UserColorID user_color_id;

    DispRecCtrlCheckExtension(
	dpy, info, DispRecCtrl_UserColorID_NULL);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_AllocateUserColorInBaseColormap, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_AllocateUserColorInBaseColormap;

    req->base_colormap = base_colormap;
    req->user_color = VtUtils_XColor_to_xColorItem(user_color);
    
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    user_color_id = rep.user_color_id;
    *provided_color = 
	VtUtils_xColorItem_to_XColor(&(rep.provided_color));
    UnlockDisplay(dpy);
    SyncHandle();

    return user_color_id;
}

void XDispRecCtrl_DeleteUserColor(
    Display* dpy, /* The rpserv-Display */
    DispRecCtrl_UserColorID user_color_id
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_DeleteUserColorReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_DeleteUserColor, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_DeleteUserColor;

    req->user_color_id = user_color_id;
    
    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_AuthorizeReuseOfUserColorID(
    Display* dpy, /* The rpserv-Display */
    DispRecCtrl_UserColorID user_color_id
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_AuthorizeReuseOfUserColorIDReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_AuthorizeReuseOfUserColorID, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_AuthorizeReuseOfUserColorID;

    req->user_color_id = user_color_id;
    
    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_ChangeBaseColormapOfReparentedWindow(
    Display* dpy, /* The rpserv-Display */
    Window window, /* Window-ID in the rpserv-display */
    Colormap base_colormap /* Colormap-ID in the basedisplay */
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_ChangeBaseColormapOfReparentedWindowReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_ChangeBaseColormapOfReparentedWindow, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_ChangeBaseColormapOfReparentedWindow;

    req->window = window;
    req->base_colormap = base_colormap;
    
    UnlockDisplay(dpy);
    SyncHandle();
}


void XDispRecCtrl_RedrawEverything(
    Display* dpy /* The rpserv-Display */
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_RedrawEverythingReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_RedrawEverything, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_RedrawEverything;

    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_InhibitColormapInstall(
    Display* dpy /* The rpserv-Display */
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_InhibitColormapInstallReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_InhibitColormapInstall, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_InhibitColormapInstall;

    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_EnableColormapInstall (
    Display* dpy /* The rpserv-Display */
)
{
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_EnableColormapInstallReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_EnableColormapInstall, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_EnableColormapInstall;

    UnlockDisplay(dpy);
    SyncHandle();
}


/* get license info in a form suitable to be displayed to the user,
    the returned license-info has to be freed by the client of this extension.
    Returns NULL on communication error. */
char* XDispRecCtrl_GetLicenseInfo( Display *dpy )
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_GetLicenseInfoReply	    rep;
    register xDispRecCtrl_GetLicenseInfoReq *req;
    int length_license_info;
    char* license_info;

    DispRecCtrlCheckExtension (dpy, info, /* default retval = */ NULL);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_GetLicenseInfo, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_GetLicenseInfo;

    if (!_XReply (dpy, (xReply *) &rep, 0, xFalse)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return NULL;
    }

    length_license_info = rep.length_license_info;

    /* Allocate and read the display_name */
   license_info = Xmalloc((length_license_info+1) * sizeof(char));
    if ( license_info == NULL ) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return NULL;
    }
    _XReadPad(dpy,license_info, length_license_info);
   license_info[length_license_info]=(char)0; /* set end of string */

    UnlockDisplay (dpy);
    SyncHandle ();
    
    return license_info;
}

void XDispRecCtrl_DeepSync( Display *dpy )
{
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_DeepSyncReply	    rep;
    register xDispRecCtrl_DeepSyncReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_DeepSync, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = X_DispRecCtrl_DeepSync;

    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return;
    }

    UnlockDisplay (dpy);
    SyncHandle ();
    
    return;
}



DispRecCtrl_GUI XDispRecCtrl_CreateGUIHandle(
    Display *dpy
) {
    DispRecCtrl_GUI guiHandle;
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_CreateGUIHandleReply rep;
    register xDispRecCtrl_CreateGUIHandleReq *req;

    DispRecCtrlCheckExtension(
	dpy, info, DispRecCtrl_GUI_None);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_CreateGUIHandle, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_CreateGUIHandle;
    
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    guiHandle = rep.guiHandle;
    UnlockDisplay(dpy);
    SyncHandle();

    return guiHandle;
}

void XDispRecCtrl_DeleteGUIHandle(
    Display *dpy,
    DispRecCtrl_GUI gui
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_DeleteGUIHandleReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_DeleteGUIHandle, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_DeleteGUIHandle;

    req->guiHandle = gui;
	
    UnlockDisplay(dpy);
    SyncHandle();
}

static void XDispRecCtrl_SendEnvData(
    Display *dpy,
    CARD16 *requestlength,
    char** environ,
    int environCount,
    VtError *err
) {
    /* adjust the total length of the request */
    {
	int envIdx;
	for ( envIdx = 0; envIdx < environCount; envIdx++ ) {
	    (*requestlength) += ( ( strlen( environ[envIdx] ) +3 ) >> 2 ) + 1;
		/* requestlength is the number of 32-bit units, hence length of string in 32-bit units plus one for carrying the length od string  */
	}
    }
    
    /* send the data */
    {
	int envIdx;
	for ( envIdx = 0; envIdx < environCount; envIdx++ ) {
	    CARD32 length = strlen( environ[envIdx] );
	    {
		Data32( dpy, &length, 4 );
	    }
	    /* since we do not  know whether bytes past the end of string are allocated
		we copy the string before sending it */
	    {
		char* tmp = NEWARRAY( char, length + 4 );
		CARD32 datalength = ( length + 3 ) & ( ~3UL ) ;
		memcpy( tmp, environ[envIdx], length );
		Data32( dpy, tmp, datalength );
		DELETEARRAY( tmp );
	    }
	}
    }    
}


static void XDispRecCtrl_StartGUI_Internal(
    Display *dpy,
    DispRecCtrl_GUI gui,
    uid_t user,
    gid_t group,
    char** environ,
    int environCount,
    Bool keepalive,
    VtError *err
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_StartGUIReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_StartGUI, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_StartGUI;

    req->guiHandle = gui;
    req->uid = user;
    req->gid = group;
    req->flags = 0;
    if ( keepalive ) {
	req->flags |= xDispRecCtrl_StartGUI_Flag_KeepAlive;
    }
    req->fEnvironCount = environCount;
    
     XDispRecCtrl_SendEnvData(
	 dpy, &req->length, environ, environCount, err );
    
    UnlockDisplay(dpy);
    SyncHandle();
}

static void XDispRecCtrl_ComputeEnvironForGUI(
    char*** environ,
    int* environCount,
    VtError *err
) {
    *environ = NEWARRAY( char*, 2 );
    *environCount = 0;
    /* use XAUTHORITY */
    {
	char* xauthority = getenv( "XAUTHORITY" );
	if ( xauthority ) {
	    char* tmp = NEWARRAY( char, strlen( xauthority ) + 100 );
	    sprintf( tmp, "XAUTHORITY=%s", xauthority );
	    (*environ)[*environCount] = tmp;
	    (*environCount)++;
	}
    }
    /* terminate the list */
    (*environ)[*environCount]  = NULL;
}


void XDispRecCtrl_StartGUI(
    Display *dpy,
    DispRecCtrl_GUI gui,
    uid_t user,
    gid_t group,
    Bool keepalive
) {
    DECLERR;
    char** environ = NULL;
    int environCount = 0;
    XDispRecCtrl_ComputeEnvironForGUI( &environ, &environCount, err );
    if ( environ != NULL ) {
	XDispRecCtrl_StartGUI_Internal(
	    dpy,
	    gui,
	    user,
	    group,
	    environ,
	    environCount,
	    keepalive,
	    err
	);
    }
}

void XDispRecCtrl_SetGUIKeepalive(
    Display *dpy,
    DispRecCtrl_GUI gui,
    Bool keepalive
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_SetGUIKeepaliveReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_SetGUIKeepalive, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_SetGUIKeepalive;

    req->guiHandle = gui;
    req->keepAlive = keepalive ? 1 : 0;

    UnlockDisplay(dpy);
    SyncHandle();
}

static DispRecCtrl_GUI XDispRecCtrl_StartGUIIfNotRunning_Internal(
    Display *dpy,
    uid_t user,
    gid_t group,
    char** environ,
    int environCount,
    Bool keepalive,
    VtError *err
) {
    DispRecCtrl_GUI guiHandle;
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_StartGUIIfNotRunningReply rep;
    register xDispRecCtrl_StartGUIIfNotRunningReq *req;

    DispRecCtrlCheckExtension (dpy, info, DispRecCtrl_GUI_None);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_StartGUIIfNotRunning, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_StartGUIIfNotRunning;

    req->uid = user;
    req->gid = group;
    req->flags = 0;
    if ( keepalive ) {
	req->flags |= xDispRecCtrl_StartGUI_Flag_KeepAlive;
    }
    req->fEnvironCount = environCount;
    
    XDispRecCtrl_SendEnvData(
	dpy, &req->length, environ, environCount, err );

    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    guiHandle = rep.guiHandle;
    UnlockDisplay(dpy);
    SyncHandle();

    return guiHandle;
}

DispRecCtrl_GUI XDispRecCtrl_StartGUIIfNotRunning(
    Display *dpy,
    uid_t user,
    gid_t group,
    Bool keepalive
) {
    DECLERR;
    char** environ = NULL;
    int environCount = 0;
    XDispRecCtrl_ComputeEnvironForGUI( &environ, &environCount, err );
    if ( environ != NULL ) {
	return
	    XDispRecCtrl_StartGUIIfNotRunning_Internal(
		dpy,
		user,
		group,
		environ,
		environCount,
		keepalive,
		err
	    );
    } else {
	return DispRecCtrl_GUI_None;
    }
}


void XDispRecCtrl_QuitGUI(
    Display *dpy,
    DispRecCtrl_GUI gui
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_QuitGUIReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_QuitGUI, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_QuitGUI;

    req->guiHandle = gui;
	
    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_KillGUI(
    Display *dpy,
    DispRecCtrl_GUI gui
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_KillGUIReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_KillGUI, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_KillGUI;

    req->guiHandle = gui;
	
    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_ShowGUI(
    Display *dpy,
    DispRecCtrl_GUI gui,
    int screen, int x, int y
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_ShowGUIReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_ShowGUI, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_ShowGUI;

    req->guiHandle = gui;
    req->screen = screen;
    req->x = x;
    req->y = y;
	
    UnlockDisplay(dpy);
    SyncHandle();
}

void XDispRecCtrl_HideGUI(
    Display *dpy,
    DispRecCtrl_GUI gui
) {
    XExtDisplayInfo *info = find_display (dpy);
    register xDispRecCtrl_HideGUIReq *req;

    DispRecCtrlSimpleCheckExtension (dpy, info);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_HideGUI, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_HideGUI;

    req->guiHandle = gui;
	
    UnlockDisplay(dpy);
    SyncHandle();
}

Bool XDispRecCtrl_GUIRunning(
    Display *dpy,
    DispRecCtrl_GUI gui
) {
    Bool running;
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_GUIRunningReply rep;
    register xDispRecCtrl_GUIRunningReq *req;

    DispRecCtrlCheckExtension (dpy, info, False);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_GUIRunning, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_GUIRunning;

    req->guiHandle = gui;
	
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    running = rep.running ? True : False;
    UnlockDisplay(dpy);
    SyncHandle();

    return running;
}

Bool XDispRecCtrl_GUIVisible(
    Display *dpy,
    DispRecCtrl_GUI gui
) {
    Bool visible;
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_GUIVisibleReply rep;
    register xDispRecCtrl_GUIVisibleReq *req;

    DispRecCtrlCheckExtension (dpy, info, False);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_GUIVisible, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_GUIVisible;

    req->guiHandle = gui;
	
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    visible = rep.visible ? True : False;
    UnlockDisplay(dpy);
    SyncHandle();

    return visible;
}

DispRecCtrl_RecordingSession
    XDispRecCtrl_GetRecordingSessionOfGUI(
	Display *dpy,
	DispRecCtrl_GUI gui
    ) 
{
    DispRecCtrl_RecordingSession result;
    XExtDisplayInfo *info = find_display (dpy);
    xDispRecCtrl_GetRecordingSessionOfGUIReply rep;
    register xDispRecCtrl_GetRecordingSessionOfGUIReq *req;

    DispRecCtrlCheckExtension (dpy, info, DispRecCtrl_RecordingSession_None);

    LockDisplay(dpy);
    GetReq(DispRecCtrl_GetRecordingSessionOfGUI, req);
    req->reqType = info->codes->major_opcode;
    req->DispRecCtrl_ReqType = 
	X_DispRecCtrl_GetRecordingSessionOfGUI;

    req->guiHandle = gui;
	
    if (!_XReply (dpy, (xReply *) &rep, 0, xTrue)) {
	UnlockDisplay (dpy);
	SyncHandle ();
	return None;
    }

    result = rep.recordingSession;
    UnlockDisplay(dpy);
    SyncHandle();

    return result;
}



/********************************************************************************/
/* Utils */
/********************************************************************************/
const char* XDispRecCtrl_ReasonToString( int reason )
{
    #define C( name ) case DispRecCtrl_Reason_##name: return #name; 
    switch( reason ) {
	C(ByClientRequest)
	C(AlreadyRecording)
	C(CannotOpenFile)
	C(MaxLengthReached)
	C(MaxSizeReached)
	C(MaxFsUsageReached)
	C(BadValue)
	C(WriteError)
	C(FileDisappeared)
	C(TotalMaxLengthReached)
	C(NoLicense)
	C(SoundDeviceReadError)
	C(SoundDeviceConfigError)
	C(SoundDeviceOpenError)
	C(SoundDeviceMissingCapabilities)
	C(ChildprocessDiedUnexpectedly)
	C(CannotStartSoundCompression)
	C(NoSoundSupport)
	C(NotRecording)
	C(Killed)
	C(AlreadyRunning)
	C(ProcessTerminated)
	C(NotPaused)
	C(DueToSwitchDir)
	C(MissingDirectory)
	C(SwitchDirNotSupportedInEndlessMode)
	C(BadDirectory)
	C(DirectoryDoesNotExist)
	C(SwitchToNextFileFailed)
	C(WritingCntFileFailed)
	C(SoundControlFileNotOpened)
	C(InternalError)
	C(ErrorReadingPipeFromSoundRecordingProcess)
	C(FailedToCreateRecDir)
	C(RecDirAlreadyExists)
	C(TiedClientDisconnected)
	C(DueToOptionFinishOtherRecordingsBeforeStart)
	C(ServerShutdown)

	default:
	    return "invalid reason value";
    }
}


