/****************************************
Filename: libDispRecCtrlExtension/DispRecCtrlExtension_str.h
Description: DispRecCtrl-Extension internal declarations
Author: Christian Linhart
Copyright: (C)1997-2011 Christian Linhart
****************************************/

/* Based on shape_str.h which is Copyright (c) 1989  X Consortium */

#ifndef _DispRecCtrlSTR_H_
#define _DispRecCtrlSTR_H_

/*
 * Protocol requests constants and alignment values
 * These would really be in DispRecCtrl's X.h and Xproto.h equivalents
 */

#include "DispRecCtrlExtension.h"
#include <X11/Xproto.h>


#define DispRecCtrl_NAME "DispRecCtrl"

#define DispRecCtrl_MAJOR_VERSION	10	/* current version numbers */
#define DispRecCtrl_MINOR_VERSION	0

/**********************************
 version history:

Version 5.0: 20080810: Author CLI
    Added the following fields to xDispRecCtrl_SetSoundRecordingConfigReq:
	xDR_TimeDiff fFadeOutTimeBeforePause;
	xDR_TimeDiff fFadeInTimeAfterResume;
	xDR_TimeDiff fSplitSoundFiles_TimeBetweenSplits;  (* if timediff is 0, then splitting is disabled *)
	
Version 6.0: 20080820: Author CLI
    Added the following fields to xDispRecCtrl_SetSoundRecordingConfigReq:
	CARD32 fFlags B32;
    Defined the following Flags:
	_DispRecCtrl_SetSoundRecordingConfig_Flags_SplitSoundFiles
    Replaced the field "fSplitSoundFiles_TimeBetweenSplits" by
	xDR_TimeDiff fSplitSoundFiles_TimeBetweenSplits_ControlFile;
	xDR_TimeDiff fSplitSoundFiles_TimeBetweenSplits_DataFile;

Version 7.0: 20101024: Author CLI
    Only changes in non-public parts of the protocol.
*/
/*
Version 8.0: 20110905: Author CLI
	TODO: create history-entry from svn-diffs
	
Version 9.0: 20120212: Author CLI
	add flags member to xDispRecCtrl_CreateRecordingSessionReq
	TODO: create more history-entries from svn-diffs

Version 10.0: 20120212: Author CLI
	padded DispRecCtrl_IndicatePixmapHasBeenWrittenToKeyFile
	to a multiple of 8 bytes because it contains an 8 byte value.

	new Requests "IndicateFinalMotFileSize", "IndicateMotFileClosed"
*/
/**********************************/

#ifndef B64
#define B64
# if !defined(WORD64) && !defined(LONG64)
typedef unsigned long long CARD64;
# endif
#endif


/**** auxiliary structs ****/

typedef struct _DR_TimeDiff{
    CARD32 fSecs B32;
    CARD32 fUsecs B32;
} xDR_TimeDiff;

/**** request-structs ****/

typedef struct _DispRecCtrl_QueryVersion {
    CARD8	reqType;		/* always DispRecCtrl_ReqCode */
    CARD8	DispRecCtrl_ReqType;		/* always X_DispRecCtrl_QueryVersion */
    CARD16	length B16;
} xDispRecCtrl_QueryVersionReq;
#define sz_xDispRecCtrl_QueryVersionReq  4

typedef struct {
	BYTE	type;			/* X_Reply */
	CARD8	unused;			/* not used */
	CARD16	sequenceNumber B16;
	CARD32	length B32;
	CARD16	majorVersion B16;	/* major version of DispRecCtrl protocol */
	CARD16	minorVersion B16;	/* minor version of DispRecCtrl protocol */
	CARD32	pad0 B32;
	CARD32	pad1 B32;
	CARD32	pad2 B32;
	CARD32	pad3 B32;
	CARD32	pad4 B32;
} xDispRecCtrl_QueryVersionReply;
#define sz_xDispRecCtrl_QueryVersionReply	32

typedef struct _DispRecCtrl_CreateRecordingSession {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_CreateRecordingSession_Key */
    CARD16  length B16; /* always 2 */
    CARD32  flags B32;
} xDispRecCtrl_CreateRecordingSessionReq;
#define sz_xDispRecCtrl_CreateRecordingSessionReq 8

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32  	recordingSession B32;    
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_CreateRecordingSessionReply;
#define sz_xDispRecCtrl_CreateRecordingSessionReply 32

typedef struct _DispRecCtrl_DeleteRecordingSession {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_DeleteRecordingSession*/
    CARD16  length B16; /* always 5 */
    CARD32  recordingSession B32;    
} xDispRecCtrl_DeleteRecordingSessionReq;
#define sz_xDispRecCtrl_DeleteRecordingSessionReq 8

typedef struct _DispRecCtrl_ListRecordingSessions {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_ListRecordingSessions */
    CARD16  length B16; /* always 2 */
    CARD32  flags B32;    
} xDispRecCtrl_ListRecordingSessionsReq;
#define sz_xDispRecCtrl_ListRecordingSessionsReq 8

typedef struct {
	BYTE	type;			/* X_Reply */
	BYTE	pad0;			
	CARD16	sequenceNumber B16;
	CARD32	length B32;		/* 4 byte quantities beyond size of GenericReply */
	CARD32	count B32;		/* number of recording sessions in the reply */
	CARD32	pad2 B32;
	CARD32	pad3 B32;
	CARD32	pad4 B32;
	CARD32	pad5 B32;
	CARD32	pad6 B32;
} xDispRecCtrl_ListRecordingSessionsReply;
#define sz_xDispRecCtrl_ListRecordingSessionsReply	32


typedef struct _DispRecCtrl_StartRecording {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_StartRecording */
    CARD16  length B16;
    CARD32  recordingSession B32;    
    CARD16 fFlags;
    CARD16 length_dirname B16;
    CARD32 length_recording_time; /* in seconds */
    /* followed by a STRING8(dirname) +pad) */
} xDispRecCtrl_StartRecordingReq;
#define sz_xDispRecCtrl_StartRecordingReq 16

#define _DispRecCtrl_StartRecording_Flags_Endless (1 << 0)
#define _DispRecCtrl_StartRecording_Flags_DoSoundRecording (1 << 1)
#define _DispRecCtrl_StartRecording_Flags_MuteSoundRecording (1 << 2)
#define _DispRecCtrl_StartRecording_Flags_CreateRecDir (1 << 3)
#define _DispRecCtrl_StartRecording_Flags_FailIfRecDirHasExistedBefore (1 << 4)
#define _DispRecCtrl_StartRecording_Flags_FinishOtherRecordingsBeforeStart (1 << 5)

typedef struct _DispRecCtrl_SwitchRecordingDir {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_SwitchRecordingDir */
    CARD16  length B16;
    CARD32  recordingSession B32;    
    CARD16 fFlags;
    CARD16 length_dirname B16;
    /* followed by a STRING8(dirname) +pad) */	
} xDispRecCtrl_SwitchRecordingDirReq;
#define sz_xDispRecCtrl_SwitchRecordingDirReq 12

#define _DispRecCtrl_SwitchRecordingDir_Flags_CreateRecDir (1 << 0 )
#define _DispRecCtrl_SwitchRecordingDir_Flags_FailIfRecDirHasExistedBefore (1 << 1 )

typedef struct _DispRecCtrl_FinishRecording {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_FinishRecording */
    CARD16  length B16; /* always 1 */
    CARD32  recordingSession B32;    
} xDispRecCtrl_FinishRecordingReq;
#define sz_xDispRecCtrl_FinishRecordingReq 8

typedef struct _DispRecCtrl_EnableCompression {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_EnableCompression */
    CARD16  length B16; 
    CARD32  recordingSession B32;    
    CARD16 length_compression_method_name;
    CARD16 length_compression_command;
    /* followed by a STRING8 (the name of the compression method) + pad*/
    /* followed by a STRING8 (the compression command) +pad */
} xDispRecCtrl_EnableCompressionReq;
#define sz_xDispRecCtrl_EnableCompressionReq 12

typedef struct _DispRecCtrl_DisableCompression {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_DisableCompression */
    CARD16  length B16; /* always 1 */
    CARD32  recordingSession B32;    
} xDispRecCtrl_DisableCompressionReq;
#define sz_xDispRecCtrl_DisableCompressionReq 8

typedef struct _DispRecCtrl_SetMaxFilesysUsage {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_DisableCompression */
    CARD16  length B16; /* always 3 */
    CARD32  recordingSession B32;    
    CARD32 max_filesys_usage_at_start; /* in ppm */
    CARD32 max_filesys_usage_while_recording; /* in ppm */
    CARD32 max_filesys_usage_tolerance; /* in ppm */
} xDispRecCtrl_SetMaxFilesysUsageReq;
#define sz_xDispRecCtrl_SetMaxFilesysUsageReq 20

typedef struct _DispRecCtrl_DisableMaxFilesysUsage {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_DisableMaxFilesysUsage */
    CARD16  length B16; /* always 2 */
    CARD32  recordingSession B32;    
} xDispRecCtrl_DisableMaxFilesysUsageReq;
#define sz_xDispRecCtrl_DisableMaxFilesysUsageReq 8

typedef struct _DispRecCtrl_Active {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_Active */
    CARD16  length B16; /* always 1 */
    CARD32  recordingSession B32;    
} xDispRecCtrl_ActiveReq;
#define sz_xDispRecCtrl_ActiveReq 8

typedef struct {
	BYTE	type;			/* X_Reply */
	BOOL active;		/* True if the given recording is active */
	CARD16	sequenceNumber B16;
	CARD32	length B32;		/* 0 */
	CARD32	pad1 B32;		/* unused */
	CARD32	pad2 B32;
	CARD32	pad3 B32;
	CARD32	pad4 B32;
	CARD32	pad5 B32;
	CARD32	pad6 B32;
} xDispRecCtrl_ActiveReply;
#define sz_xDispRecCtrl_ActiveReply	32


typedef struct _DispRecCtrl_GetBaseDisplay {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_GetBaseDisplay */
    CARD16  length B16; /* always 1 */
} xDispRecCtrl_GetBaseDisplayReq;
#define sz_xDispRecCtrl_GetBaseDisplayReq 4

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD16 length_display_name B16; /* length of the display name */
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
    /* followed by a STRING8 (display_name) */
} xDispRecCtrl_GetBaseDisplayReply;
#define sz_xDispRecCtrl_GetBaseDisplayReply	32

typedef struct _DispRecCtrl_SelectInput {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_SelectInput */
    CARD16  length B16; /* always 2 */
    CARD32 event_mask B32;
} xDispRecCtrl_SelectInputReq;
#define sz_xDispRecCtrl_SelectInputReq 8

typedef struct _DispRecCtrl_InputSelected {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_InputSelected */
    CARD16  length B16; /* always 1 */
} xDispRecCtrl_InputSelectedReq;
#define sz_xDispRecCtrl_InputSelectedReq 4

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32  event_mask B32;
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_InputSelectedReply;
#define sz_xDispRecCtrl_InputSelectedReply 32


typedef struct _DispRecCtrl_ReparentToBaseWindow{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_ReparentToBaseWindow */
    CARD16  length B16; /* always 7 */
    CARD32 window B32;
    CARD32 appwindow B32;
    CARD32 base_window B32;
    CARD32 base_colormap B32;
    INT16 x B16;
    INT16 y B16;
    CARD32 flags B32; /* new in ProtocolVersion 1 */
} xDispRecCtrl_ReparentToBaseWindowReq;
#define sz_xDispRecCtrl_ReparentToBaseWindowReq 28
/* flags */
#define xDispRecCtrl_ReparentToBaseWindow_Flag_CoordsAreOffsets ( 1 << 0 )
#define xDispRecCtrl_ReparentToBaseWindow_Flag_DoDepReparenting ( 1 << 1 )

typedef struct _DispRecCtrl_UnreparentWindow{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_ReparentToBaseWindow */
    CARD16  length B16; /* always 2 */
    CARD32 window B32;
} xDispRecCtrl_UnreparentWindowReq;
#define sz_xDispRecCtrl_UnreparentWindowReq 8


typedef struct _DispRecCtrl_CreateBaseColormap {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_CreateBaseColormap */
    CARD16  length B16; /* always 4 */
    CARD32 base_window B32;
    CARD32 base_visual_id B32; /* a VisualID */
    CARD32 alloc B32;
} xDispRecCtrl_CreateBaseColormapReq;
#define sz_xDispRecCtrl_CreateBaseColormapReq 16

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32  base_colormap B32;
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_CreateBaseColormapReply;
#define sz_xDispRecCtrl_CreateBaseColormapReply 32

typedef struct _DispRecCtrl_FreeBaseColormap {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_FreeBaseColormap */
    CARD16  length B16; /* always 2 */
    CARD32 base_colormap;
} xDispRecCtrl_FreeBaseColormapReq;
#define sz_xDispRecCtrl_FreeBaseColormapReq 8

typedef struct _DispRecCtrl_BaseColormap_StoreColors {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_BaseColormap_StoreColors */
    CARD16  length B16; /* always 2+3n, where n is the number
			 of entries in the LISTofCOLORITEM */
    CARD32 base_colormap;
    /* followed by LISTofCOLORITEM */
} xDispRecCtrl_BaseColormap_StoreColorsReq;
#define sz_xDispRecCtrl_BaseColormap_StoreColorsReq 8

typedef struct _DispRecCtrl_AllocateUserColorInBaseColormap {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_AllocateUserColorInBaseColormap */
    CARD16  length B16; /* always 5 */
    CARD32 base_colormap B32;
    xColorItem user_color;
} xDispRecCtrl_AllocateUserColorInBaseColormapReq;
#define sz_xDispRecCtrl_AllocateUserColorInBaseColormapReq 20

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32  user_color_id B32;
    xColorItem provided_color;
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
} xDispRecCtrl_AllocateUserColorInBaseColormapReply;
#define sz_xDispRecCtrl_AllocateUserColorInBaseColormapReply 32

typedef struct _DispRecCtrl_DeleteUserColor {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_DeleteUserColor */
    CARD16  length B16; /* always 2 */
    CARD32 user_color_id B32;
} xDispRecCtrl_DeleteUserColorReq;
#define sz_xDispRecCtrl_DeleteUserColorReq 8

typedef struct _DispRecCtrl_AuthorizeReuseOfUserColorID {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_AuthorizeReuseOfUserColorID */
    CARD16  length B16; /* always 2 */
    CARD32 user_color_id B32;
} xDispRecCtrl_AuthorizeReuseOfUserColorIDReq;
#define sz_xDispRecCtrl_AuthorizeReuseOfUserColorIDReq 8

typedef struct _DispRecCtrl_ChangeBaseColormapOfReparentedWindow {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_ChangeBaseColormapOfReparentedWindow */
    CARD16  length B16; /* always 3 */
    CARD32 window B32;
    CARD32 base_colormap B32;
} xDispRecCtrl_ChangeBaseColormapOfReparentedWindowReq;
#define sz_xDispRecCtrl_ChangeBaseColormapOfReparentedWindowReq 12

typedef struct _DispRecCtrl_RedrawEverything {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_RedrawEverything */
    CARD16  length B16; /* always 1 */
} xDispRecCtrl_RedrawEverythingReq;
#define sz_xDispRecCtrl_RedrawEverythingReq 4

typedef struct _DispRecCtrl_InhibitColormapInstall {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_InhibitColormapInstall */
    CARD16  length B16; /* always 1 */
} xDispRecCtrl_InhibitColormapInstallReq;
#define sz_xDispRecCtrl_InhibitColormapInstallReq 4

typedef struct _DispRecCtrl_EnableColormapInstall {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_EnableColormapInstall */
    CARD16  length B16; /* always 1 */
} xDispRecCtrl_EnableColormapInstallReq;
#define sz_xDispRecCtrl_EnableColormapInstallReq 4

#define DispRecCtrl_MotionKeyRatioBase 0x10000
typedef struct _DispRecCtrl_SetMotionKeyRatio {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_SetMotionKeyRatio */
    CARD16  length B16; /* always 2 */
    CARD32  recordingSession B32;    
    CARD32  motionKeyRatioFixedPoint; /* motionKeyRatio * DispRecCtrl_MotionKeyRatioBase */
} xDispRecCtrl_SetMotionKeyRatioReq;
#define sz_xDispRecCtrl_SetMotionKeyRatioReq 12

typedef struct _DispRecCtrl_SetSoundRecordingConfig{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_SetMotionKeyRatio */
    CARD16  length B16; 
    CARD32  recordingSession B32;
    CARD32 fFlags B32;	
    CARD32 fSamplingRate B32;
    CARD8 fChannels;
    CARD8 fBitsPerSample;
    CARD16 fLength_DeviceName B16; /* length of string fDeviceName */
    CARD32 fOutputBitrate B32;
    CARD32 fRecLevelUpdateRate B32; /* updates per 1000 seconds */
    CARD16 fLength_CompressionMethod B16; /* length of string fCompressionMethod */
    CARD16 fLength_CompressionProgram B16; /* length of string fCompressionProgram */
    xDR_TimeDiff fFadeOutTimeBeforePause;
    xDR_TimeDiff fFadeInTimeAfterResume;
    xDR_TimeDiff fSplitSoundFiles_TimeBetweenSplits_ControlFile;
    xDR_TimeDiff fSplitSoundFiles_TimeBetweenSplits_DataFile;
    /* followed by a STRING8 (fDeviceName) + pad */
    /* followed by a STRING8 (fCompressionMethod) + pad */
    /* followed by a STRING8 (fCompressionProgram) + pad */
} xDispRecCtrl_SetSoundRecordingConfigReq;
#define sz_xDispRecCtrl_SetSoundRecordingConfigReq 64

/* Flags values */
#define _DispRecCtrl_SetSoundRecordingConfig_Flags_SplitSoundFiles (1 << 0)
#define _DispRecCtrl_SetSoundRecordingConfig_Flags_HaveSoundApiName (1 << 1)

/********************************/
/* Requests for License-Management */
/********************************/


typedef struct _DispRecCtrl_GetLicenseInfo {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_GetLicenseInfo */
    CARD16  length B16; /* always 1 */
} xDispRecCtrl_GetLicenseInfoReq;
#define sz_xDispRecCtrl_GetLicenseInfoReq 4

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD16    length_license_info B16; /* length of the display name */
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
    /* followed by a STRING8 (license_info) */
} xDispRecCtrl_GetLicenseInfoReply;
#define sz_xDispRecCtrl_GetLicenseInfoReply	32

/********************************/
/* misc Requests  */
/********************************/

typedef struct _DispRecCtrl_DeepSync {
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_DeepSync */
    CARD16  length B16; /* always 1 */
} xDispRecCtrl_DeepSyncReq;
#define sz_xDispRecCtrl_DeepSyncReq 4

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32	pad0 B32;
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_DeepSyncReply;
#define sz_xDispRecCtrl_DeepSyncReply 32


/********************************/
/* GUI handle requests  */
/********************************/

typedef struct _DispRecCtrl_CreateGUIHandleReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_CreateGUIHandle */
    CARD16  length B16;
} xDispRecCtrl_CreateGUIHandleReq;
#define sz_xDispRecCtrl_CreateGUIHandleReq 4

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32  	guiHandle B32;    
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_CreateGUIHandleReply;
#define sz_xDispRecCtrl_CreateGUIHandleReply 32


typedef struct _DispRecCtrl_DeleteGUIHandleReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_DeleteGUIHandle */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_DeleteGUIHandleReq;
#define sz_xDispRecCtrl_DeleteGUIHandleReq 8

typedef struct _DispRecCtrl_StartGUIReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_StartGUI */
    CARD16  length B16;
    CARD32  guiHandle B32;    
    CARD32  uid B32;    
    CARD32  gid B32;    
    CARD32  flags B32; 
    CARD32 fEnvironCount B32;   
} xDispRecCtrl_StartGUIReq;
#define sz_xDispRecCtrl_StartGUIReq 24
/* flags */
#define xDispRecCtrl_StartGUI_Flag_KeepAlive ( 1 << 0 )

typedef struct _DispRecCtrl_SetGUIKeepaliveReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_SetGUIKeepalive */
    CARD16  length B16;
    CARD32  guiHandle B32;    
    CARD8    keepAlive;
    CARD8    pad1;
    CARD16  pad2 B16;    
} xDispRecCtrl_SetGUIKeepaliveReq;
#define sz_xDispRecCtrl_SetGUIKeepaliveReq 12

typedef struct _DispRecCtrl_StartGUIIfNotRunningReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_StartGUIIfNotRunning */
    CARD16  length B16;
    CARD32  uid B32;    
    CARD32  gid B32;    
    CARD32  flags B32;    /* use flags from xDispRecCtrl_StartGUIReq */
    CARD32 fEnvironCount B32;   
} xDispRecCtrl_StartGUIIfNotRunningReq;
#define sz_xDispRecCtrl_StartGUIIfNotRunningReq 20

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32  	guiHandle B32;    
    CARD32	flags B32;
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
} xDispRecCtrl_StartGUIIfNotRunningReply;
#define sz_xDispRecCtrl_StartGUIIfNotRunningReply 32
/* flags */
#define xDispRecCtrl_StartGUIIfNotRunningReply_Flag_AlreadyRunning ( 1<< 0 )

typedef struct _DispRecCtrl_QuitGUIReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_QuitGUI */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_QuitGUIReq;
#define sz_xDispRecCtrl_QuitGUIReq 8

typedef struct _DispRecCtrl_KillGUIReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_KillGUI */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_KillGUIReq;
#define sz_xDispRecCtrl_KillGUIReq 8

typedef struct _DispRecCtrl_ShowGUIReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_ShowGUI */
    CARD16  length B16;
    CARD32  guiHandle B32;    
    CARD32  screen B32;
    CARD32  x B32;
    CARD32  y B32;
} xDispRecCtrl_ShowGUIReq;
#define sz_xDispRecCtrl_ShowGUIReq 20

typedef struct _DispRecCtrl_HideGUIReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_HideGUI */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_HideGUIReq;
#define sz_xDispRecCtrl_HideGUIReq 8

typedef struct _DispRecCtrl_GUIRunningReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_GUIRunning */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_GUIRunningReq;
#define sz_xDispRecCtrl_GUIRunningReq 8

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD8  	running;
    CARD8	pad0;		/* unused */
    CARD16	pad1 B16;
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_GUIRunningReply;
#define sz_xDispRecCtrl_GUIRunningReply 32


typedef struct _DispRecCtrl_GUIVisibleReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_GUIVisible */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_GUIVisibleReq;
#define sz_xDispRecCtrl_GUIVisibleReq 8

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD8  	visible;
    CARD8	pad0;		/* unused */
    CARD16	pad1 B16;
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_GUIVisibleReply;
#define sz_xDispRecCtrl_GUIVisibleReply 32

typedef struct _DispRecCtrl_GetRecordingSessionOfGUIReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_GetRecordingSessionOfGUI */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_GetRecordingSessionOfGUIReq;
#define sz_xDispRecCtrl_GetRecordingSessionOfGUIReq 8

typedef struct {
    BYTE	type;			/* X_Reply */
    CARD8	unused;			/* not used */
    CARD16	sequenceNumber B16;
    CARD32	length B32;	      
    CARD32  	recordingSession B32;    
    CARD32	pad1 B32;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_GetRecordingSessionOfGUIReply;
#define sz_xDispRecCtrl_GetRecordingSessionOfGUIReply 32

typedef struct _DispRecCtrl_IAmThisGUIReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_IAmThisGUI */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_IAmThisGUIReq;
#define sz_xDispRecCtrl_IAmThisGUIReq 8

typedef struct _DispRecCtrl_IndicateThisGUIShuttingDownReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_IndicateThisGUIShuttingDown */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_IndicateThisGUIShuttingDownReq;
#define sz_xDispRecCtrl_IndicateThisGUIShuttingDownReq 8

typedef struct _DispRecCtrl_IndicateThisGUIHiddenReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_IndicateThisGUIHidden */
    CARD16  length B16;
    CARD32  guiHandle B32;    
} xDispRecCtrl_IndicateThisGUIHiddenReq;
#define sz_xDispRecCtrl_IndicateThisGUIHiddenReq 8

typedef struct _DispRecCtrl_IndicateThisGUIVisibleReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_IndicateThisGUIVisible */
    CARD16  length B16;
    CARD32  guiHandle B32;    
    CARD32 screen B32;
    CARD32 x B32;
    CARD32 y B32;
} xDispRecCtrl_IndicateThisGUIVisibleReq;
#define sz_xDispRecCtrl_IndicateThisGUIVisibleReq 20

typedef struct _DispRecCtrl_IndicateThisGUIControlsRecordingSessionReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_IndicateThisGUIControlsRecordingSession */
    CARD16  length B16;
    CARD32  guiHandle B32;    
    CARD32  recordingSession B32;    
} xDispRecCtrl_IndicateThisGUIControlsRecordingSessionReq;
#define sz_xDispRecCtrl_IndicateThisGUIControlsRecordingSessionReq 12

typedef struct _DispRecCtrl_IndicateThisGUILeavesRecordingSessionReq{
    CARD8   reqType;	/* always DispRecCtrl_ReqCode */
    CARD8   DispRecCtrl_ReqType;	/* always X_DispRecCtrl_IndicateThisGUILeavesRecordingSession */
    CARD16  length B16;
    CARD32  guiHandle B32;    
    CARD32  recordingSession B32;    
} xDispRecCtrl_IndicateThisGUILeavesRecordingSessionReq;
#define sz_xDispRecCtrl_IndicateThisGUILeavesRecordingSessionReq 12


/*******************/
/****** Events ******/
/******************/

/************************/
/* Events for RecordControl */
/************************/

typedef struct _DispRecCtrl_Notify {
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; 
    CARD16	sequenceNumber B16;
    CARD32  recordingSession B32;    
    CARD32	pad1 B32;		/* unused or used by subtypes */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifyEvent;
#define sz_xDispRecCtrl_NotifyEvent 32

/**** Subtypes of _DispRecCtrl_Notify ****/
typedef struct _DispRecCtrl_NotifyStarted {
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyStarted */
    CARD16	sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifyStartedEvent;
#define sz_xDispRecCtrl_NotifyStartedEvent 32
/* Subtypes of _DispRecCtrl_Notify */

typedef struct _DispRecCtrl_NotifyNotStarted {
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyNotStarted */
    CARD16	sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD32	reason B32;
    CARD32	reasonArg1 B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
} xDispRecCtrl_NotifyNotStartedEvent;
#define sz_xDispRecCtrl_NotifyNotStartedEvent 32

typedef struct _DispRecCtrl_NotifySwitchedDir {
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifySwitchedDir */
    CARD16	sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifySwitchedDirEvent;
#define sz_xDispRecCtrl_NotifySwitchedDirEvent 32
/* Subtypes of _DispRecCtrl_Notify */

typedef struct _DispRecCtrl_NotifyNotSwitchedDir {
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyNotSwitchedDir */
    CARD16	sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD32	reason B32;
    CARD32	flags B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
} xDispRecCtrl_NotifyNotSwitchedDirEvent;
#define sz_xDispRecCtrl_NotifyNotSwitchedDirEvent 32
#define flagDispRecCtrl_NotifyNotSwitchedDirEvent_StillRecording 1
    
typedef struct _DispRecCtrl_NotifyFinished {
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyFinished */
    CARD16 sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD32 	reason B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyFinishedEvent;
#define sz_xDispRecCtrl_NotifyFinishedEvent 32

typedef struct _DispRecCtrl_NotifyNotFinished {
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyFinished */
    CARD16 sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD32 	reason B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyNotFinishedEvent;
#define sz_xDispRecCtrl_NotifyNotFinishedEvent 32

typedef struct _DispRecCtrl_NotifyRecordingSessionDeleted{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyRecordingSessionDeleted */
    CARD16	sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD32 	reason B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyRecordingSessionDeletedEvent;
#define sz_xDispRecCtrl_NotifyRecordingSessionDeletedEvent 32

typedef struct _DispRecCtrl_NotifyGUIStarted{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIStarted */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifyGUIStartedEvent;
#define sz_xDispRecCtrl_NotifyGUIStartedEvent 32

typedef struct _DispRecCtrl_NotifyGUINotStarted{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUINotStarted */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD32 	reason B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyGUINotStartedEvent;
#define sz_xDispRecCtrl_NotifyGUINotStartedEvent 32

typedef struct _DispRecCtrl_NotifyGUIExited{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIExited */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD32 	reason B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyGUIExitedEvent;
#define sz_xDispRecCtrl_NotifyGUIExitedEvent 32

typedef struct _DispRecCtrl_NotifyGUIRestarting{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIRestarting */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD32 	reason B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyGUIRestartingEvent;
#define sz_xDispRecCtrl_NotifyGUIRestartingEvent 32

typedef struct _DispRecCtrl_NotifyGUIRestarted{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIRestarted */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifyGUIRestartedEvent;
#define sz_xDispRecCtrl_NotifyGUIRestartedEvent 32

typedef struct _DispRecCtrl_NotifyGUINotRestarted{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUINotRestarted */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD32 	reason B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;    
} xDispRecCtrl_NotifyGUINotRestartedEvent;
#define sz_xDispRecCtrl_NotifyGUINotRestartedEvent 32

typedef struct _DispRecCtrl_NotifyGUIVisible{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIVisible */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD32 	screen B32;
    CARD32 	x B32;
    CARD32 	y B32;
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
} xDispRecCtrl_NotifyGUIVisibleEvent;
#define sz_xDispRecCtrl_NotifyGUIVisibleEvent 32

typedef struct _DispRecCtrl_NotifyGUIHidden{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIHidden */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifyGUIHiddenEvent;
#define sz_xDispRecCtrl_NotifyGUIHiddenEvent 32

typedef struct _DispRecCtrl_NotifyGUIControlsRecordingSession{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIControlsRecordingSession */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD32  	recordingSession B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyGUIControlsRecordingSessionEvent;
#define sz_xDispRecCtrl_NotifyGUIControlsRecordingSessionEvent 32

typedef struct _DispRecCtrl_NotifyGUILeavesRecordingSession{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUILeavesRecordingSession */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD32  	recordingSession B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
} xDispRecCtrl_NotifyGUILeavesRecordingSessionEvent;
#define sz_xDispRecCtrl_NotifyGUILeavesRecordingSessionEvent 32

typedef struct _DispRecCtrl_NotifyGUIHandleDeleted{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUIHandleDeleted */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD16	pad0 B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifyGUIHandleDeletedEvent;
#define sz_xDispRecCtrl_NotifyGUIHandleDeletedEvent 32

typedef struct _DispRecCtrl_NotifyGUICommand{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUICommand */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD16	commandId B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_NotifyGUICommandEvent;
#define sz_xDispRecCtrl_NotifyGUICommandEvent 32

typedef xDispRecCtrl_NotifyGUICommandEvent
    xDispRecCtrl_NotifyGUICommand_HideEvent;
#define sz_xDispRecCtrl_NotifyGUICommand_HideEvent \
    sz_xDispRecCtrl_NotifyGUICommandEvent
    
typedef struct _DispRecCtrl_NotifyGUICommand_Show{
    BYTE type; /* always eventBase + DispRecCtrl_Notify */
    BYTE subtype; /* always DispRecCtrl_NotifyGUICommand */
    CARD16	sequenceNumber B16;
    CARD32  	gui B32;    
    CARD16	commandId B16;
    CARD16	pad1 B16;		/* unused */
    CARD32	screen B32;
    CARD32	x B32;
    CARD32	y B32;
    CARD32	pad2 B32;
    CARD32	pad3 B32;
} xDispRecCtrl_NotifyGUICommand_ShowEvent;
#define sz_xDispRecCtrl_NotifyGUICommand_ShowEvent 32

typedef xDispRecCtrl_NotifyGUICommandEvent
    xDispRecCtrl_NotifyGUICommand_QuitEvent;
#define sz_xDispRecCtrl_NotifyGUICommand_QuitEvent \
    sz_xDispRecCtrl_NotifyGUICommandEvent

/********************************/
/* Events for UserColor-Management */
/********************************/

typedef struct _DispRecCtrl_UserColor {
    BYTE type; /* always eventBase + DispRecCtrl_UserColor */
    BYTE subtype; 
    CARD16	sequenceNumber B16;
    CARD32	pad1 B32;		/* unused or used by subtypes */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
    CARD32	pad7 B32;
} xDispRecCtrl_UserColorEvent;
#define sz_xDispRecCtrl_UserColorEvent 32

/**** Subtypes of UserColorEvent ****/

typedef struct _DispRecCtrl_UserColor_ChangePixel {
    BYTE type; /* always eventBase + DispRecCtrl_UserColor */
    BYTE subtype; /* always DispRecCtrl_UserColor_ChangePixel */
    CARD16	sequenceNumber B16;
    CARD32 user_color_id B32;
    CARD32 oldpixel B32;
    xColorItem provided_color;
    CARD32	pad1 B32;		/* unused  */
    CARD32	pad2 B32;
} xDispRecCtrl_UserColor_ChangePixelEvent;
#define sz_xDispRecCtrl_UserColor_ChangePixelEvent 32

typedef struct _DispRecCtrl_UserColor_Delete {
    BYTE type; /* always eventBase + DispRecCtrl_UserColor */
    BYTE subtype; /* always DispRecCtrl_UserColor_Delete */
    CARD16	sequenceNumber B16;
    CARD32 user_color_id B32;
    CARD32	pad1 B32;		/* unused  */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_UserColor_DeleteEvent;
#define sz_xDispRecCtrl_UserColor_DeleteEvent 32

/*************************************/
/* Events for DRSoundRec (SoundRecording) */
/*************************************/

typedef struct _DispRecCtrl_DRSoundRec {
    BYTE type; /* always eventBase + DispRecCtrl_DRSoundRec */
    BYTE subtype; 
    CARD16	sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD32	pad1 B32;		/* unused or used by subtypes */
    CARD32	pad2 B32;
    CARD32	pad3 B32;
    CARD32	pad4 B32;
    CARD32	pad5 B32;
    CARD32	pad6 B32;
} xDispRecCtrl_DRSoundRecEvent;
#define sz_xDispRecCtrl_DRSoundRecEvent 32

/* subtypes of DRSoundRec */


typedef struct _DispRecCtrl_DRSoundRec_RecLevelEvent {
    BYTE type; /* always eventBase + DispRecCtrl_DRSoundRec */
    BYTE subtype;  /* event subtype: always XDispRecCtrl_DRSoundRec_RecLevel */
    CARD16	sequenceNumber B16;
    CARD32  	recordingSession B32;    
    CARD32	fVeff B32; 		/* effective voltage (square-root of arithmetic average of square of sample) */
    CARD32	fOverdrivePpm B32;	/* how manyoverdrive samples per million samples */
    CARD64	fVeffSquare B64;		/* effective voltage squared (omit square-root of formula for Veff)  */
    CARD32	fMaxLevel B32;	/* maximum level, e.g. 32768 for 16bit (=15 bit level, 1 bit sign) */
    CARD32	pad1 B32;		/*unused */
} xDispRecCtrl_DRSoundRec_RecLevelEvent;
#define sz_xDispRecCtrl_DRSoundRec_RecLevelEvent 32


#endif /* _DispRecCtrlSTR_H_ */
