#tcl script

########################################################
# DemoRecorder recording finished dialog
#
# Copyright (C) 2009 Christian Linhart <chris@DemoRecorder.com>
########################################################

package require drWidgetFileSelection 1.0
package require drUtils 1.0
package require drGuiLayoutWithSections 1.0

#puts "start of script"

set unsaved_recording_name $env(DEMOREC_FILENAME)
set DEMOREC_DIR $env(DEMOREC_DIR)
set defaultSaveDir $env(FILE_DIR)

set currentRecordingName "${unsaved_recording_name}"

#config
set outputFileTypes {
	{{Demorecorder Video} {.demorec}}
}

#*** Aux functions ***


#*** GUI ***
frame .t

wm title . "DemoRecorder Recording Finished"


::dr::gui::layoutWithSections::StartSectionList3 .t "top" 


#Headline
::dr::gui::layoutWithSections::StartSection3 .t.top "mainFeatures" "Recording finished." "" "What do you want to do with it? Please choose a tool by pressing the corresponding button:"


proc StartPreviewPlayback { extraOpts } {
	global currentRecordingName
	global unsaved_recording_name
	global env

	if { "${currentRecordingName}" eq "${unsaved_recording_name}" } {
		#unsaved: only allow preview without export or save
		set command { "$env(DEMOREC_DIR)/bin/demorec" "--playback-only" "--load" "${currentRecordingName}" }
	} else {
		set command { "$env(DEMOREC_DIR)/bin/demorec" "--playback-save-and-export-only" "--load" "${currentRecordingName}" }
	}

	append command " $extraOpts"

	eval "exec $command &"	
}

::dr::gui::layoutWithSections::ButtonsAndLabel3 .t.top \
	{
		{ "play" "Preview Playback" \
			{
				StartPreviewPlayback ""
			}
		}
		{ "play-with-options" "with options" \
			{
				StartPreviewPlayback "--with-demorec-gui-options-dialog"
			}
		}
	} \
	"Press this button to play a raw, unexported DemoRecorder video"

if (0) {
::dr::gui::layoutWithSections::ButtonAndLabel3 .t.top "play" "Preview Playback" \
	"Press this button to play a raw, unexported DemoRecorder video" \
	{
		StartPreviewPlayback ""
	}

::dr::gui::layoutWithSections::ButtonAndLabel3 .t.top "playOpt" "Preview Playback with Options" \
	"Like above but you can specify additional options before starting the preview playback" \
	{
		StartPreviewPlayback "--with-demorec-gui-options-dialog"
	}
}

::dr::gui::layoutWithSections::ButtonAndLabel3 .t.top "save" "Save" \
	"Press this button to save the video" \
	{
		set saveTargetName [ tk_getSaveFile -title "Save DemoRecorder video" -initialdir "$defaultSaveDir" -filetypes $outputFileTypes ]

		if { "$saveTargetName" eq "" } {
			#save canceled 
			#change gui
			::dr::gui::layoutWithSections::ChangeHeadlineOfSection3 .t.top "mainFeatures" "Saving of Recording canceled"
		} else {

			puts "save: file=$saveTargetName"

			if { [ file exists "$saveTargetName" ] } {
				#user has chosen "overwrite" -> make backup
				set backupnameStem "${saveTargetName}-bak"
				set backupnameNum 0
				while { [ 
					file exists "$backupnameStem$backupnameNum" 
				] } {
					incr backupnameNum
				}
				file rename "${saveTargetName}" "$backupnameStem$backupnameNum"
			}

			#actually do the saving 
			#( i.e. move the recording to the chosen filename ) 
			set moveStatus [
				::dr::utils::moveDemorecRecording \
					"${currentRecordingName}" \
					"${saveTargetName}"
			]

			puts "save: moveStatus=$moveStatus"

			if { "$moveStatus" eq 0 } {
				#successful save 
				#set new currentRecordingName
				set currentRecordingName ${saveTargetName}
				#change gui 
				::dr::gui::layoutWithSections::HideButtonAndLabel3 .t.top "discard"
				#::dr::gui::layoutWithSections::ButtonAndLabel3_SetButtonlabel .t.top "save" "Save As"
				::dr::gui::layoutWithSections::DeactivateButtonAndLabel3 .t.top "save"
				
				::dr::gui::layoutWithSections::ButtonAndLabel3_SetText .t.top "save" \
					"Save is disabled because you have already saved it to \"${saveTargetName}\"" 
				::dr::gui::layoutWithSections::ShowButtonAndLabel3 .t.top "close"
				::dr::gui::layoutWithSections::ActivateButtonAndLabel3 .t.top "export" 
				::dr::gui::layoutWithSections::ButtonAndLabel3_SetText .t.top "export" \
					"Press this button to export this video to some common file format such as AVI, FLV, MPEG 1/2/4, ..." 

				::dr::gui::layoutWithSections::ChangeHeadlineOfSection3 .t.top "mainFeatures" "Recording saved"
				wm title . "DemoRecorder Recording Saved"
			} else {
				#save failed 
				#-> error popup
				#TODO
				#change gui
				::dr::gui::layoutWithSections::ChangeHeadlineOfSection3 .t.top "mainFeatures" "Saving of Recording failed"
			}
		}
	}


::dr::gui::layoutWithSections::ButtonAndLabel3 .t.top "export" "Export" \
	"Export is disabled because the video is not saved yet. Therefore please save the video before you can export it to some common file format such as AVI, FLV, MPEG 1/2/4, ..." \
	{
		puts  "starting $env(DEMOREC_DIR)/bin/demorec-exporter"
		exec "$env(DEMOREC_DIR)/bin/demorec-exporter" "${currentRecordingName}" &
	}

::dr::gui::layoutWithSections::DeactivateButtonAndLabel3 .t.top "export"

::dr::gui::layoutWithSections::ButtonAndLabel3 .t.top "discard" "Discard" \
	"Press this button to permanently delete this video and close this window." \
	{
		#ask again
		set answer [tk_messageBox -message "Are you sure that you want to discard your recording?" -type yesno -icon question]
		if { "$answer" eq "yes" } {
			if { "${unsaved_recording_name}" ne "" } {
				exec "rm" "-rf" "${unsaved_recording_name}"
			}
			exit 0
		}
	}

::dr::gui::layoutWithSections::ButtonAndLabel3 .t.top "close" "Close" \
	"Close this window" \
	{
		exit 0
	}

::dr::gui::layoutWithSections::HideButtonAndLabel3 .t.top "close"

::dr::gui::layoutWithSections::EndSection3 .t.top "mainFeatures"

::dr::gui::layoutWithSections::EndSectionList3 .t "top"

