#tcl script

########################################################
# DemoRecorder exporter GUI
#
# Copyright (C) 2009 Christian Linhart <chris@DemoRecorder.com>
########################################################

package require drWidgetFileSelection 1.0
package require drUtils 1.0

#puts "start of script"

set inputFileTypes {
	{{Demorecorder Video} {no-extension}}
}

set export_renderers {
	{{demorec-to-flv} {.flv} {"FLV" "Flash Video, using lossless (high image quality) screenvideo-codec"} }
	{{demorec-to-flv-with-player} {.flv} { "For Web publishing using Flash" "File-combo of HTML, SWF and FLV.  Lossless, i.e. high image quality."} }
	{{demorec-to-avi} {.avi} {"AVI" "can be played back under Windows ( with Windows Media Player ) and Linux ( with gxine etc )"} }
	{{demorec-to-mpeg1} {.mpeg} {"MPEG1" ""} }
	{{demorec-to-mpeg2} {.mp2 .mpeg2} {"MPEG2" "can be used as input for DVD authoring software"} }
	{{demorec-to-mpeg4} {.mp4 .mpeg4} {"MPEG4" ""} }
	{{demorec-to-h264} {.h264 .mp4 .mpeg4} {"H264" ""} }
	{{demorec-to-ogg-theora} {.ogg} {"Ogg/Theora" "Open Source Video format, can be played back with gxine etc"} }
	{{demorec-to-youtube-avi} {.avi} {"For YouTube.com (low res)" "AVI-file with paramaters optimized for YouTube"} }
	{{demorec-to-youtube-hd-avi} {.avi} {"For YouTube.com (hd 720)" "AVI-file with paramaters optimized for YouTube"} }
	{{demorec-to-youtube-full-hd-avi} {.avi} {"For YouTube.com (full hd 1080p)" "AVI-file with paramaters optimized for YouTube"} }
	{{demorec-to-vob-ntsc} {.vob} {"VOB NTSC"  "For generating Video-DVDs for the US-American standard-resolution TV-system ( NTSC  )"} }
	{{demorec-to-vob-pal} {.vob} {"VOB PAL"  "For generating Video-DVDs for the European standard-resolution TV-system ( PAL  )"}}
	{{demorec-to-dv-ntsc} {.dv}  {"DV NTSC" "\"Digital Video\" high quality, low compression format, US-American standard-TV-resolution ( NTSC  )"} }
	{{demorec-to-dv-pal} {.dv} {"DV PAL" "\"Digital Video\" high quality, low compression format, European standard-TV-resolution ( PAL  )"}}
}

set default_export_renderer "demorec-to-avi"

set startdir $env(FILE_DIR)
set DEMOREC_DIR $env(DEMOREC_DIR)

set video ""
set resultName ""
set options ""

set filenameFieldSize 50
set optionsFieldSize 50


#compute list of exporters 
set exporterList {}
array set export_renderer_of_exporterListKey {}
array set fileExtensions_of_exporterListKey {}
array set fileType_of_exporterListKey {}
foreach exporter $export_renderers {
	set filetype [ lindex $exporter 2 0 ]
	if { [ lindex $exporter 2 1 ] == "" } {
		set listEntry  "$filetype"
	} else {
		 set listEntry "$filetype ( [ lindex $exporter 2 1 ] )"
	}
	set export_renderer [ lindex $exporter 0 0 ]
	set file_extensions [ lindex $exporter 1 ]

	lappend exporterList $listEntry
	if { $export_renderer == $default_export_renderer } {
		set selectedExporter $listEntry
		set selectedFileType $filetype
		set allowedFileExtensions $file_extensions
		set primaryOutputFileExtension [ lindex $file_extensions 0 ]
	}
	set export_renderer_of_exporterListKey($listEntry) $export_renderer
	set fileExtensions_of_exporterListKey($listEntry) $file_extensions
	set fileType_of_exporterListKey($listEntry) $filetype
}


#*** Aux functions ***
proc updateCommandline {varname args} {	
	upvar #0 $varname var
	computeCommandline
}

proc isAllowedFileExtension {fileExtension} {
	global allowedFileExtensions
	return [ expr [ lsearch -exact $allowedFileExtensions $fileExtension ] != -1 ]
}

proc updateResultName {} {
	global resultName
	global primaryOutputFileExtension
	if { "$resultName" != "" } {
		set fileExtension [ ::dr::utils::computeFileExtension $resultName ]
		if { $fileExtension == "" } {
			set resultName "$resultName$primaryOutputFileExtension"
		} else {
			#TODO: Check if it is allowed --> if no, replace with $primaryOutputFileExtension
			if { ! [ isAllowedFileExtension $fileExtension ] } {
				set resultName [ ::dr::utils::replaceFileExtension $resultName $primaryOutputFileExtension ]
			}
		}
	}
}


#*** GUI ***
frame .t

wm title . "DemoRecorder exporter"

#Headline
#label .t.hello -text "DemoRecorder exporter"
#grid configure .t.hello - - -in .t

#Inputvideo
button .t.choosevideoButton -text "Choose video" -command {
	global startdir
	global video
	if { $video == "" } { 
	    set video [ tk_getOpenFile -title "Choose DemoRecorder video" -initialdir $startdir -filetypes $inputFileTypes ]
	} else {
	    set video [ tk_getOpenFile -title "Choose DemoRecorder video" -initialfile $video -filetypes $inputFileTypes ]
	}
	#set video [ tk_chooseDirectory -title "Choose DemoRecorder video" -initialdir $initialdir -mustexist true ] 
	if { $resultName == "" } { 
		set resultName "$video" 
		updateResultName
	}
}

proc changedVideo {} {
	global resultName
	global video
	if { $resultName == "" } { 
		set resultName "$video" 
		updateResultName
	}
	computeCommandline
}


proc traceproc_changedVideo {varname args} {
	changedVideo
}

label .t.videonameLabel -text "DemoRecorder Video: " -state normal
entry .t.videoname -textvariable "video" 
trace add variable "video" write  "traceproc_changedVideo"

grid configure .t.videonameLabel .t.videoname .t.choosevideoButton -in .t
grid configure .t.videonameLabel -sticky e
grid configure .t.videoname -sticky ew
grid configure .t.choosevideoButton -sticky w


#grid configure .videonameLabel -row 0 -column 0  -in .t
#grid configure .videoname -row 0 -column 1 -in .t
#grid configure .choosevideoButton -row 0 -column 2 -in .t
#pack .videonameLabel -side left -in .choosevideo
#pack .videoname -side left -in .choosevideo 
#pack .choosevideoButton -side left -in .choosevideo

#dropdown menu for selecting the exporter ( autoselect based on filename )
label .t.selectExporterLabel -text "Output Video Format:"

#define list of exporters
eval tk_optionMenu .t.selectExporter selectedExporter $exporterList


proc changedExporter {} {
	global selectedExporter
	global fileExtensions_of_exporterListKey
	global fileType_of_exporterListKey
	global primaryOutputFileExtension
	global allowedFileExtensions
	global selectedFileType
	
	set file_extensions $fileExtensions_of_exporterListKey($selectedExporter)
	set allowedFileExtensions $file_extensions
	set selectedFileType $fileType_of_exporterListKey($selectedExporter)
	set primaryOutputFileExtension [ lindex $file_extensions 0 ]
	
	updateResultName
	computeCommandline
}

proc traceproc_changedExporter {varname args} {
	changedExporter
}

trace add variable "selectedExporter" write  "traceproc_changedExporter"

grid configure .t.selectExporterLabel .t.selectExporter - -in .t
grid configure .t.selectExporterLabel -sticky e
grid configure .t.selectExporter -sticky w

#Outputvideo
button .t.resultButton -text "Choose Name of Exported Video" -command {
	global selectedFileType
	global allowedFileExtensions
	#puts "choose output-videoname: allowedFileExtensions = $allowedFileExtensions"
	set filetypes {}
	lappend filetypes [ list [ list "$selectedFileType" ] "$allowedFileExtensions" ] 
	#puts "choose output-videoname: filetypes = $filetypes"
	set resultName [
		tk_getSaveFile -title "Choose Resultname" -filetypes $filetypes -initialfile $resultName -defaultextension $primaryOutputFileExtension
	]
	updateResultName
}
label .t.resultLabel -text "Exported Video: " -state normal
entry .t.resultName -textvariable "resultName" 
trace add variable "resultName" write  "updateCommandline"

grid configure .t.resultLabel .t.resultName .t.resultButton -in .t
grid configure .t.resultLabel -sticky e
grid configure .t.resultName -sticky ew
grid configure .t.resultButton -sticky w
 

#make textwidget readonly ( taken from http://wiki.tcl.tk/1152 )
   proc makeReadOnly {textwidget} {
     rename $textwidget $textwidget.internal
     proc $textwidget {args} [string map [list WIDGET $textwidget] {
       switch [lindex $args 0] {
         "insert" {}
         "delete" {}
         "default" { return [eval WIDGET.internal $args] }
       }
     }]
   }
#end of "make textwidget readonly"

#TODO: GUI for most important options:
#		Depending on the exporter selected, some options are grayed out/disabled or enabled
#		Each option has a checkbox "UseDefault" + the default will be displayed(?)

#extra Options

label .t.optionsLabel -text "Extra Options: " -state normal
entry .t.optionsValue -textvariable "extraOptions" 
trace add variable "extraOptions" write  "updateCommandline"

button .t.optionsHelpButton -text "Help" -command {
	set commandline [computeCommand]
	append commandline " --help"
	#puts "would call $commandline"

	set cmd "/bin/sh -c \"$commandline 2>&1\""
	set f [open "| $cmd" r]
	set helptext [ read $f ]
	close $f
	#puts "helptext = $helptext"	 
	
	destroy .optionsHelp
	set t [ toplevel .optionsHelp ]

	set frame [ frame $t.frame ] 

	wm title $t "Help for \"Extra Options\""

	text $frame.text -yscrollcommand {$frame.scrollbar set}
	scrollbar $frame.scrollbar -command {$frame.text yview}
	grid configure $frame.text $frame.scrollbar -in $frame
	grid configure $frame.text -sticky nsew 
	grid columnconfigure $frame 0 -weight 1
	grid configure $frame.scrollbar -sticky nsew
	grid columnconfigure $frame 1 -weight 0
	grid rowconfigure $frame 0 -weight 1
	
	button $frame.closeButton -text "Close" -command {
		#puts "close window"
		destroy $t
	}

	grid configure $frame.closeButton - -in $frame

	$frame.text insert end "$helptext"
	#-text "$helptext"
	
	#grid rowconfigure $frame 1 -weight 1 

	 
	#grid rowconfigure .t 1 -weight 0
	#grid columnconfigure $frame 1 -weight 100
	#grid columnconfigure $frame 1 -minsize 600

	pack $frame -expand 1 -anchor nw -fill both	

	makeReadOnly $frame.text
}


grid configure .t.optionsLabel .t.optionsValue .t.optionsHelpButton -in .t
grid configure .t.optionsLabel -sticky e
grid configure .t.optionsValue -sticky ew
grid configure .t.optionsHelpButton -sticky w


#Process Output of script

set f ""

proc handleOutput {} {
	global f
	global progress
	global gotFrame
    	if {[eof $f]} {
		close $f
		if { $gotFrame } {
			set progress "Finished." 
		} else {
			set progress "Failed."
		}
		.t.start configure -state active -bg lightgreen -activebackground green
		.t.stop configure -state disabled -bg grey -activebackground green
	} else {
		while { [ gets $f line ] >= 0 } {
			if { 
				[ string equal [ string range $line 0 5 ] "frame=" ] 
				||
				[ string equal [ string range $line 0 6 ] "frame =" ]
				||
				[ regexp {.*[0-9]+ frames:|^Importing} $line ] 
			} { 
				set progress $line	
				set gotFrame true
			} else {
				.t.console insert end $line
				.t.console insert end "\n"
			}
		} 
	}
}



proc computeOptions {} {
	global extraOptions
	set options $extraOptions
	return $options
}

proc computeCommand {} {
	global selectedExporter
	global export_renderer_of_exporterListKey
	global DEMOREC_DIR
	set export_renderer $export_renderer_of_exporterListKey($selectedExporter)
	return "$DEMOREC_DIR/bin/$export_renderer"
}

proc computeCommandline {} {
	global video
	global resultName
	global displayedCommandline

	set commandline "'" 
	append commandline [ computeCommand ]
	append commandline "' " 
	append commandline [ computeOptions ]
	append commandline " '$video' '$resultName'"

	set displayedCommandline $commandline
	
	return $commandline
}

#display of commandline to be executed
label .t.commandlineLabel -text "Commandline: "
label .t.commandlineText -textvariable displayedCommandline -wraplength 1024 -justify left

grid configure .t.commandlineLabel .t.commandlineText - -in .t
grid configure .t.commandlineLabel -sticky e
grid configure .t.commandlineText -sticky w



proc doExport {} {
	.t.start configure -state disabled -bg grey
	.t.stop configure -state active -bg "#FF8080" -activebackground red
	global progress
	global f
	global gotFrame
	set gotFrame false
	set progress "Starting..."
	.t.console delete 1.0 end 
	#set cmd "/home/chris/bin/export-example.sh"
	#TODO: aus Fileextension den export-filter berechnen 
	set commandline [computeCommandline]
	set cmd "/bin/sh -c \"$commandline 2>&1\""
	set f [open "| $cmd" r]
	fconfigure $f -blocking false
	fileevent $f readable  [ list handleOutput ]
}

proc stopExport {} {
	global f
	global progress
	exec kill [pid $f]
	close $f
	.t.start configure -state active -bg lightgreen -activebackground green
	.t.stop configure -state disabled -bg grey -activebackground green
	set progress "Aborted"
}

frame .t.startStop
button .t.start -text "Start Export" -command { doExport } -state active -bg lightgreen -activebackground green
button .t.stop -text "Stop Export" -command { stopExport } -state disabled -bg grey -activebackground green

pack .t.start -side left -in .t.startStop
pack .t.stop -side left -in .t.startStop

grid configure .t.startStop - - -in .t -sticky w 




#TODO: progress in another color

frame .t.progress
label .t.progressLabel -text "Progress: "
label .t.progressText -textvariable progress

pack .t.progressLabel -side left -in .t.progress
pack .t.progressText -side left -in .t.progress

grid configure .t.progress - - -in .t -sticky ew

#Toggle button for show/hide console
set consoleVisible 0

proc hideShowConsoleProc {} {
	global consoleVisible
	if { $consoleVisible == 0 } {
		grid configure .t.consoleLabel - - -in .t -sticky w
		grid configure .t.console - - -in .t -sticky ewns
		.t.hideShowConsole configure -text "Hide Console"
		set consoleVisible 1
	} else {
		grid remove .t.consoleLabel
		grid remove .t.console
		.t.hideShowConsole configure -text "Show Console"
		set consoleVisible 0
	}
	#puts "consoleVisible = $consoleVisible"
}

button .t.hideShowConsole -text "Show Console" -command { hideShowConsoleProc }

grid configure .t.hideShowConsole - - -in .t -sticky w

#TODO: Headline for console

#Console
label .t.consoleLabel -text "Console:"
text .t.console


grid rowconfigure .t 5 -weight 1 

 
#grid rowconfigure .t 1 -weight 0
grid columnconfigure .t 1 -weight 100
grid columnconfigure .t 1 -minsize 600
#grid rowconfigure .t 3 -weight 0


pack .t -expand 1 -anchor nw -fill both

#pack .hello .options .start .console .progress

#*** file-type stuff ***
set demorecVideoIcon [ image create photo -file "$DEMOREC_DIR/share/pixmaps/demorec-video-fileicon.ppm" ] 

proc isDemoRecVideo { name } {
	set numKeyFiles [ llength [ glob -nocomplain -directory "$name" "key.*" ] ]
	if { $numKeyFiles > 0 } {
		return true
	} else {
		return false
	}
}

proc computeDirProperties { dirname } {
	global demorecVideoIcon
	array set result {}
	if { [ isDemoRecVideo $dirname ] } {
		set result(icon) $demorecVideoIcon
		set result(treatAsFile) true 
	}
	return [ array get result ]
}

proc isRealDirectory { path } {
	array unset dirProperties
	set dirPropertiesAsList [ ::computeDirProperties "$path" ] 
	array set dirProperties $dirPropertiesAsList 
	if { [ info exists dirProperties(treatAsFile) ] && $dirProperties(treatAsFile) } {
	    #puts "isRealDirectory: isDirectory false for $path"
	    set isDirectory false
	} else {
	    #fallback
	    set isDirectory [file isdirectory $path]
	}
	return $isDirectory
}

#*** Init ***

changedExporter
computeCommandline

if { $argc >= 0 } {
	#puts "got videoname on the commandline"
	set video [lindex $argv 0]
}

