#tcl script

########################################################
# DemoRecorder preview playback with options
#
# Copyright (C) 2009 Christian Linhart <chris@DemoRecorder.com>
########################################################

package require drWidgetFileSelection 1.0
package require drUtils 1.0
package require drGuiTextDisplay 1.0

#puts "start of script"

#init vars
set startdir $env(FILE_DIR)
set DEMOREC_DIR $env(DEMOREC_DIR)

set video ""
set resultName ""
set options ""
set origOptions ""

set filenameFieldSize 50
set optionsFieldSize 50

#read and process commandline
set dialogMode "preview-playback"
for { set argIdx 0 } { $argIdx < $argc } { incr argIdx } {
	set arg [ lindex $argv $argIdx ]
	puts "arg=$arg"
	switch "$arg" {
 		"--demorec-gui-options-dialog-mode" {
			incr argIdx
			if { $argIdx < $argc } {
				set dialogMode [ lindex $argv $argIdx ] 
			}
		}
		default {
			puts "default: argIdx = $argIdx"
			append origOptions "'" "$arg" "' "
		}
	}
}

puts "dialogMode: $dialogMode" 
puts "origOptions: $origOptions"

switch $dialogMode {
	"nested-rec" {
		set startButtonText "Open Nested Desktop"
		set windowTitle "DemoRecorder Nested Desktop Recording"
	}
	"preview-playback" -
	default {
		set startButtonText "Preview Playback"
		set windowTitle "DemoRecorder Preview Playback"
	}
}
#end of processing commandline



#*** Aux functions ***
proc updateCommandline {varname args} {	
	upvar #0 $varname var
	computeCommandline
}

#*** GUI ***
frame .t

wm title . "$windowTitle" 

#Headline
#label .t.hello -text "DemoRecorder exporter"
#grid configure .t.hello - - -in .t


#make textwidget readonly ( taken from http://wiki.tcl.tk/1152 )
   proc makeReadOnly {textwidget} {
     rename $textwidget $textwidget.internal
     proc $textwidget {args} [string map [list WIDGET $textwidget] {
       switch [lindex $args 0] {
         "insert" {}
         "delete" {}
         "default" { return [eval WIDGET.internal $args] }
       }
     }]
   }
#end of "make textwidget readonly"

#extra Options

label .t.optionsLabel -text "Extra Options: " -state normal
entry .t.optionsValue -textvariable "extraOptions" 
trace add variable "extraOptions" write  "updateCommandline"

button .t.optionsHelpButton -text "Help" -command {
	set commandline [computeCommand]
	append commandline " --help"
	#puts "would call $commandline"

	set cmd "/bin/sh -c \"$commandline 2>&1\""
	set f [open "| $cmd" r]
	set helptext [ read $f ]
	close $f
	#puts "helptext = $helptext"	 
	
	::dr::gui::TextDisplay::ShowTextInPopup optionsHelp \
		"Help for \"Extra Options\"" \
		"$helptext"
}


grid configure .t.optionsLabel .t.optionsValue .t.optionsHelpButton -in .t
grid configure .t.optionsLabel -sticky e
grid configure .t.optionsValue -sticky ew
grid configure .t.optionsHelpButton -sticky w


proc computeOptions {} {
	global extraOptions
	set options $extraOptions
	return $options
}

proc computeCommand {} {
	global DEMOREC_DIR

	return "$DEMOREC_DIR/bin/demorec"
}

proc computeCommandline {} {
	global video
	global resultName
	global displayedCommandline
	global origOptions

	set commandline "'" 
	append commandline [ computeCommand ]
	append commandline "' "
	append commandline "$origOptions" 
	append commandline [ computeOptions ]

	set displayedCommandline $commandline
	
	return $commandline
}

#display of commandline to be executed
label .t.commandlineLabel -text "Commandline: "
label .t.commandlineText -textvariable displayedCommandline -wraplength 1024 -justify left

grid configure .t.commandlineLabel .t.commandlineText - -in .t
grid configure .t.commandlineLabel -sticky e
grid configure .t.commandlineText -sticky w



proc PreviewPlayback {} {
	set commandline [computeCommandline]
	exec "/bin/sh" "-c" "$commandline 2>&1" &
	exit 0
}

proc Close {} {
	puts "close"
	exit 0
}

frame .t.start
button .t.previewPlayback -text $startButtonText  -command { PreviewPlayback } -state normal -bg lightgreen -activebackground green
button .t.close -text "Close" -command { Close } -state normal -bg lightgreen -activebackground green

pack .t.previewPlayback -side left -in .t.start
pack .t.close -side left -in .t.start

grid configure .t.start - - -in .t -sticky w 






grid rowconfigure .t 5 -weight 1 

 
#grid rowconfigure .t 1 -weight 0
grid columnconfigure .t 1 -weight 100
grid columnconfigure .t 1 -minsize 600
#grid rowconfigure .t 3 -weight 0


pack .t -expand 1 -anchor nw -fill both

#pack .hello .options .start .console .progress


#*** Init ***


computeCommandline


