#!/bin/bash

#functions
function analyze_xsession() 
{
	local xsession_file="$1"
	if fgrep case $xsession_file | fgrep -e '$1' >/dev/null ; then
		#has a case statement which switches session types
		for session in \
			$( \
				fgrep -A 1000 "$(fgrep case $xsession_file | fgrep -e '$1')" $xsession_file \
					| grep -e ')$' \
					| sed -e 's/[ \t]//g' -e 's/)//g' -e 's/|/\n/g' \
					| grep -v '\*' \
			) ; \
		do
			echo $session
		done
	elif which wmlist >/dev/null 2>&1 ; then
		#doesn't have a case statement but has wmlist (like Suse)
		for i in $( wmlist ) ; do
			echo $i
		done
	fi
}

#get the sessions (in a subshell beause we sort and uniq the output
(
#gdm

	gdmConfigPath=/etc/X11/gdm
	if [ ! -f $gdmConfigPath/gdm.conf ] ; then
		for candidate in $( find /etc -name gdm -type d -print 2>/dev/null ) ; do
			if [ -f "$candidate/gdm.conf" ] ; then
				gdmConfigPath=$candidate
			fi
		done
	fi

	if [ -d $gdmConfigPath/Sessions ] ; then
		#1. Sessions dir (old gdm)
		for i in $( cd $gdmConfigPath/Sessions ; find . -type f ) ; do
			echo gdm `basename $i`
		done
	fi
	
	#2. Follow the session path gdm.conf:SessionDesktopDir with the new ini-style session files (new gdm)
	gdmconf=$gdmConfigPath/gdm.conf
	if [ -f $gdmconf ] ; then
		echo gdm failsafe
		if grep -e '^SessionDesktopDir=' $gdmconf >/dev/null ; then
			sessionDesktopPath=$( grep -e '^SessionDesktopDir=' $gdmconf | sed -e 's/^SessionDesktopDir=//' )
		else
			sessionDesktopPath="/etc/X11/sessions/:/etc/dm/Sessions/:/etc/X11/dm/Sessions/:/usr/share/gdm/BuiltInSessions:/usr/share/xsessions/"
		fi
		for pathDir in $( echo $sessionDesktopPath | sed -e 's/:/ /g' ) ; do
			if [ -d $pathDir ] ; then
				(
					cd $pathDir
					for desktopFile in $( ls *.desktop 2>/dev/null ) ; do
						desktopName=$( basename $desktopFile .desktop )
						#check whether the desktop is actually installed (use TryExec)
						active=false
						for tryExecFile in \
							$( grep -e '^TryExec=' $desktopFile | sed -e 's/^TryExec=//' | sed -e 's/;/\n/g' ) \
							$( grep -e '^Exec=' $desktopFile | sed -e 's/^Exec=//' | sed -e 's/;/\n/g' ) 
						do
							if [ -x $tryExecFile ] ; then
								active=true
							fi
						done
						if [ "$active" = "true" ] ; then
							echo gdm $desktopName
						fi
					done
				)
			fi
		done
	fi

#kdm
	kdmconfig=/etc/X11/xdm
	for candidate in $( 
		find /etc -name kdm -type d -print 2>/dev/null 
		find /opt/kde3 -name kdm -type d -print 2>/dev/null 
		find /opt/kde4 -name kdm -type d -print 2>/dev/null 
	) ; do
		if [  -x "$candidate/Xsession" -o -f  "$candidate/kdmrc" ]  ; then
			kdmconfig=$candidate
		fi
	done
	
	kxsession=$kdmconfig/Xsession
	if [ -x $kxsession ] ; then
		#Analyze the Xsession file 
		for i in $( analyze_xsession $kxsession ) ; do
			echo kdm $i
		done
	fi
	
	kdmrc=$kdmconfig/kdmrc
	if [ -f $kdmrc ] ; then
		echo kdm failsafe
		echo kdm default
		#1. SessionTypes entries
		for i in $( grep '^SessionTypes=' $kdmrc | sed -e 's/^SessionTypes=//' -e 's/,/\n/g' ) ; do
			echo kdm $i
		done
	
		#2. follow the session path kdmrc:SessionsDir
		if grep -e '^SessionsDir=' $kdmrc >/dev/null ; then
			kdmSessionDesktopPath=$( grep -e '^SessionsDir=' $kdmrc | sed -e 's/^SessionsDir=//' )
		else
			#default from kubuntu 6.10
			kdmSessionDesktopPath="/usr/share/xsessions,/var/lib/menu-xdg/xsessions,/usr/share/apps/kdm/sessions"
		fi
		for pathDir in $( echo $kdmSessionDesktopPath | sed -e 's/,/ /g' ) ; do
			if [ -d $pathDir ] ; then
				(
					cd $pathDir
					for desktopFile in $( ls *.desktop 2>/dev/null ) ; do
						desktopName=$( basename $desktopFile .desktop )
						#echo checking $desktopName  in $pathDir/$desktopFile
						#check whether the desktop is actually installed (use TryExec)
						active=false
						for tryExecFile in \
							$( grep -e '^TryExec=' $desktopFile | sed -e 's/^TryExec=//' | sed -e 's/;/\n/g' ) \
							$( grep -e '^Exec=' $desktopFile | sed -e 's/^Exec=//' | sed -e 's/;/\n/g' ) 
						do
							for  tryExecFQFile in $( which $tryExecFile ) ; do
								if [ -x $tryExecFQFile ] ; then
									active=true
								fi
							done
						done
						if [ "$active" = "true" ] ; then
							echo kdm $desktopName
						fi
					done
				)
			fi
		done
	fi

	
#xdm
	#Analyze the Xsession file 
	if [ -f /etc/X11/xdm/Xsession ] ; then
		for i in $( analyze_xsession /etc/X11/xdm/Xsession ) ; do
			echo xdm $i
		done
	fi

#dtlogin (CDE)
	if [ -f /usr/dt/bin/dtlogin ] ; then
		echo dtlogin default
	fi
) | sort -u

#mdkkdm
	#get the directory /etc/X11/wmsession.d
	if [ -d /etc/X11/wmsession.d ] ; then
		( 
			cd /etc/X11/wmsession.d
			for i in `ls * | sed -e 's/^[0-9]*//' `; do 
				echo mdkkdm $i
			done
		)
	fi
