#tcl script

########################################################
# DemoRecorder exporter GUI
#
# Copyright (C) 2009 Christian Linhart <chris@DemoRecorder.com>
########################################################

package require drWidgetFileSelection 1.0
package require drUtils 1.0
package require drGuiTextDisplay 1.0

#puts "start of script"

set outputFileTypes {
	{{Demorecorder Video} {.demorec}}
}


set startdir $env(FILE_DIR)
set DEMOREC_DIR $env(DEMOREC_DIR)

set video ""
set resultName ""
set options ""

set filenameFieldSize 50
set optionsFieldSize 50


#*** Aux functions ***
proc updateCommandline {varname args} {	
	upvar #0 $varname var
	computeCommandline
}

proc isAllowedFileExtension {fileExtension} {
	global allowedFileExtensions
	return [ expr [ lsearch -exact $allowedFileExtensions $fileExtension ] != -1 ]
}

proc updateResultName {} {
	global resultName
	global primaryOutputFileExtension
	if { "$resultName" != "" } {
		set fileExtension [ ::dr::utils::computeFileExtension $resultName ]
		if { $fileExtension == "" } {
			set resultName "$resultName$primaryOutputFileExtension"
		} else {
			#TODO: Check if it is allowed --> if no, replace with $primaryOutputFileExtension
			if { ! [ isAllowedFileExtension $fileExtension ] } {
				set resultName [ ::dr::utils::replaceFileExtension $resultName $primaryOutputFileExtension ]
			}
		}
	}
}


#*** GUI ***
frame .t

wm title . "DemoRecorder Record"

#Headline
#label .t.hello -text "DemoRecorder exporter"
#grid configure .t.hello - - -in .t

#Inputvideo
if { 0 == 1 } {
	button .t.choosevideoButton -text "Choose video" -command {
		global startdir
		global video
		set primaryOutputFileExtension ".demorec"
		set allowedFileExtensions { ".demorec" "" }
		if { $video == "" } { 
		    set video [ tk_getSaveFile -title "Choose DemoRecorder video" -initialdir $startdir -filetypes $outputFileTypes ]
		} else {
		    set video [ tk_getSaveFile -title "Choose DemoRecorder video" -initialfile $video -filetypes $outputFileTypes ]
		}
		#set video [ tk_chooseDirectory -title "Choose DemoRecorder video" -initialdir $initialdir -mustexist true ] 
		if { $resultName == "" } { 
			set resultName "$video" 
			updateResultName
		}
	}

	proc changedVideo {} {
		global resultName
		global video
		if { $resultName == "" } { 
			set resultName "$video" 
			updateResultName
		}
		computeCommandline
	}


	proc traceproc_changedVideo {varname args} {
		changedVideo
	}

	label .t.videonameLabel -text "Choose name of your video: " -state normal
	entry .t.videoname -textvariable "video" 
	trace add variable "video" write  "traceproc_changedVideo"

	grid configure .t.videonameLabel .t.videoname .t.choosevideoButton -in .t
	grid configure .t.videonameLabel -sticky e
	grid configure .t.videoname -sticky ew
	grid configure .t.choosevideoButton -sticky w 

}

#make textwidget readonly ( taken from http://wiki.tcl.tk/1152 )
   proc makeReadOnly {textwidget} {
     rename $textwidget $textwidget.internal
     proc $textwidget {args} [string map [list WIDGET $textwidget] {
       switch [lindex $args 0] {
         "insert" {}
         "delete" {}
         "default" { return [eval WIDGET.internal $args] }
       }
     }]
   }
#end of "make textwidget readonly"

#extra Options

label .t.optionsLabel -text "Extra Options: " -state normal
entry .t.optionsValue -textvariable "extraOptions" 
trace add variable "extraOptions" write  "updateCommandline"

button .t.optionsHelpButton -text "Help" -command {
	set commandline [computeCommand]
	append commandline " --help"
	#puts "would call $commandline"

	set cmd "/bin/sh -c \"$commandline 2>&1\""
	set f [open "| $cmd" r]
	set helptext [ read $f ]
	close $f
	#puts "helptext = $helptext"	 

	::dr::gui::TextDisplay::ShowTextInPopup optionsHelp \
		"Help for \"Extra Options\"" \
		"$helptext"	
}


grid configure .t.optionsLabel .t.optionsValue .t.optionsHelpButton -in .t
grid configure .t.optionsLabel -sticky e
grid configure .t.optionsValue -sticky ew
grid configure .t.optionsHelpButton -sticky w


proc computeOptions {} {
	global extraOptions
	set options $extraOptions
	return $options
}

proc computeCommand {} {
	global DEMOREC_DIR

	return "$DEMOREC_DIR/bin/demorec-opengl"
}

proc computeCommandline {} {
	global video
	global resultName
	global displayedCommandline

	set commandline "'" 
	append commandline [ computeCommand ]
	append commandline "' " 
	append commandline [ computeOptions ]
	#append commandline " --out '$video'"

	set displayedCommandline $commandline
	
	return $commandline
}

#display of commandline to be executed
label .t.commandlineLabel -text "Commandline: "
label .t.commandlineText -textvariable displayedCommandline -wraplength 1024 -justify left

grid configure .t.commandlineLabel .t.commandlineText - -in .t
grid configure .t.commandlineLabel -sticky e
grid configure .t.commandlineText -sticky w



proc ChooseAreaAndRecord {} {
	puts "choose area"
	set commandline [computeCommandline]
	exec "/bin/sh" "-c" "$commandline 2>&1" &
	exit 0
	#set cmd "/bin/sh -c \"$commandline 2>&1\""	
}

proc RecordFullscreen {} {
	puts "record fullscreen"
	set commandline [computeCommandline]
	append commandline " --fullscreen"
	exec "/bin/sh" "-c" "$commandline 2>&1" &
	exit 0
	#set cmd "/bin/sh -c \"$commandline --fullscreen 2>&1\""	
}

proc Close {} {
	puts "close"
	exit 0
}

frame .t.start
button .t.chooseAreaAndRec -text "Choose Area and Record" -command { ChooseAreaAndRecord } -state normal -bg lightgreen -activebackground green
button .t.recFullscreen -text "Record Fullscreen" -command { RecordFullscreen } -state normal -bg lightgreen -activebackground green
button .t.close -text "Close" -command { Close } -state normal -bg lightgreen -activebackground green

pack .t.chooseAreaAndRec -side left -in .t.start
pack .t.recFullscreen -side left -in .t.start
pack .t.close -side left -in .t.start

grid configure .t.start - - -in .t -sticky w 






grid rowconfigure .t 5 -weight 1 

 
#grid rowconfigure .t 1 -weight 0
grid columnconfigure .t 1 -weight 100
grid columnconfigure .t 1 -minsize 600
#grid rowconfigure .t 3 -weight 0


pack .t -expand 1 -anchor nw -fill both

#pack .hello .options .start .console .progress


#*** Init ***


computeCommandline

if { $argc >= 0 } {
	#puts "got videoname on the commandline"
	set video [lindex $argv 0]
}

