#!/bin/bash

#shell config
#when exec fails, this script keeps running
shopt -s execfail

#script
if [ -z "$DEMOREC_DIR" ] ; then
	DEMOREC_DIR=/opt/DemoRecorder
	export DEMOREC_DIR
fi
if env | grep -e '^ORIG_LD_LIBRARY_PATH=' >/dev/null ; then
	LD_LIBRARY_PATH="$ORIG_LD_LIBRARY_PATH"
	export LD_LIBRARY_PATH
fi

unset LD_PRELOAD

display=:20
displayManager=gdm
session=gnome
auto=false
gdmConfigPath=/etc/X11/gdm

tmpprefix=/tmp/start-linux-xsession.$$

#parse commandline
while [ "$1" != "" ] ; do
	case "$1" in
		--display)
			display=$2
			shift
			shift
		;;
		--display-manager)
			displayManager=$2
			shift
			shift
		;;
		--session)
			session=$2
			shift
			shift
		;;		
		--auto)
			auto=true
			shift
		;;	
		*)
			echo "start-linux-session.sh: Unrecognized arg \"$1\""
			shift
	esac
done	

if [ "$auto" = "true" ] ; then
	$DEMOREC_DIR/auxbin/guess-current-xsession.sh >$tmpprefix.guessedXsession
	read displayManager session <$tmpprefix.guessedXsession
	rm $tmpprefix.guessedXsession
fi

echo "displayManager=$displayManager, session=$session"
echo "old DISPLAY=\"$DISPLAY\", display=\"$display\""

#start the session
DISPLAY=$display
export DISPLAY

#this is necessary for that gnome-session does not complain that a session is already running
unset SESSION_MANAGER
export -n SESSION_MANAGER
		
#do displayManager and session-specific startup
case "$displayManager" in 
	gdm)

		gdmConfigFile=$gdmConfigPath/gdm.conf
		if [ ! -f $gdmConfigPath/gdm.conf ] ; then
			for candidate in $( find /etc -name gdm -type d -print 2>/dev/null ) /etc/gdm ; do
				if [ -f "$candidate/gdm.conf" ] ; then
					gdmConfigPath="$candidate"
					gdmConfigFile="$candidate/gdm.conf"
				elif [ -f "$candidate/custom.conf" ] ; then
					gdmConfigPath="$candidate"
					gdmConfigFile="$candidate/custom.conf"
				fi
			done
		fi

		if [ -f $gdmConfigPath/gdm.conf ] ; then
			echo "found gdmConfigFile=$gdmConfigFile"
		else
			echo "didn\'t find gdmConfigFile=$gdmConfigFile"
			echo "fallback: trying to find gdmConfigPath"
			for candidate in $( find /etc -name gdm -type d -print 2>/dev/null ) /etc/gdm ; do
				if [ -f "$candidate/PreSession" -o -f "$candidate/Xsession" ] ; then
					gdmConfigPath="$candidate"
				fi
			done
			gdmConfigFile="none"
			echo "did not find gdmConfigPath=$gdmConfigPath"
		fi

		#start presession -script (often sets the background of the screen)
		if [ -x $gdmConfigPath/PreSession/Default ] ; then
			echo "execute precession script"
			$gdmConfigPath/PreSession/Default
		fi
		
		#start Session 
			#Follow the session path gdm.conf:SessionDesktopDir with the new ini-style session files (new gdm)
			gdmconf="$gdmConfigFile"
			if [ -f $gdmconf -o -d $gdmConfigPath ] ; then
				#get path for Xsession
				if grep -e '^BaseXsession=' $gdmconf >/dev/null ; then
					gdm_Xsession=$( grep -e '^BaseXsession=' $gdmconf | sed -e 's/^BaseXsession=//' )
				elif [ -x "$gdmConfigPath/Xsession" ] ; then
					gdm_Xsession="$gdmConfigPath/Xsession"
				elif [ -x "/etc/X11/xdm/Xsession" ] ; then
					gdm_Xsession="/etc/X11/xdm/Xsession"
				else
					#unqualified name: lets try the PATH and hope for the best :-)
					gdm_Xsession=Xsession
				fi				
				#get path for session desktop files				
				if grep -e '^SessionDesktopDir=' $gdmconf >/dev/null ; then
					sessionDesktopPath=$( grep -e '^SessionDesktopDir=' $gdmconf | sed -e 's/^SessionDesktopDir=//' )
				else
					sessionDesktopPath="/etc/X11/sessions/:/etc/dm/Sessions/:/etc/X11/dm/Sessions/:/usr/share/gdm/BuiltInSessions:/usr/share/xsessions/"
				fi
				echo "sessionDesktopPath=$sessionDesktopPath"
				#look for the session desktop file and start the session
				for pathDir in $( echo $sessionDesktopPath | sed -e 's/:/ /g' ) ; do
					if [ -d $pathDir ] ; then
						echo "Checking1 $pathDir/$session.desktop"
						if [ ! -f $pathDir/$session.desktop ] ; then
							#could not find session config --> make case insensitive search
							newSession=$( ( cd $pathDir ; ls *.desktop | grep -iw $session.desktop | head --lines=1 ) )
							if [ -f $pathDir/$newsession ] ; then
								session=`basename $newSession .desktop`
							fi
						fi
						echo "Checking2  $pathDir/$session.desktop"
						if [ -f $pathDir/$session.desktop ] ; then
							echo "desktopfile=\"$pathDir/$session.desktop\""
							sessionCommand=`grep -e '^Exec=' $pathDir/$session.desktop | sed -e 's/^Exec=//'`
							echo "sessionCommand=$sessionCommand"
							if [ -n"$sessionCommand" ] ; then
								echo "executing sessionCommand $sessionCommand"
								if ( exec $sessionCommand ) >/dev/null 2>&1 ; then
									#OK
									exit 0 
								else
									#if we come to here, then exec failed --> lets try Xsession
									#echo try $gdm_Xsession $sessionCommand
									$gdm_Xsession $sessionCommand
									exit 0
								fi
							else
								echo "did not find a sessionCommand"
							fi
						fi
					fi
				done
				#fallback to old gdm session startup
				#repair session name if necessary
				if [ ! -f $gdmConfigPath/Sessions/$session ] ; then
					#could not find session script --> make case insensitive search
					newSession=$( ( cd $gdmConfigPath/Sessions ; ls * | grep -iw $session | head --lines=1 ) )
					if [ -f $gdmConfigPath/Sessions/$newSession ] ; then
						session=$newSession
					fi
				fi
				if [ -x $gdmConfigPath/Sessions/$session ] ; then
					$gdmConfigPath/Sessions/$session
				else
					#if we come here, it didn't work, so lets try Xsession
					echo try $gdm_Xsession $session
					$gdm_Xsession $session
				fi
			else
				echo "did neither find gdmconf=$gdmconf nor gdmConfigPath=$gdmConfigPath"
			fi

		;;
	
	xdm|mdkkdm)
		if [ -x /etc/X11/xdm/Xstartup ] ; then /etc/X11/xdm/Xstartup ; fi
		/etc/X11/xdm/Xsession $session
		if [ -x /etc/X11/xdm/Xreset ] ; then /etc/X11/xdm/Xreset ; fi
		;;
		
	kdm)
		#find kdm-config dir
		try_again=true
		while [ "$try_again" = "true" ] ; do
			echo "trying kdm $session"
			try_again=false
			kdmconfig=/etc/X11/xdm
			for candidate in $( 
				find /etc -name kdm -type d -print 2>/dev/null 
				find /opt/kde3 -name kdm -type d -print 2>/dev/null 
				find /opt/kde4 -name kdm -type d -print 2>/dev/null 
			) ; do
				if [  -x "$candidate/Xsession" -o -f  "$candidate/kdmrc" ]  ; then
					kdmconfig=$candidate
				fi
			done
			echo "kdmconfig=$kdmconfig"
			if [ -x $kdmconfig/Xstartup ] ; then 
				echo "calling $kdmconfig/Xstartup"
				$kdmconfig/Xstartup
			fi
	
				kdmrc=$kdmconfig/kdmrc
			
			if grep -e '^SessionsDir=' $kdmrc >/dev/null ; then
				kdmSessionDesktopPath=$( grep -e '^SessionsDir=' $gdmconf | sed -e 's/^SessionsDir=//' )
			else
							#default from kubuntu 6.10
							kdmSessionDesktopPath="/usr/share/xsessions,/var/lib/menu-xdg/xsessions,/usr/share/apps/kdm/sessions"
			fi
			echo "kdmSessionDesktopPath=$kdmSessionDesktopPath"
			#look for the session desktop file and start the session
			for pathDir in $( echo $kdmSessionDesktopPath | sed -e 's/,/ /g' ) ; do
				if [ -d $pathDir ] ; then
					echo "Checking1 $pathDir/$session.desktop"
					if [ ! -f $pathDir/$session.desktop ] ; then
						#could not find session config --> make case insensitive search
						newSession=$( ( cd $pathDir ; ls *.desktop | grep -iw $session.desktop | head --lines=1 ) )
						if [ -f $pathDir/$newsession ] ; then
							session=`basename $newSession .desktop`
						fi
					fi
					echo "Checking2  $pathDir/$session.desktop"
					if [ -f $pathDir/$session.desktop ] ; then
						echo "desktopfile=\"$pathDir/$session.desktop\""
						sessionCommand=`grep -e '^Exec=' $pathDir/$session.desktop | sed -e 's/^Exec=//'`
						if [ -n"$sessionCommand" ] ; then
							if ( exec $sessionCommand ) >/dev/null 2>&1 ; then
								#OK
								exit 0 
							elif [ -x "$kdmconfig/Xsession" ] ; then
								#if we come to here, then exec failed --> lets try Xsession
								#echo try $gdm_Xsession $sessionCommand
								$kdmconfig/Xsession $sessionCommand
								exit 0
							fi
						fi
					fi
				fi
			done
			#Fallback to old kdm session startup
			if [ -x "$kdmconfig/Xsession" ] ; then
				echo "calling $kdmconfig/Xsession $session"
				$kdmconfig/Xsession $session
				if [ -x $kdmconfig/Xreset ] ; then
					echo "calling $kdmconfig/Xreset"
					$kdmconfig/Xreset
				fi
			elif [ "$session" = "default" ] ; then
				session="kde"
				try_again=true
			elif [ "$session" = "failsafe" ] ; then
				session="kde"
				try_again=true
			else 
				echo "Don\'t know how to start kdm-session $session"
				exit 1
			fi
		done
		;;
		
	dtlogin)
		if [ "$session" = "default" ] ; then
			exec /usr/dt/bin/Xsession
		else
			exec /usr/dt/bin/Xsession -session "$session"
		fi
		;;
		
	xinit)
		if [ -x $session ] ; then
			exec $session
		else
			exec /bin/sh $session
		fi
		;;
		
	none)
		cd /etc
		Xsession=`find . -name Xsession -print | head  --lines=1`
		if [ -n "$Xsession" ] ; then
			exec $Xsession $session
		fi
		;;
		
	demorec)
		;;		

	script)
		exec $session
		;;

	*)
		echo "start-linux-session.sh: Unsupported displayManager $displayManager"
esac


