#!/bin/sh

DEMOREC_DIR=/opt/DemoRecorder
export LD_LIBRARY_PATH="$DEMOREC_DIR/lib:$DEMOREC_DIR/3rdparty/lib:$LD_LIBRARY_PATH"

#echo encoder.sh: $*

soundpipe=""
videopipe=""
framesPerSecond=25
output_target_opt=""
output_config_opts=""
ppmtoy4m_output_config_opts=""
x264_output_config_opts=""
audio_output_config_opts=""
nosound=""
tempdir="/var/tmp"

mult1000() {
	$DEMOREC_DIR/auxbin/MultFloat2Int "$1" 1000
}


while [ "$1" != "" ] ; do
	case "$1" in
		--video-pipe)
			videopipe=$2
			shift
			shift
		;;
		--sound-pipe)
			soundpipe=$2
			shift
			shift
		;;
		--input-frames-per-second)
			inputFramesPerSecond=$2
			shift
			shift
		;;
		--input-resolution)
			inputResolution=$2
			shift
			shift
		;;
		--input-sound-samples-per-second)
			inputSoundSamplesPerSecond=$2
			shift ; shift
		;;
		--input-sound-channels)
			inputSoundChannels=$2
			shift ; shift
		;;
		--input-bits-per-sound-sample)
			inputBitsPerSoundSample=$2
			shift ; shift
		;;
		--input-sound-signedness)
			inputSoundSignedness=$2
			shift ; shift
		;;
		--input-sound-endianness)
			inputSoundEndianness=$2
			shift ; shift
		;;
		--input-sound-scale)
			inputSoundScale=$2
			shift ; shift
		;;
		--input-sound-compression-method)
			inputSoundCompressionMethod=$2
			shift ; shift
		;;
		--output-bitrate)
			bps=`mult1000 $2`
		 	output_config_opts="$output_config_opts -b $bps -bt $bps"
			x264_output_config_opts="$x264_output_config_opts --bitrate $2"
			shift ; shift
		;;
		--output-max-bitrate)
			bps=`mult1000 $2`
		 	output_config_opts="$output_config_opts -maxrate $bps"
			x264_output_config_opts="$x264_output_config_opts --vbv-maxrate $2"
			shift ; shift
		;;
		--output-min-bitrate)
			bps=`mult1000 $2`
		 	output_config_opts="$output_config_opts -minrate $bps"
			shift ; shift
		;;
		--output-resolution)
			output_config_opts="$output_config_opts -s $2"
			shift
			shift
		;;
		--output-frames-per-second)
			output_config_opts="$output_config_opts -r $2"
			shift
			shift
		;;
		--output-sound-bitrate)
			bps=`mult1000 $2`
			audio_output_config_opts="$audio_output_config_opts -ab $bps"
			shift
			shift
		;;
		--output-file)
		 	outputFile="$2"
			shift ; shift
		;;
		--output-codec)
			output_target_opt="$output_target_opt -vcodec $2"
			shift ; shift
		;;
		--no-sound)
		 	nosound="nosound"
			shift
		;;
		--tempdir)
			tempdir="$2"
			shift ; shift
		;;
		#special opts for mpeg
		--special-mpeg-target)
			output_target_opt="$output_target_opt -target $2"
			shift ; shift
		;;
		--special-mpeg-bframes)
			output_target_opt="$output_target_opt -bf $2"
			shift ; shift
		;;
		*)
			echo "encoder.sh: Unrecognized arg \"$1\""
			shift
	esac
done

#echo "videopipe=$videopipe"
#echo "soundpipe=$soundpipe"
#echo "inputFramesPerSecond=$inputFramesPerSecond"
#echo "inputResolution=$inputResolution"
#echo "output_target_opt=$output_target_opt"
#echo "output_config_opts=$output_config_opts"
#echo "outputFile=$outputFile"

#( sed -e 's/^/videopipe: /' <$videopipe ) &
#( sed -e 's/^/soundpipe: /' <$soundpipe ) &
#wait

#cat $soundpipe >/tmp/sound.raw

fps1000=`mult1000 $inputFramesPerSecond` 
ppmtoy4m_output_config_opts="$ppmtoy4m_output_config_opts -F $fps1000:1000"
x264_output_config_opts="$x264_output_config_opts --fps $inputFramesPerSecond"


output_target_opt="$output_target_opt -vcodec libx264 -vpre slow -f h264"

if [  "$nosound" = "nosound" ] ; then
	#echo "encoder.sh without sound"
	y4m_pipe="$tempdir/demorec-y4m_pipe-$$.y4m"
	rm -f $y4m_pipe
	mkfifo $y4m_pipe

	( "$DEMOREC_DIR/3rdparty/bin/ppmtoy4m" \
		$ppmtoy4m_output_config_opts \
		$videopipe >$y4m_pipe \
	) 2>&1 | while read a ; do echo "demorecvideo->y4m: $a" ; done &

	"$DEMOREC_DIR/3rdparty/bin/x264" $x264_output_config_opts --threads auto --output "$outputFile" \
		$y4m_pipe \

	#2>&1 | while read a ; do echo "yuv4mpeg->x264: $a" ; done

	#"$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
	#	-y \
	#	-s $inputResolution -vcodec ppm -r $inputFramesPerSecond -f image2pipe -i $videopipe \
	#	-an \
	#	$output_target_opt $output_config_opts "$outputFile"
else
	#echo "encoder.sh with sound"
	if [ \
		"$inputBitsPerSoundSample" = "16" \
		-a \
		"$inputSoundSignedness" = "signed" \
	] ; then
		if [ "$inputSoundEndianness" = "little" ] ; then
			soundFormat="s16le"
		else
			soundFormat="s16be"
		fi
	elif [ \
		"$inputBitsPerSoundSample" = "8" \
		-a \
		"$inputSoundSignedness" = "signed" \
	] ; then
		soundFormat="s8"
	else
		#use sox or a similar program to convert
		soundFormat="s16le"
	fi 

	#yuv444p_pipe=/tmp/demorec-yuv444p_pipe-$$.yuv444p
	#rm -f $yuv444p_pipe
	#mkfifo $yuv444p_pipe

	y4m_pipe="$tempdir/demorec-y4m_pipe-$$.y4m"
	rm -f $y4m_pipe
	mkfifo $y4m_pipe

	h264_file="$tempdir/demorec-h264_file-$$.264"
	rm -f $h264_file
	#mkfifo $h264_file	

	faac_file="$tempdir/demorec-faac_file-$$.mp4"
	rm -f $faac_file
	#mkfifo $faac_file
	#touch $faac_file

	#"$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
	#	-y \
	#	-s $inputResolution -vcodec ppm -r $inputFramesPerSecond -f image2pipe -i $videopipe \
	#	-f image2pipe -pix_fmt yuv444p -vcodec rawvideo $yuv444p_pipe \
	#2>&1 | while read a ; do echo "demorecvideo->yuv444p: $a" ; done &

	#"$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
	#	-y \
	#	-s $inputResolution -vcodec rawvideo -pix_fmt yuv444p -r $inputFramesPerSecond -f image2pipe -i $yuv444p_pipe \
	#	-f yuv4mpegpipe $y4m_pipe \
	#2>&1 | while read a ; do echo "yuv444p->yuv4mpeg: $a" ; done &

	#"$DEMOREC_DIR/3rdparty/bin/ppmtoy4m"

	"$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
		-ar $inputSoundSamplesPerSecond -ac $inputSoundChannels  \
		-f $soundFormat -i $soundpipe \
		-acodec libfaac $audio_output_config_opts $faac_file \
	
	#2>&1 | while read a ; do echo "step1: demorecsound->faac: $a" ; done

	( "$DEMOREC_DIR/3rdparty/bin/ppmtoy4m" \
		$ppmtoy4m_output_config_opts \
		$videopipe >$y4m_pipe \
	) 2>&1 | while read a ; do echo "step2: demorecvideo->y4m: $a" ; done &

	"$DEMOREC_DIR/3rdparty/bin/x264" $x264_output_config_opts --threads auto --output $h264_file \
		$y4m_pipe \

	#2>&1 | while read a ; do echo "step2: yuv4mpeg->x264: $a" ; done

	echo step3: mergeav: start
	rm -f "$outputFile"
	"$DEMOREC_DIR/3rdparty/bin/MP4Box" \
		-fps $inputFramesPerSecond \
		-add $h264_file -add $faac_file \
		"$outputFile" \

	#2>&1 | while read a ; do echo "step3: mergeav: $a" ; done

	rm -f $y4m_pipe
	rm -f $h264_file
	rm -f $faac_file

	exit

	echo "$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
		-y \
		-s $inputResolution -vcodec ppm -r $inputFramesPerSecond -f image2pipe -i $videopipe \
		-ar $inputSoundSamplesPerSecond -ac $inputSoundChannels  \
		-f $soundFormat -i $soundpipe \
		-acodec libfaac $output_target_opt $output_config_opts "$outputFile"
	"$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
		-y \
		-s $inputResolution -vcodec ppm -r $inputFramesPerSecond -f image2pipe -i $videopipe \
		-ar $inputSoundSamplesPerSecond -ac $inputSoundChannels  \
		-f $soundFormat -i $soundpipe \
		-acodec libfaac $output_target_opt $output_config_opts "$outputFile"
fi
#endif
#( cat $soundpipe >/tmp/sound.raw ) &
#ffmpeg -an -s 720x576 -vcodec ppm -f image2pipe -i $videopipe -target dvd as.mp2

