#!/bin/sh

DEMOREC_DIR=/opt/DemoRecorder
export LD_LIBRARY_PATH="$DEMOREC_DIR/lib:$DEMOREC_DIR/3rdparty/lib:$LD_LIBRARY_PATH"

#echo encoder.sh: $*

soundpipe=""
videopipe=""
framesPerSecond=25
output_target_opt=""
output_config_opts=""
nosound=""

mult1000() {
	$DEMOREC_DIR/auxbin/MultFloat2Int "$1" 1000
}

while [ "$1" != "" ] ; do
	case "$1" in
		--video-pipe)
			videopipe=$2
			shift
			shift
		;;
		--sound-pipe)
			soundpipe=$2
			shift
			shift
		;;
		--input-frames-per-second)
			inputFramesPerSecond=$2
			shift
			shift
		;;
		--input-resolution)
			inputResolution=$2
			shift
			shift
		;;
		--input-sound-samples-per-second)
			inputSoundSamplesPerSecond=$2
			shift ; shift
		;;
		--input-sound-channels)
			inputSoundChannels=$2
			shift ; shift
		;;
		--input-bits-per-sound-sample)
			inputBitsPerSoundSample=$2
			shift ; shift
		;;
		--input-sound-signedness)
			inputSoundSignedness=$2
			shift ; shift
		;;
		--input-sound-endianness)
			inputSoundEndianness=$2
			shift ; shift
		;;
		--input-sound-scale)
			inputSoundScale=$2
			shift ; shift
		;;
		--input-sound-compression-method)
			inputSoundCompressionMethod=$2
			shift ; shift
		;;
		--output-bitrate)
			bps=`mult1000 $2`
		 	output_config_opts="$output_config_opts -b $bps"
			shift ; shift
		;;
		--output-max-bitrate)
			bps=`mult1000 $2`
		 	output_config_opts="$output_config_opts -maxrate $bps"
			shift ; shift
		;;
		--output-min-bitrate)
			bps=`mult1000 $2`
		 	output_config_opts="$output_config_opts -minrate $bps"
			shift ; shift
		;;
		--output-resolution)
			output_config_opts="$output_config_opts -s $2"
			shift
			shift
		;;
		--output-frames-per-second)
			output_config_opts="$output_config_opts -r $2"
			shift
			shift
		;;
		--output-sound-bitrate)
			bps=`mult1000 $2`
			output_config_opts="$output_config_opts -ab $bps"
			shift
			shift
		;;
		--output-file)
		 	outputFile="$2"
			shift ; shift
		;;
		--output-codec)
			output_target_opt="$output_target_opt -vcodec $2"
			shift ; shift
		;;
		--no-sound)
		 	nosound="nosound"
			shift
		;;
		--tempdir)
			tempdir="$2"
			shift ; shift
		;;
		#special opts for mpeg
		--special-mpeg-target)
			output_target_opt="$output_target_opt -target $2"
			shift ; shift
		;;
		--special-mpeg-bframes)
			output_target_opt="$output_target_opt -bf $2"
			shift ; shift
		;;
		*)
			echo "encoder.sh: Unrecognized arg \"$1\""
			shift
	esac
done

#echo "videopipe=$videopipe"
#echo "soundpipe=$soundpipe"
#echo "inputFramesPerSecond=$inputFramesPerSecond"
#echo "inputResolution=$inputResolution"
#echo "output_target_opt=$output_target_opt"
#echo "output_config_opts=$output_config_opts"
#echo "outputFile=$outputFile"

#( sed -e 's/^/videopipe: /' <$videopipe ) &
#( sed -e 's/^/soundpipe: /' <$soundpipe ) &
#wait

#cat $soundpipe >/tmp/sound.raw

output_target_opt="$output_target_opt -vcodec mpeg4 -f mp4"

if [  "$nosound" = "nosound" ] ; then
	#echo "encoder.sh without sound"
	"$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
		-y \
		-s $inputResolution -vcodec ppm -r $inputFramesPerSecond -f image2pipe -i $videopipe \
		-an \
		$output_target_opt $output_config_opts "$outputFile"
else
	#echo "encoder.sh with sound"
	if [ \
		"$inputBitsPerSoundSample" = "16" \
		-a \
		"$inputSoundSignedness" = "signed" \
	] ; then
		if [ "$inputSoundEndianness" = "little" ] ; then
			soundFormat="s16le"
		else
			soundFormat="s16be"
		fi
	elif [ \
		"$inputBitsPerSoundSample" = "8" \
		-a \
		"$inputSoundSignedness" = "signed" \
	] ; then
		soundFormat="s8"
	else
		#use sox or a similar program to convert
		soundFormat="s16le"
	fi 

	"$DEMOREC_DIR/3rdparty/bin/ffmpeg" \
		-y \
		-s $inputResolution -vcodec ppm -r $inputFramesPerSecond -f image2pipe -i $videopipe \
		-ar $inputSoundSamplesPerSecond -ac $inputSoundChannels -f $soundFormat -i $soundpipe \
		$output_target_opt $output_config_opts "$outputFile"
fi
#endif
#( cat $soundpipe >/tmp/sound.raw ) &
#ffmpeg -an -s 720x576 -vcodec ppm -f image2pipe -i $videopipe -target dvd as.mp2

