#!/bin/sh

DEMOREC_DIR=/opt/DemoRecorder
export LD_LIBRARY_PATH="$DEMOREC_DIR/lib:$DEMOREC_DIR/3rdparty/lib:$LD_LIBRARY_PATH"

#echo encoder.sh: $*

soundpipe=""
videopipe=""
framesPerSecond=25
output_target_opt=""
output_config_opts=""
sound_output_config_opts=""
nosound=""
output_codec="theora"
bframes="0"

while [ "$1" != "" ] ; do
	case "$1" in
		--video-pipe)
			videopipe=$2
			shift
			shift
		;;
		--sound-pipe)
			soundpipe=$2
			shift
			shift
		;;
		--input-frames-per-second)
			inputFramesPerSecond=$2
			shift
			shift
		;;
		--input-resolution)
			inputResolution=$2
			shift
			shift
		;;
		--input-sound-samples-per-second)
			inputSoundSamplesPerSecond=$2
			shift ; shift
		;;
		--input-sound-channels)
			inputSoundChannels=$2
			shift ; shift
		;;
		--input-bits-per-sound-sample)
			inputBitsPerSoundSample=$2
			shift ; shift
		;;
		--input-sound-signedness)
			inputSoundSignedness=$2
			shift ; shift
		;;
		--input-sound-endianness)
			inputSoundEndianness=$2
			shift ; shift
		;;
		--input-sound-scale)
			inputSoundScale=$2
			shift ; shift
		;;
		--input-sound-compression-method)
			inputSoundCompressionMethod=$2
			shift ; shift
		;;
		--output-bitrate)
		 	output_config_opts="$output_config_opts -V $2"
			shift ; shift
		;;
#		--output-max-bitrate)
#		 	output_config_opts="$output_config_opts -maxrate $2"
#			shift ; shift
#		;;
#		--output-min-bitrate)
#		 	output_config_opts="$output_config_opts -minrate $2"
#			shift ; shift
#		;;
		--output-resolution)
			#parse width and height
			width=` echo $2 | sed -e 's/x.*$//' `
			height=` echo $2 | sed -e 's/^.*x//' `
			output_config_opts="$output_config_opts --width $width --height $height"
			shift
			shift
		;;
		--output-frames-per-second)
			#demorec-to-ogg-theora does not support framerate conversion yet
			#output_config_opts="$output_config_opts -r $2"
			shift
			shift
		;;
		--output-sound-bitrate)
			sound_output_config_opts="$sound_output_config_opts -b $2"
			shift
			shift
		;;
		--output-file)
		 	outputFile="$2"
			shift ; shift
		;;
#		--output-codec)
#			output_codec="$2"
#			shift ; shift
#		;;
		--no-sound)
		 	nosound="nosound"
			shift
		;;
		--tempdir)
			tempdir="$2"
			shift ; shift
		;;
		#special opts for avi
		#special opts for mpeg contained in avi
		--special-mpeg-target)
			output_target_opt="$output_target_opt -target $2"
			shift ; shift
		;;
		--special-mpeg-bframes)
			bframes="$2"
			shift ; shift
		;;
		#default rule for catching errors
		*)
			echo "encoder.sh: Unrecognized arg \"$1\""
			shift
	esac
done

#echo "videopipe=$videopipe"
#echo "soundpipe=$soundpipe"
#echo "inputFramesPerSecond=$inputFramesPerSecond"
#echo "inputResolution=$inputResolution"
#echo "output_target_opt=$output_target_opt"
#echo "output_config_opts=$output_config_opts"
#echo "outputFile=$outputFile"

#( sed -e 's/^/videopipe: /' <$videopipe ) &
#( sed -e 's/^/soundpipe: /' <$soundpipe ) &
#wait

#output_target_opt="$output_target_opt -vcodec $output_codec"
#if [ "$bframes" != "0" ] ; then
#	output_target_opt="$output_target_opt -bf $bframes"
#fi
#output_target_opt="$output_target_opt -f ogg"

#cat $soundpipe >/tmp/sound.raw

if [  "$nosound" = "nosound" ] ; then
	#echo "encoder.sh without sound"
	ppmtoy4m -F "$inputFramesPerSecond:1" -S 444 < $videopipe \
	| ffmpeg2theora \
		-y \
		-s $inputResolution --inputfps $inputFramesPerSecond -f yuv4mpegpipe \
		$output_target_opt $output_config_opts -o "$outputFile" -
else
	#echo "encoder.sh with sound"
	if [ \
		"$inputBitsPerSoundSample" = "16" \
		-a \
		"$inputSoundSignedness" = "signed" \
	] ; then
		if [ "$inputSoundEndianness" = "little" ] ; then
			soundFormat="s16le"
		else
			soundFormat="s16be"
		fi
	elif [ \
		"$inputBitsPerSoundSample" = "8" \
		-a \
		"$inputSoundSignedness" = "signed" \
	] ; then
		soundFormat="s8"
	else
		#use sox or a similar program to convert
		soundFormat="s16le"
	fi 


	#statt ffmpeg verwenden wir ppmtoy4m aus den mjpegtools,
	#um den ppm-stream in einen yuvXXX stream umzuwandeln.

#	$DEMOREC_DIR/3rdparty/bin/ffmpeg2theora 

# yuv444p, yuv422p, yuv420p, yuv411p

		theorapipe="/tmp/drexporter-ogg-theora.$$.theorapipe"
		vorbispipe="/tmp/drexporter-ogg-theora.$$.vorbispipe"
		
		#remove old pipes just in case they are here
		rm -f $theorapipe $vorbispipe
		#create new pipes
		mknod $theorapipe p
		mknod $vorbispipe p

		#Encode video
		ppmtoy4m -F "$inputFramesPerSecond:1" -S 444 < $videopipe \
	| ffmpeg2theora \
		-y \
		-s $inputResolution --inputfps $inputFramesPerSecond -f yuv4mpegpipe \
		$output_target_opt $output_config_opts -o $theorapipe - &

		#Encode audio
		if [ "$inputSoundEndianness" = "little" ] ; then
			oggenc_endianness=0
		else
			oggenc_endianness=1
		fi

		"$DEMOREC_DIR/3rdparty/bin/oggenc" \
			-r -B $inputBitsPerSoundSample \
			--raw-endianness $oggenc_endianness \
			-R $inputSoundSamplesPerSecond \
			-C $inputSoundChannels \
			$sound_output_config_opts \
			-o $vorbispipe \
			$soundpipe &

		#Merge video and audio
		if which oggzmerge ; then
			oggzmerge -o "$outputFile" $theorapipe $vorbispipe
		elif which oggz-merge ; then
			oggz-merge -o "$outputFile" $theorapipe $vorbispipe
		else
			echo "oggz-merge not found. Please install it or ask chris@DemoRecorder.com for assistence."
			echo "if you ask chris@DemoRecorder.com, please indicate your Linux distribution and version."
		fi

		#finished: remove the pipes
		rm -f $theorapipe $vorbispipe

fi
#endif
#( cat $soundpipe >/tmp/sound.raw ) &
#ffmpeg -an -s 720x576 -vcodec ppm -f image2pipe -i $videopipe -target dvd as.mp2

