########################################################
# DemoRecorder tcl layout with sections
#
# Copyright (C) 2012 Christian Linhart <chris@DemoRecorder.com>
########################################################

package provide drGuiLayoutWithSections 1.0

namespace eval dr { namespace eval gui { namespace eval layoutWithSections {

set leftColumns ""
set rowCount 0
set explainTextWidth "300p"
set labelTextWidth "150p"

proc StartSectionList3 { parent name  } {
	global ::dr::gui::layoutWithSections::explainTextWidth
	global ::dr::gui::layoutWithSections::rowCount
	global ::dr::gui::layoutWithSections::labelTextWidth
	global ::dr::gui::layoutWithSections::leftColumns

	set leftColumns ""
	set rowCount 0
	set explainTextWidth "300p"
	set labelTextWidth "150p"

	frame ${parent}.${name}
}

proc getStyle { widget } {
	global ::dr::gui::layoutWithSections::rowCount
	
	if { ( $rowCount % 2 ) == 0 } { 
		return EvenStyle.$widget;
	} else { 
		return OddStyle.$widget;
	}	
}

proc getStyleOpt { widget } {
	return [ list -style [ getStyle $widget ] ]
}

proc StartSection { parent name headline margintext } {
	global ::dr::gui::layoutWithSections::leftColumns
	global ::dr::gui::layoutWithSections::rowCount
	
	label ${parent}.${name}Headline -text $headline \
		-state normal \
		-font "-size 10" \
		-justify left \
		-anchor w \
		-background #88FF88

	label ${parent}.${name}Margin -text $margintext \
		-state normal \
		-font "-size 10" \
		-justify left \
		-anchor w \
		-background #88FF88
		
	eval [ concat grid configure ${leftColumns} ${parent}.${name}Margin ${parent}.${name}Headline - -in ${parent} ]
	grid configure ${parent}.${name}Headline -sticky ewns -pady 1
	grid configure ${parent}.${name}Margin -sticky ewns -pady 1
	
	lappend leftColumns "^"	
}

proc EndSection { parent name } {
	global ::dr::gui::layoutWithSections::leftColumns

	puts stdout "before remove last: leftColumns = $leftColumns"

	#remove last element of leftColumns
	set leftColumns [ lreplace leftColumns end end ]
		
	puts stdout "after remove last: leftColumns = $leftColumns"

	label ${parent}.${name}Footer -text "" \
		-state normal \
		-font "-size 5" \
		-justify left \
		-anchor w \
		-background #BB8866

	eval [ concat grid configure ${leftColumns} ${parent}.${name}Footer - - - -in ${parent} ]
	grid configure ${parent}.${name}Footer -sticky ewns -pady 1
	
}

proc ButtonAndLabel { parent name buttonLabel text command } {
	global ::dr::gui::layoutWithSections::explainTextWidth
	global ::dr::gui::layoutWithSections::rowCount
	global ::dr::gui::layoutWithSections::leftColumns

	button ${parent}.${name}Button -text $buttonLabel -font "-size 20" -command $command  
	label ${parent}.${name}Label -text $text \
		-state normal \
		-font "-size 10" \
		-wraplength $explainTextWidth \
		-justify left \
		-background [ if { ( $rowCount % 2 ) == 0 } {
				::dr::utils::identity #9999DD 
			} else { 
				::dr::utils::identity #BBBBFF 
			} ] \
		-anchor w
	eval [ concat grid configure ${leftColumns} ${parent}.${name}Button ${parent}.${name}Label -in ${parent} ]
	grid configure ${parent}.${name}Button -sticky ew 
	grid configure ${parent}.${name}Label -sticky ewns -pady 1
	set rowCount [ expr $rowCount + 1 ]
}

proc ButtonAndLabel3 { parent name buttonLabel text command } {
	global ::dr::gui::layoutWithSections::explainTextWidth
	global ::dr::gui::layoutWithSections::rowCount
	global ::dr::gui::layoutWithSections::leftColumns

	#label ${parent}.${name}Label -text ""
	button ${parent}.${name}Button -text $buttonLabel -font "-size 12" -command $command  
	label ${parent}.${name}ExplainLabel -text $text \
		-state normal \
		-font "-size 10" \
		-wraplength $explainTextWidth \
		-justify left \
		-background [ if { ( $rowCount % 2 ) == 0 } { 
				::dr::utils::identity #9999DD 
			} else { 
				::dr::utils::identity #BBBBFF 
			} ] \
		-anchor w
	eval [ concat grid configure ${leftColumns} ${parent}.${name}Button - ${parent}.${name}ExplainLabel -in ${parent} ]
	grid configure ${parent}.${name}Button -sticky ew 
	grid configure ${parent}.${name}ExplainLabel -sticky ewns -pady 1
	set rowCount [ expr $rowCount + 1 ]
}

proc ButtonsAndLabel3 { parent buttonList text } {
	global ::dr::gui::layoutWithSections::explainTextWidth
	global ::dr::gui::layoutWithSections::rowCount
	global ::dr::gui::layoutWithSections::leftColumns




	#label ${parent}.${name}Label -text ""

	set buttonListName [ lindex $buttonList 0 0 ]

	set buttonListWidget [
		frame ${parent}.${buttonListName}ButtonList
	]

	#$buttonListWidget configure -bd 3 -relief raised

	puts "buttonListWidget=$buttonListWidget"

	set buttonWidgets ""

	foreach button $buttonList {
		set name [ lindex $button 0 ]
		set buttonLabel [ lindex $button 1 ]
		set command [ lindex $button 2 ]

		set button [
			button ${buttonListWidget}.${name}Button \
				-text $buttonLabel \
				-font "-size 12" \
				-command $command  
		]
		puts "button=$button"
		lappend buttonWidgets $button
	}


	eval [ concat grid configure ${buttonWidgets} -in ${buttonListWidget} ]

	foreach buttonWidget $buttonWidgets {
		grid configure $buttonWidget -sticky ew		
	}

	grid columnconfigure $buttonListWidget "all" -weight 1

	label ${parent}.${name}ExplainLabel -text $text \
		-state normal \
		-font "-size 10" \
		-wraplength $explainTextWidth \
		-justify left \
		-background [ if { ( $rowCount % 2 ) == 0 } { 
				::dr::utils::identity #9999DD 
			} else { 
				::dr::utils::identity #BBBBFF 
			} ] \
		-anchor w
	eval [ concat grid configure ${leftColumns} ${buttonListWidget} - ${parent}.${name}ExplainLabel -in ${parent} ]
	grid configure $buttonListWidget -sticky ew 
	grid configure ${parent}.${name}ExplainLabel -sticky ewns -pady 1
	set rowCount [ expr $rowCount + 1 ]


}


proc DeactivateButtonAndLabel3 { parent name } {
	${parent}.${name}Button configure -state disabled
}

proc ActivateButtonAndLabel3 { parent name } {
	${parent}.${name}Button configure -state active
}

proc ButtonAndLabel3_SetText { parent name text } {
	${parent}.${name}ExplainLabel configure -text "$text"
}

proc ButtonAndLabel3_SetButtonlabel { parent name buttonLabel } {
	${parent}.${name}Button configure -text "$buttonLabel"
}

proc HideButtonAndLabel3 { parent name } {
		grid remove ${parent}.${name}Button
		grid remove ${parent}.${name}ExplainLabel
}


proc ShowButtonAndLabel3 { parent name } {
		grid ${parent}.${name}Button
		grid ${parent}.${name}ExplainLabel
}

proc ComboBoxAndLabel3 { parent name label text var values } {
	global ::dr::gui::layoutWithSections::explainTextWidth
	global ::dr::gui::layoutWithSections::rowCount
	global ::dr::gui::layoutWithSections::labelTextWidth
	global $var
	global ::dr::gui::layoutWithSections::leftColumns

	label ${parent}.${name}Label -text $label \
		-state normal \
		-font "-size 10" \
		-wraplength $labelTextWidth \
		-justify left \
		-anchor w
	ttk::combobox ${parent}.${name}Combobox -values $values -font "-size 10" -textvariable $var  
	label ${parent}.${name}ExplainLabel -text $text \
		-state normal \
		-font "-size 10" \
		-wraplength $explainTextWidth \
		-justify left \
		-background [ if { ( $rowCount % 2 ) == 0 } { 
				::dr::utils::identity #9999DD 
			} else { 
				::dr::utils::identity #BBBBFF 
			} ] \
		-anchor w
	eval [ concat grid configure ${leftColumns} ${parent}.${name}Label ${parent}.${name}Combobox ${parent}.${name}ExplainLabel -in ${parent} ]
	grid configure ${parent}.${name}Combobox -sticky ew 
	grid configure ${parent}.${name}ExplainLabel -sticky ewns -pady 1
	grid configure ${parent}.${name}Label -sticky ewns -pady 1
	set rowCount [ expr $rowCount + 1 ]
}

proc RadioButtonSet3 { parent name label text var values  } {
	global ::dr::gui::layoutWithSections::explainTextWidth
	global ::dr::gui::layoutWithSections::rowCount
	global ::dr::gui::layoutWithSections::labelTextWidth
	global $var 
	global ::dr::gui::layoutWithSections::leftColumns

	label ${parent}.${name}Label -text $label \
		-state normal \
		-font "-size 10" \
		-wraplength $labelTextWidth \
		-justify left \
		-anchor w
		
	set first 1
	
	foreach option $values {
		lassign $option value active buttonLabel explainText
		
			
		frame ${parent}.${name}RadioButtonFrame${value} \
			-background [ if { ( $rowCount % 2 ) == 0 } { 
					::dr::utils::identity #999999 
				} else { 
					::dr::utils::identity #BBBBBB 
				} ]

		ttk::radiobutton ${parent}.${name}RadioButton${value} \
			-text $buttonLabel \
			-variable $var \
			-value $value \
			-state [ if { $active != 0 } { 
					::dr::utils::identity "normal" 
				} else { 
					::dr::utils::identity "disabled" 
				} ] \
			{*}[ getStyleOpt TRadiobutton ]
			
		
		#set style [ ${parent}.${name}RadioButton${value} cget -style ]
		set style [ winfo class ${parent}.${name}RadioButton${value} ]
		puts stdout "style=$style" 
			
		label ${parent}.${name}ExplainLabel${value} -text $explainText \
			-state normal \
			-font "-size 10" \
			-wraplength $explainTextWidth \
			-justify left \
			-background [ if { ( $rowCount % 2 ) == 0 } {
					::dr::utils::identity #9999DD 
				} else { 
					::dr::utils::identity #BBBBFF 
				} ] \
			-anchor w
			
		eval [ concat grid configure \
			${leftColumns} \
			[ \
				if { $first == 1 } {
					::dr::utils::identity "${parent}.${name}Label";
				} else {
					::dr::utils::identity "^";
				} \
			] \
			${parent}.${name}RadioButton${value} \
			${parent}.${name}ExplainLabel${value} -in ${parent} \
		]
			 
		grid configure ${parent}.${name}RadioButton${value} -sticky ewns -pady 1
		grid configure ${parent}.${name}ExplainLabel${value} -sticky ewns -pady 1
		if { $first == 1 } {
			grid configure ${parent}.${name}Label -sticky ewns -pady 1
		}
			
		if { $first == 1 } { set first 0 }
		
		set rowCount [ expr $rowCount + 1 ]
	}

}

proc StartSection3 { parent name headline margintext explainText  } {
	global ::dr::gui::layoutWithSections::leftColumns
	global ::dr::gui::layoutWithSections::explainTextWidth
	global ::dr::gui::layoutWithSections::rowCount
	
	label ${parent}.${name}Headline -text $headline \
		-state normal \
		-font "-size 10" \
		-justify left \
		-anchor w \
		-background #88FF88

	label ${parent}.${name}Margin -text $margintext \
		-state normal \
		-font "-size 10" \
		-justify left \
		-anchor w \
		-background #88FF88
		
	label ${parent}.${name}ExplainLabel -text $explainText \
		-state normal \
		-font "-size 10" \
		-wraplength $explainTextWidth \
		-justify left \
		-background #66DD66\
		-anchor w

	eval [ concat grid configure ${leftColumns} ${parent}.${name}Margin ${parent}.${name}Headline - ${parent}.${name}ExplainLabel -in ${parent} ]
	grid configure ${parent}.${name}Headline -sticky ewns -pady 1
	grid configure ${parent}.${name}Margin -sticky ewns -pady 1
	grid configure ${parent}.${name}ExplainLabel -sticky ewns -pady 1
	
	lappend leftColumns "^"	
}

proc ChangeHeadlineOfSection3 { parent name headline } {
	${parent}.${name}Headline configure -text $headline	
}

proc EndSection3 { parent name } {
	global ::dr::gui::layoutWithSections::leftColumns

	puts stdout "before remove last: leftColumns = $leftColumns"

	#remove last element of leftColumns
	set leftColumns [ lreplace leftColumns end end ]
		
	puts stdout "after remove last: leftColumns = $leftColumns"

	label ${parent}.${name}Footer -text "" \
		-state normal \
		-font "-size 5" \
		-justify left \
		-anchor w \
		-background #BB8866

	eval [ concat grid configure ${leftColumns} ${parent}.${name}Footer - - - -in ${parent} ]
	grid configure ${parent}.${name}Footer -sticky ewns -pady 1
	
}

proc EndSectionList3 { parent name  } {
	global ::dr::gui::layoutWithSections::leftColumns
	global ::dr::gui::layoutWithSections::explainTextWidth
	
	puts "EndSectionList3: leftColumns = ${leftColumns}"

	grid columnconfigure ${parent}.${name} \
		[ expr 1 + \
			[ llength ${leftColumns} ] \
		] \
		-weight 100

	grid columnconfigure ${parent}.${name} \
		[ expr 3 + \
			[ llength ${leftColumns} ] \
		] \
		-minsize ${explainTextWidth}

	pack ${parent} -expand 0 -anchor nw -fill both

	grid configure ${parent}.${name} -in ${parent}
	grid configure ${parent}.${name} -sticky ew 
}

#grid columnconfigure .t.top 1 -weight 100
#grid columnconfigure .t.top 3 \
#	-minsize ${::dr::gui::layoutWithSections::explainTextWidth}
#pack .t.top -expand 0 -anchor nw -fill both

#grid configure .t.top -in .t
#grid configure .t.top -sticky ew 


#close namespaces
} } }
