########################################################
# DemoRecorder tcl widgets
#
# Copyright (C) 2012 Christian Linhart <chris@DemoRecorder.com>
########################################################

package provide drUtils 1.0

#repair scaling on systems where X11 is misconfigured
if { [ tk scaling ] > 2 } {
	tk scaling 1.5
}

namespace eval dr { namespace eval utils {
	set DEMOREC_DIR $env(DEMOREC_DIR)

	proc computeFileExtension {filename} {
		return [ file extension $filename ]
	}

	proc stripFileExtension {filename} {
		set extension [ computeFileExtension $filename ]
		return [ string range $filename 0 end-[ string length $extension ] ]
	}

	proc replaceFileExtension {filename newExtension} {
		return [ stripFileExtension $filename ]$newExtension
	}

	proc OpenWebpage { url } {
		set browserList { 
			"/etc/alternatives/x-www-browser" 
			"/usr/bin/firefox" 
			"/usr/bin/konqueror"
		} 

		#Set original LD_LIBRARY_PATH so that the browser does not try to load DemoRecorder versions of shared libs
		if { [ info exists $::env(LD_LIBRARY_PATH) ] } {
			set savedLdLibraryPath $::env(LD_LIBRARY_PATH)
		} else {
			set savedLdLibraryPath ""
		}
		set ::env(LD_LIBRARY_PATH) $::env(ORIG_LD_LIBRARY_PATH)

		#start the browser
		foreach browser $browserList {
			if { [ file exists "$browser" ] } {
				puts "starting browser"
				puts [ exec "sh" "-c" "env" ]
				exec "$browser" "$url" &
				break
			}
		}

		#restore LD_LIBRARY_PATH
		set ::env(LD_LIBRARY_PATH) "$savedLdLibraryPath" 
	}

	proc identity { x } {
		return $x
	}


	## utils with recordings ##
	proc moveDemorecRecording { src dst } {
		global DEMOREC_DIR;
		puts "$DEMOREC_DIR/auxbin/DRSaveRecording -- ${src} ${dst}"
		if { [ 
			catch { 
				exec "$DEMOREC_DIR/auxbin/DRSaveRecording" "--" "${src}" "${dst}" 
			} catchResult 
		] } {
			if {[lindex $::errorCode 0] eq "CHILDSTATUS"} {
				set status [lindex $::errorCode 2]
			} else {
				# Some kind of unexpected failure
				return 1
			}
		} else {
			#catch didn't set an error
			#do some additional checks:
			#1. does the src still exist?
			#(if yes, this indicates that due to option "-n",
			#nothing was overwritten. Or some other error )
			if { [ file exists "$src" ] } {
				#src still exists -> something went wrong
				return 2
			}

			#everything ok
			return 0
		}


	}


	#*** file-type stuff ***
	set demorecVideoIcon [ image create photo -file "$DEMOREC_DIR/share/pixmaps/demorec-video-fileicon.ppm" ] 

	proc isDemoRecVideo { name } {
		if {[ file isdirectory "$name" ] && [file readable "$name"] && [file executable "$name"]} {
			set numKeyFiles [ llength [ glob -nocomplain -directory "$name" "key.*" ] ]
			if { $numKeyFiles > 0 } {
				return true
			} else {
				return false
			}
		} else {
			return false
		}
	}

	proc computeDirProperties { dirname } {
		global ::dr::utils::demorecVideoIcon
		array set result {}
		if { [ isDemoRecVideo $dirname ] } {
			set result(icon) $demorecVideoIcon
			set result(treatAsFile) true 
		}
		return [ array get result ]
	}

	proc isRealDirectory { path } {
		array unset dirProperties
		set dirPropertiesAsList [ ::dr::utils::computeDirProperties "$path" ] 
		array set dirProperties $dirPropertiesAsList 
		if { [ info exists dirProperties(treatAsFile) ] && $dirProperties(treatAsFile) } {
		    #puts "isRealDirectory: isDirectory false for $path"
		    set isDirectory false
		} else {
		    #fallback
		    set isDirectory [file isdirectory $path]
		}
		return $isDirectory
	}


} }

