########################################################
# DemoRecorder tcl widgets
#
# Copyright (C) 2009 Christian Linhart <chris@DemoRecorder.com>
# Some of the Code has been taken from tkfbox.tcl and then modified 
# The code from tkfbox.tcl is Copyright (c) 1994-1998 Sun Microsystems, Inc.
########################################################

package provide drWidgetFileSelection 1.0

package require drUtils 1.0

#*** Modify TK-framework ***

proc rewireFunction { funcname } {
	auto_load $funcname
	rename $funcname "${funcname}_TheOriginalOne"
}

rewireFunction ::tk::dialog::file::Update
rewireFunction ::tk::dialog::file::ResolveFile
rewireFunction ::tk::dialog::file::OkCmd
rewireFunction ::tk::dialog::file::ListInvoke
rewireFunction ::tk::dialog::file::ListBrowse

### the contents of this procedure are mostly copied from the tk framework version 8.4 ###
### file tkfbox.tcl ###
### modifications: add callback for custom icons ####

proc ::tk::dialog::file::Update {w} {
    # This proc may be called within an idle handler. Make sure that the
    # window has not been destroyed before this proc is called
    if {![winfo exists $w]} {
	return
    }
    set class [winfo class $w]
    if {($class ne "TkFDialog") && ($class ne "TkChooseDir")} {
	return
    }

    set dataName [winfo name $w]
    upvar ::tk::dialog::file::$dataName data
    variable ::tk::Priv
    global tk_library
    unset -nocomplain data(updateId)

    if {![info exists Priv(folderImage)]} {
	set Priv(folderImage) [image create photo -data {
R0lGODlhEAAMAKEAAAD//wAAAPD/gAAAACH5BAEAAAAALAAAAAAQAAwAAAIghINhyycvVFsB
QtmS3rjaH1Hg141WaT5ouprt2HHcUgAAOw==}]
	set Priv(fileImage)   [image create photo -data {
R0lGODlhDAAMAKEAALLA3AAAAP//8wAAACH5BAEAAAAALAAAAAAMAAwAAAIgRI4Ha+IfWHsO
rSASvJTGhnhcV3EJlo3kh53ltF5nAhQAOw==}]
    }
    set folder $Priv(folderImage)
    set file   $Priv(fileImage)

    set appPWD [pwd]
    if {[catch {
	cd $data(selectPath)
    }]} {
	# We cannot change directory to $data(selectPath). $data(selectPath)
	# should have been checked before ::tk::dialog::file::Update is called, so
	# we normally won't come to here. Anyways, give an error and abort
	# action.
	tk_messageBox -type ok -parent $w -icon warning -message \
	    [mc "Cannot change to the directory \"%1\$s\".\nPermission denied." $data(selectPath)]
	cd $appPWD
	return
    }

    # Turn on the busy cursor. BUG?? We haven't disabled X events, though,
    # so the user may still click and cause havoc ...
    #
    set entCursor [$data(ent) cget -cursor]
    set dlgCursor [$w         cget -cursor]
    $data(ent) configure -cursor watch
    $w         configure -cursor watch
    update idletasks

    ::tk::IconList_DeleteAll $data(icons)

    set showHidden $::tk::dialog::file::showHiddenVar

    # Make the dir list
    # Using -directory [pwd] is better in some VFS cases.
    set cmd [list glob -tails -directory [pwd] -type d -nocomplain *]
    if {$showHidden} { lappend cmd .* }
    set dirs [lsort -dictionary -unique [eval $cmd]]
    set dirList {}
    set parentDir [pwd]
    foreach d $dirs {
	if {$d eq "." || $d eq ".."} {
	    continue
	}
	lappend dirList $d
	#CLINHART 20090516-20090517
	#call a callback which determines the icon and whether the directory is to be treated as file
	array unset dirProperties
	set dirPropertiesAsList [ ::dr::utils::computeDirProperties "$parentDir/$d" ] 
	array set dirProperties $dirPropertiesAsList 
	#extract properties
	if { [ info exists dirProperties(icon) ] } { 
		set icon $dirProperties(icon)
	} else {
		#fallback
		set icon $folder
	}
	if { [ info exists dirProperties(treatAsFile) ] && $dirProperties(treatAsFile) } {
		#puts "treat $d as file"
		set treatAsFileDirs($d)  $dirPropertiesAsList 
	} else {
		#treat as directory --> add icon
    		::tk::IconList_Add $data(icons) $icon [regsub -all {( )} $d {\ }]
	}
	#CLINHART 20090516-20090517: End
    }
    #::tk::IconList_Add $data(icons) $folder $dirList

    if {$class eq "TkFDialog"} {
	# Make the file list if this is a File Dialog, selecting all
	# but 'd'irectory type files.
	#
	set cmd [list glob -tails -directory [pwd] \
		     -type {f b c l p s} -nocomplain]
	if {$data(filter) eq "*"} {
	    lappend cmd *
	    if {$showHidden} { lappend cmd .* }
	} else {
	    eval [list lappend cmd] $data(filter)
	}
	#CLINHART 20090516-20090517
	set unsortedFileList [eval $cmd]
	set treatAsFileDirsList [ array names treatAsFileDirs ]
	foreach treatAsFileDir [ array names treatAsFileDirs ] {
		lappend unsortedFileList $treatAsFileDir
	}
	set fileList [lsort -dictionary -unique $unsortedFileList]
	foreach fileName $fileList {
		if { [ info exists treatAsFileDirs($fileName) ] } {
			array unset dirProperties
			array set dirProperties $treatAsFileDirs($fileName)
			#extract properties
			if { [ info exists dirProperties(icon) ] } { 
				set icon $dirProperties(icon)
			} else {
				#fallback
				set icon $file
			}		
		} else {
			#fallback
			set icon $file
		}
		::tk::IconList_Add $data(icons) $icon [regsub -all {( )} $fileName {\ }]
	}
#	::tk::IconList_Add $data(icons) $file $fileList
    }

    ::tk::IconList_Arrange $data(icons)

    # Update the Directory: option menu
    #
    set list ""
    set dir ""
    foreach subdir [file split $data(selectPath)] {
	set dir [file join $dir $subdir]
	lappend list $dir
    }

    $data(dirMenu) delete 0 end
    set var [format %s(selectPath) ::tk::dialog::file::$dataName]
    foreach path $list {
	$data(dirMenu) add command -label $path -command [list set $var $path]
    }

    # Restore the PWD to the application's PWD
    #
    cd $appPWD

    if { $class eq "TkFDialog" } {
	# Restore the Open/Save Button if this is a File Dialog
	#
	if {$data(type) eq "open"} {
	    ::tk::SetAmpText $data(okBtn) [mc "&Open"]
	} else {
	    ::tk::SetAmpText $data(okBtn) [mc "&Save"]
	}
    }

    # turn off the busy cursor.
    #
    $data(ent) configure -cursor $entCursor
    $w         configure -cursor $dlgCursor


}

proc ::tk::dialog::file::ResolveFile {context text defaultext} {


    set appPWD [pwd]

    set path [::tk::dialog::file::JoinFile $context $text]

    # If the file has no extension, append the default.  Be careful not
    # to do this for directories, otherwise typing a dirname in the box
    # will give back "dirname.extension" instead of trying to change dir.
    if {![file isdirectory $path] && [file ext $path] eq ""} {
	set path "$path$defaultext"
    }


    if {[catch {file exists $path}]} {
	# This "if" block can be safely removed if the following code
	# stop generating errors.
	#
	#	file exists ~nonsuchuser
	#
	return [list ERROR $path ""]
    }

    if {[file exists $path]} {
	#CLINHART 20090516-20090517: use our "file isdirectory" function
	if { [::dr::utils::isRealDirectory $path] } {
	    if {[catch {cd $path}]} {
		return [list CHDIR $path ""]
	    }
	    set directory [pwd]
	    set file ""
	    set flag OK
	    cd $appPWD
	} else {
	    if {[catch {cd [file dirname $path]}]} {
		return [list CHDIR [file dirname $path] ""]
	    }
	    set directory [pwd]
	    set file [file tail $path]
	    set flag OK
	    cd $appPWD
	}
    } else {
	set dirname [file dirname $path]
	if {[file exists $dirname]} {
	    if {[catch {cd $dirname}]} {
		return [list CHDIR $dirname ""]
	    }
	    set directory [pwd]
	    set file [file tail $path]
	    if {[regexp {[*]|[?]} $file]} {
		set flag PATTERN
	    } else {
		set flag FILE
	    }
	    cd $appPWD
	} else {
	    set directory $dirname
	    set file [file tail $path]
	    set flag PATH
	}
    }

    return [list $flag $directory $file]
}

proc ::tk::dialog::file::OkCmd {w} {
    upvar ::tk::dialog::file::[winfo name $w] data

   #puts "okCmd"

    set filenames {}
    foreach item [::tk::IconList_CurSelection $data(icons)] {
	lappend filenames [::tk::IconList_Get $data(icons) $item]
    }

    if {([llength $filenames] && !$data(-multiple)) || \
	    ($data(-multiple) && ([llength $filenames] == 1))} {
	set filename [lindex $filenames 0]
	set file [::tk::dialog::file::JoinFile $data(selectPath) $filename]

	#CLINHART 20090516-20090517: use our "file isdirectory" function
	if { [::dr::utils::isRealDirectory $file] } {
	    ::tk::dialog::file::ListInvoke $w [list $filename]
	    return
	}
    }

    ::tk::dialog::file::ActivateEnt $w
}

proc ::tk::dialog::file::ListInvoke {w filenames} {
    upvar ::tk::dialog::file::[winfo name $w] data

   #puts "ListInvoke $filenames"

    if {[llength $filenames] == 0} {
	return
    }

    set file [::tk::dialog::file::JoinFile $data(selectPath) \
	    [lindex $filenames 0]]
    
    set class [winfo class $w]
      #CLINHART 20090516-20090517: use our "file isdirectory" function
    if {$class eq "TkChooseDir" || [::dr::utils::isRealDirectory $file]} {
	set appPWD [pwd]
	if {[catch {cd $file}]} {
	    tk_messageBox -type ok -parent $w -message \
	       "[mc "Cannot change to the directory \"%1\$s\".\nPermission denied." $file]"\
		-icon warning
	} else {
	    cd $appPWD
	    set data(selectPath) $file
	}
    } else {
	if {$data(-multiple)} {
	    set data(selectFile) $filenames
	} else {
	    set data(selectFile) $file
	}
	::tk::dialog::file::Done $w
    }
}

proc ::tk::dialog::file::ListBrowse {w} {
    upvar ::tk::dialog::file::[winfo name $w] data

    set text {}
    foreach item [::tk::IconList_CurSelection $data(icons)] {
	lappend text [::tk::IconList_Get $data(icons) $item]
    }
    if {[llength $text] == 0} {
	return
    }
    if { [llength $text] > 1 } {
	set newtext {}
	foreach file $text {
	    set fullfile [::tk::dialog::file::JoinFile $data(selectPath) $file]
	    #CLINHART 20090516-20090517: use our "file isdirectory" function
	    if { ![::dr::utils::isRealDirectory $fullfile] } {
		lappend newtext $file
	    }
	}
	set text $newtext
	set isDir 0
    } else {
	set text [lindex $text 0]
	set file [::tk::dialog::file::JoinFile $data(selectPath) $text]
	#CLINHART 20090516-20090517: use our "file isdirectory" function
	set isDir [::dr::utils::isRealDirectory $file]
    }
    if {!$isDir} {
	$data(ent) delete 0 end
	$data(ent) insert 0 $text

	if { [winfo class $w] eq "TkFDialog" } {
	    if {$data(type) eq "open"} {
		::tk::SetAmpText $data(okBtn) [mc "&Open"]
	    } else {
		::tk::SetAmpText $data(okBtn) [mc "&Save"]
	    }
	}
    } else {
	if { [winfo class $w] eq "TkFDialog" } {
	    ::tk::SetAmpText $data(okBtn) [mc "&Open"]
	}
    }
}
